@@ -29,7 +29,7 @@
 #include <glib-object.h>
 #include <numcosmo/build_cfg.h>
 #include <numcosmo/nc_hicosmo.h>
-#include <numcosmo/lss/nc_mass_function.h>
+#include <numcosmo/lss/nc_halo_mass_function.h>
 #include <numcosmo/lss/nc_halo_bias_func.h>
 #include <numcosmo/lss/nc_cluster_redshift.h>
 #include <numcosmo/lss/nc_cluster_mass.h>
@@ -47,6 +47,7 @@ G_BEGIN_DECLS
 
 typedef struct _NcClusterAbundanceClass NcClusterAbundanceClass;
 typedef struct _NcClusterAbundance NcClusterAbundance;
+
 typedef struct _NcClusterAbundanceDataP NcClusterAbundanceDataP;
 typedef struct _NcClusterAbundanceDataBinz NcClusterAbundanceDataBinZ;
 typedef struct _NcClusterAbundanceDataBinM NcClusterAbundanceDataBinM;
@@ -60,20 +61,24 @@ typedef gdouble (*NcClusterAbundanceIntPd2N) (NcClusterAbundance *cad, NcHICosmo
 #define nc_cluster_abundance_dNdlnM_val(cad,cp,lnM,zl,zu) (cad)->dNdlnM_val(cad,cp,lnM,zl,zu)
 #define nc_cluster_abundance_N_val(cad,cp,lnMl,lnMu,zl,zu) (cad)->N_val(cad,cp,lnMl,lnMu,zl,zu)
 
+struct _NcClusterAbundanceClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+};
+
 struct _NcClusterAbundance
 {
   /*< private >*/
   GObject parent_instance;
-  NcMassFunction *mfp;
+  NcHaloMassFunction *mfp;
   NcHaloBiasFunc *mbiasf;    /* new FIXME */
   NcClusterAbundanceN N;
   NcClusterAbundanceIntPd2N intp_d2N;
   gdouble norma, log_norma;
   gdouble lnMi, lnMf, zi, zf;
-  gdouble completeness_factor;
   gdouble lnM_epsilon, z_epsilon;
   gboolean optimize;
-  gsl_histogram2d *completeness;
   gsl_histogram2d *purity;
   gsl_histogram2d *sd_lnM;
   NcmSpline2d *dbdlnM;    /* To compute the mean bias. FIXME*/
@@ -87,30 +92,24 @@ struct _NcClusterAbundance
   NcmModelCtrl *ctrl_m;
 };
 
-struct _NcClusterAbundanceClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-};
-
 GType nc_cluster_abundance_get_type (void) G_GNUC_CONST;
 
-NcClusterAbundance *nc_cluster_abundance_new (NcMassFunction *mfp, NcHaloBiasFunc *mbiasf);
-NcClusterAbundance *nc_cluster_abundance_nodist_new (NcMassFunction *mfp, NcHaloBiasFunc *mbiasf);
+NcClusterAbundance *nc_cluster_abundance_new (NcHaloMassFunction *mfp, NcHaloBiasFunc *mbiasf);
+NcClusterAbundance *nc_cluster_abundance_nodist_new (NcHaloMassFunction *mfp, NcHaloBiasFunc *mbiasf);
 NcClusterAbundance *nc_cluster_abundance_ref (NcClusterAbundance *cad);
 
 void nc_cluster_abundance_free (NcClusterAbundance *cad);
 void nc_cluster_abundance_clear (NcClusterAbundance **cad);
 
-void nc_cluster_abundance_prepare (NcClusterAbundance *cad, NcHIReion *reion, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm);
-G_INLINE_FUNC void nc_cluster_abundance_prepare_if_needed (NcClusterAbundance *cad, NcHIReion *reion, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm);
+void nc_cluster_abundance_prepare (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm);
+G_INLINE_FUNC void nc_cluster_abundance_prepare_if_needed (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm);
 
-void nc_cluster_abundance_prepare_inv_dNdz (NcClusterAbundance *cad, NcHICosmo *cosmo);
-void nc_cluster_abundance_prepare_inv_dNdlnM_z (NcClusterAbundance *cad, NcHICosmo *cosmo, gdouble z);
+void nc_cluster_abundance_prepare_inv_dNdz (NcClusterAbundance *cad, NcHICosmo *cosmo, const gdouble lnMi);
+void nc_cluster_abundance_prepare_inv_dNdlnM_z (NcClusterAbundance *cad, NcHICosmo *cosmo, const gdouble lnMi, gdouble z);
 
-gdouble nc_cluster_abundance_z_p_lnm_p_d2n (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble *lnM_obs, gdouble *lnM_obs_params, gdouble *z_obs, gdouble *z_obs_params);
+gdouble nc_cluster_abundance_z_p_lnM_p_d2n (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble *lnM_obs, gdouble *lnM_obs_params, gdouble *z_obs, gdouble *z_obs_params);
 gdouble nc_cluster_abundance_z_p_d2n (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble lnM, gdouble *z_obs, gdouble *z_obs_params);
-gdouble nc_cluster_abundance_lnm_p_d2n (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble *lnM_obs, gdouble *lnM_obs_params, gdouble z);
+gdouble nc_cluster_abundance_lnM_p_d2n (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble *lnM_obs, gdouble *lnM_obs_params, gdouble z);
 gdouble nc_cluster_abundance_d2n (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm, gdouble lnM, gdouble z);
 
 gdouble nc_cluster_abundance_true_n (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm);
@@ -159,15 +158,14 @@ G_END_DECLS
 G_BEGIN_DECLS
 
 G_INLINE_FUNC void
-nc_cluster_abundance_prepare_if_needed (NcClusterAbundance *cad, NcHIReion *reion, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm)
+nc_cluster_abundance_prepare_if_needed (NcClusterAbundance *cad, NcHICosmo *cosmo, NcClusterRedshift *clusterz, NcClusterMass *clusterm)
 {
   const gboolean cosmo_up    = ncm_model_ctrl_update (cad->ctrl_cosmo, NCM_MODEL (cosmo));
-  const gboolean reion_up    = ncm_model_ctrl_update (cad->ctrl_reion, NCM_MODEL (reion));
   const gboolean clusterz_up = ncm_model_ctrl_model_update (cad->ctrl_z, NCM_MODEL (clusterz));
   const gboolean clusterm_up = ncm_model_ctrl_model_update (cad->ctrl_m, NCM_MODEL (clusterm));
 
-  if (cosmo_up || reion_up || clusterz_up || clusterm_up)
-    nc_cluster_abundance_prepare (cad, reion, cosmo, clusterz, clusterm);
+  if (cosmo_up || clusterz_up || clusterm_up)
+    nc_cluster_abundance_prepare (cad, cosmo, clusterz, clusterm);
 }
 
 G_END_DECLS
