@@ -31,11 +31,12 @@
 #include <numcosmo/build_cfg.h>
 #include <numcosmo/math/ncm_model.h>
 #include <numcosmo/nc_hicosmo.h>
-#include <numcosmo/lss/nc_mass_function.h>
+#include <numcosmo/lss/nc_halo_mass_function.h>
 #include <numcosmo/lss/nc_cluster_abundance.h>
 #include <numcosmo/lss/nc_cluster_mass.h>
 #include <numcosmo/lss/nc_cluster_mass_plcl.h>
 #include <gsl/gsl_rng.h>
+#include <gsl/gsl_multifit_nlin.h>
 
 G_BEGIN_DECLS
 
@@ -51,8 +52,8 @@ typedef struct _NcClusterPseudoCounts NcClusterPseudoCounts;
 
 /**
  * NcClusterPseudoCountsParams:
- * @NC_CLUSTER_PSEUDO_COUNTS_LNMCUT: lower mass cut-off
- * @NC_CLUSTER_PSEUDO_COUNTS_SD_MCUT: standard deviation of the mass cut-off
+ * @NC_CLUSTER_PSEUDO_COUNTS_LNMCUT: logarithm base e of the mass cut-off
+ * @NC_CLUSTER_PSEUDO_COUNTS_SD_MCUT: standard deviation of mass cut-off
  * @NC_CLUSTER_PSEUDO_COUNTS_ZMIN: minimum redshift
  * @NC_CLUSTER_PSEUDO_COUNTS_DELTAZ: redshift interval size
  * 
@@ -74,20 +75,22 @@ typedef enum _NcClusterPseudoCountsParams
 
 #define NC_CLUSTER_PSEUDO_COUNTS_DEFAULT_PARAMS_ABSTOL (0.0)
 
+struct _NcClusterPseudoCountsClass
+{
+  /*< private >*/
+  NcmModelClass parent_class;
+};
+
 struct _NcClusterPseudoCounts
 {
   /*< private >*/
   NcmModel parent_instance;
   guint nclusters;
+  const gsl_multifit_fdfsolver_type *T;
+  gsl_multifit_fdfsolver *s;
   gdouble *workz;
 };
 
-struct _NcClusterPseudoCountsClass
-{
-  /*< private >*/
-  NcmModelClass parent_class;
-};
-
 GType nc_cluster_pseudo_counts_get_type (void) G_GNUC_CONST;
 
 NCM_MSET_MODEL_DECLARE_ID (nc_cluster_pseudo_counts);
@@ -97,12 +100,14 @@ NcClusterPseudoCounts *nc_cluster_pseudo_counts_ref (NcClusterPseudoCounts *cpc)
 void nc_cluster_pseudo_counts_free (NcClusterPseudoCounts *cpc);
 void nc_cluster_pseudo_counts_clear (NcClusterPseudoCounts **cpc);
 
-gdouble nc_cluster_pseudo_counts_posterior_ndetone (NcClusterPseudoCounts *cpc, NcMassFunction *mfp, NcClusterMass *clusterm, gdouble z, gdouble Mpl, gdouble Mcl, gdouble sigma_pl, gdouble sigma_cl);
-gdouble nc_cluster_pseudo_counts_selection_function (NcClusterPseudoCounts *cpc, gdouble lnM);
+gdouble nc_cluster_pseudo_counts_posterior_ndetone (NcClusterPseudoCounts *cpc, NcHaloMassFunction *mfp, NcHICosmo *cosmo, NcClusterMass *clusterm, gdouble z, gdouble Mpl, gdouble Mcl, gdouble sigma_pl, gdouble sigma_cl);
+gdouble nc_cluster_pseudo_counts_selection_function (NcClusterPseudoCounts *cpc, gdouble lnM, gdouble z);
+gdouble nc_cluster_pseudo_counts_selection_function_lnMi (NcClusterPseudoCounts *cpc, NcHICosmo *cosmo);
 gdouble nc_cluster_pseudo_counts_ndet_no_z_integral (NcClusterPseudoCounts *cpc, NcHICosmo *cosmo, gdouble z);
-gdouble nc_cluster_pseudo_counts_ndet (NcClusterPseudoCounts *cpc, NcMassFunction *mfp, NcHICosmo *cosmo);
-gdouble nc_cluster_pseudo_counts_posterior_numerator (NcClusterPseudoCounts *cpc, NcMassFunction *mfp, NcClusterMass *clusterm, NcHICosmo *cosmo, const gdouble z, const gdouble *Mobs, const gdouble *Mobs_params);
-gdouble nc_cluster_pseudo_counts_posterior_numerator_plcl (NcClusterPseudoCounts *cpc, NcMassFunction *mfp, NcClusterMass *clusterm, NcHICosmo *cosmo, const gdouble z, const gdouble Mpl, const gdouble Mcl, const gdouble sigma_pl, const gdouble sigma_cl);
+gdouble nc_cluster_pseudo_counts_ndet (NcClusterPseudoCounts *cpc, NcHaloMassFunction *mfp, NcHICosmo *cosmo);
+gdouble nc_cluster_pseudo_counts_posterior_numerator (NcClusterPseudoCounts *cpc, NcHaloMassFunction *mfp, NcClusterMass *clusterm, NcHICosmo *cosmo, const gdouble z, const gdouble *Mobs, const gdouble *Mobs_params);
+gdouble nc_cluster_pseudo_counts_mf_lognormal_integral (NcClusterPseudoCounts *cpc, NcHaloMassFunction *mfp, NcClusterMass *clusterm, NcHICosmo *cosmo, const gdouble lnMsz, const gdouble lnMl, const gdouble z);
+gdouble nc_cluster_pseudo_counts_posterior_numerator_plcl (NcClusterPseudoCounts *cpc, NcHaloMassFunction *mfp, NcClusterMass *clusterm, NcHICosmo *cosmo, const gdouble z, const gdouble Mpl, const gdouble Mcl, const gdouble sigma_pl, const gdouble sigma_cl);
 
 G_END_DECLS
 
