@@ -24,10 +24,13 @@
  
 /**
  * SECTION:dividedifference
- * @title: Divided Difference
- * @short_description: Divided difference methods for function interpolation with derivatives
- *
+ * @title: NcmDividedDifference
+ * @short_description: Divided difference methods for function interpolation with derivatives.
+ * @stability: Unstable
+ * @include: numcosmo/math/dividedifference.h
+ * 
  * FIXME
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -150,7 +153,7 @@ _interp_dd_2_4 (const gdouble *vx, gdouble *dd)
 /************************************************************************************************************/
 
 void
-nc_interp_dd_init (const gdouble *vx, gdouble *dd, const gint np, const gint nf)
+ncm_interp_dd_init (const gdouble *vx, gdouble *dd, const gint np, const gint nf)
 {
   g_assert (nf >= 1);
   
@@ -165,7 +168,7 @@ nc_interp_dd_init (const gdouble *vx, gdouble *dd, const gint np, const gint nf)
 }
 
 void
-nc_interp_dd_init_2_4 (const gdouble *vx, gdouble *dd)
+ncm_interp_dd_init_2_4 (const gdouble *vx, gdouble *dd)
 {
   _interp_dd_2_4 (vx, dd);
 }
@@ -185,13 +188,13 @@ _interp_dd_eval (const gdouble *vx, const gdouble *dd, const gdouble x, const gi
 }
 
 gdouble 
-nc_interp_dd_eval (const gdouble *vx, const gdouble *dd, const gdouble x, const gint np, const gint nf)
+ncm_interp_dd_eval (const gdouble *vx, const gdouble *dd, const gdouble x, const gint np, const gint nf)
 {
   return _interp_dd_eval (vx, dd, x, np, nf, 0, 0, 0);
 }
 
 gdouble 
-nc_interp_dd_eval_2_4 (const gdouble *vx, const gdouble *dd, const gdouble x)
+ncm_interp_dd_eval_2_4 (const gdouble *vx, const gdouble *dd, const gdouble x)
 {
   gdouble dx0 = (x - vx[0]);
   gdouble dx1 = (x - vx[1]);
