@@ -22,8 +22,8 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _NC_MEMORY_POOL_H
-#define _NC_MEMORY_POOL_H
+#ifndef _NCM_MEMORY_POOL_H_
+#define _NCM_MEMORY_POOL_H_
 
 #include <glib.h>
 #include <glib-object.h>
@@ -32,7 +32,7 @@
 
 G_BEGIN_DECLS
 
-typedef gpointer (*NcmMemoryPoolAlloc) (void);
+typedef gpointer (*NcmMemoryPoolAlloc) (gpointer userdata);
 
 typedef struct _NcmMemoryPool NcmMemoryPool;
 
@@ -44,33 +44,37 @@ typedef struct _NcmMemoryPool NcmMemoryPool;
 struct _NcmMemoryPool
 {
   /*< private >*/
-  _NCM_MUTEX_TYPE append;
+  GMutex update;
+  GCond finish;
+  gint slices_in_use;
   GPtrArray *slices;
   NcmMemoryPoolAlloc alloc;
   GDestroyNotify free;
+  gpointer userdata;
 };
 
 typedef struct _NcmMemoryPoolSlice NcmMemoryPoolSlice;
 
 /**
  * NcmMemoryPoolSlice:
- * @p: Pointer to the actual slice
- * @lock: Mutex lock used by the pool
- * @mp: A back pointer to the pool
+ * @p: Pointer to the actual slice.
+ * @in_use: Boolean determining if the slice is in use.
+ * @mp: A back pointer to the pool.
  */
 struct _NcmMemoryPoolSlice
 {
   gpointer p;
-  _NCM_MUTEX_TYPE lock;
+  gboolean in_use;
   NcmMemoryPool *mp;
 };
 
-NcmMemoryPool *ncm_memory_pool_new (NcmMemoryPoolAlloc mp_alloc, GDestroyNotify mp_free);
+NcmMemoryPool *ncm_memory_pool_new (NcmMemoryPoolAlloc mp_alloc, gpointer userdata, GDestroyNotify mp_free);
 void ncm_memory_pool_free (NcmMemoryPool *mp, gboolean free_slices);
 void ncm_memory_pool_set_min_size (NcmMemoryPool *mp, gsize n);
+void ncm_memory_pool_add (NcmMemoryPool *mp, gpointer p);
 gpointer ncm_memory_pool_get (NcmMemoryPool *mp);
 void ncm_memory_pool_return (gpointer p);
 
 G_END_DECLS
 
-#endif /* _NC_MEMORY_POOL_H */
+#endif /* _NCM_MEMORY_POOL_H_ */
