@@ -103,7 +103,7 @@ _ncm_abc_set_property (GObject *object, guint prop_id, const GValue *value, GPar
       g_assert (abc->mcat == NULL);
       if (mset == NULL)
         g_error ("ncm_abc_new: mset (NcmMSet) cannot be NULL."); 
-      abc->mcat = ncm_mset_catalog_new (mset, 1, 1, TRUE, "NcmABC:Distance");
+      abc->mcat = ncm_mset_catalog_new (mset, 1, 1, TRUE, "NcmABC:Distance", "d", NULL);
       break;
     }
     case PROP_PRIOR:
@@ -595,7 +595,7 @@ _ncm_abc_update (NcmABC *abc, NcmMSet *mset, gdouble dist, gdouble weight)
   const guint step = (abc->n / part) == 0 ? 1 : (abc->n / part);
   const guint pindex = abc->cur_sample_id % abc->nparticles;
 
-  ncm_mset_catalog_add_from_mset (abc->mcat, mset, dist, weight);
+  ncm_mset_catalog_add_from_mset (abc->mcat, mset, dist, weight, NULL);
   ncm_timer_task_increment (abc->nt);
   g_array_index (abc->weights, gdouble, pindex) = weight;
   g_array_index (abc->dists,   gdouble, pindex) = dist;
@@ -638,13 +638,6 @@ _ncm_abc_update (NcmABC *abc, NcmMSet *mset, gdouble dist, gdouble weight)
       ncm_cfg_logfile_flush_now ();
       break;      
   }
-
-  if ((abc->mcat->fmode != NCM_ABC_MIN_FLUSH_INTERVAL) &&
-      (ncm_timer_task_mean_time (abc->nt) < NCM_ABC_MIN_FLUSH_INTERVAL))
-  {
-    ncm_mset_catalog_set_flush_mode (abc->mcat, NCM_MSET_CATALOG_FLUSH_TIMED);
-    ncm_mset_catalog_set_flush_interval (abc->mcat, NCM_ABC_MIN_FLUSH_INTERVAL);
-  }
 }
 
 static void ncm_abc_intern_skip (NcmABC *abc, guint n);
@@ -705,7 +698,10 @@ ncm_abc_start_run (NcmABC *abc)
     abc->thetastar = ncm_vector_new (fparam_len);
     abc->covar     = ncm_matrix_new (fparam_len, fparam_len);
   }
-  
+
+  ncm_mset_catalog_set_sync_mode (abc->mcat, NCM_MSET_CATALOG_SYNC_TIMED);
+  ncm_mset_catalog_set_sync_interval (abc->mcat, NCM_ABC_MIN_SYNC_INTERVAL);
+
   ncm_mset_catalog_sync (abc->mcat, TRUE);
   if (abc->mcat->cur_id > abc->cur_sample_id)
   {
@@ -934,11 +930,11 @@ typedef struct _NcmABCThread
 static gpointer
 _ncm_abc_dup_thread (gpointer userdata)
 {
-  _NCM_STATIC_MUTEX_DECL (dup_thread);
+  G_LOCK_DEFINE_STATIC (dup_thread);
   NcmABC *abc = NCM_ABC (userdata);
   NcmABCThread *abct = g_new (NcmABCThread, 1);
 
-  _NCM_MUTEX_LOCK (&dup_thread);
+  G_LOCK (dup_thread);
   {
     abct->mset      = ncm_mset_dup (abc->mcat->mset, abc->ser);
     abct->dset      = ncm_dataset_dup (abc->dset, abc->ser);
@@ -949,7 +945,7 @@ _ncm_abc_dup_thread (gpointer userdata)
 
     ncm_serialize_clear_instances (abc->ser);
 
-    _NCM_MUTEX_UNLOCK (&dup_thread);
+    G_UNLOCK (dup_thread);
     return abct;
   }
 }
@@ -969,7 +965,7 @@ _ncm_abc_free_thread (gpointer data)
 static void 
 _ncm_abc_thread_eval (glong i, glong f, gpointer data)
 {
-  _NCM_STATIC_MUTEX_DECL (update_lock);
+  G_LOCK_DEFINE_STATIC (update_lock);
   NcmABC *abc = NCM_ABC (data);
   NcmABCThread **abct_ptr = ncm_memory_pool_get (abc->mp);
   NcmABCThread *abct = *abct_ptr;
@@ -986,17 +982,17 @@ _ncm_abc_thread_eval (glong i, glong f, gpointer data)
     dist = ncm_abc_mock_distance (abc, abct->dset, abct->thetastar, abct->thetastar, abct->rng);
     prob = ncm_abc_distance_prob (abc, dist);
 
-    _NCM_MUTEX_LOCK (&update_lock);
+    G_LOCK (update_lock);
     abc->ntotal++;
-    _NCM_MUTEX_UNLOCK (&update_lock);
+    G_UNLOCK (update_lock);
     if (prob == 1.0 || (prob != 0.0 && gsl_rng_uniform (abct->rng->r) < prob))
     {
-      _NCM_MUTEX_LOCK (&update_lock);
+      G_LOCK (update_lock);
       abc->cur_sample_id++;
       abc->naccepted++;
       _ncm_abc_update (abc, abct->mset, dist, 1.0);
       j++;
-      _NCM_MUTEX_UNLOCK (&update_lock);
+      G_UNLOCK (update_lock);
     }
   }
 
@@ -1335,7 +1331,7 @@ _ncm_abc_update_single (NcmABC *abc)
 static void 
 _ncm_abc_thread_update_eval (glong i, glong f, gpointer data)
 {
-  _NCM_STATIC_MUTEX_DECL (update_lock);
+  G_LOCK_DEFINE_STATIC (update_lock);
   NcmABC *abc = NCM_ABC (data);
   NcmABCThread **abct_ptr = ncm_memory_pool_get (abc->mp);
   NcmABCThread *abct = *abct_ptr;
@@ -1357,18 +1353,18 @@ _ncm_abc_thread_update_eval (glong i, glong f, gpointer data)
     dist = ncm_abc_mock_distance (abc, abct->dset, theta, abct->thetastar, abct->rng);
     prob = ncm_abc_distance_prob (abc, dist);
 /*
-    _NCM_MUTEX_LOCK (&update_lock);
+    G_LOCK (update_lock);
     printf ("# choice %zu\n", np);
     ncm_vector_log_vals (theta,           "# v_choice = ", "% 20.15g");
     ncm_vector_log_vals (abct->thetastar, "# v_tkern  = ", "% 20.15g");
     printf ("# dist % 20.15g prob % 20.15g\n", dist, prob);
-    _NCM_MUTEX_UNLOCK (&update_lock);
+    G_UNLOCK (update_lock);
 */
     ncm_vector_free (theta);
     
-    _NCM_MUTEX_LOCK (&update_lock);
+    G_LOCK (update_lock);
     abc->ntotal++;
-    _NCM_MUTEX_UNLOCK (&update_lock);
+    G_UNLOCK (update_lock);
     
     if (prob == 1.0 || (prob != 0.0 && gsl_rng_uniform (abct->rng->r) < prob))
     {
@@ -1384,12 +1380,12 @@ _ncm_abc_thread_update_eval (glong i, glong f, gpointer data)
       }
       new_weight = new_weight / denom; 
 
-      _NCM_MUTEX_LOCK (&update_lock);
+      G_LOCK (update_lock);
       abc->cur_sample_id++;
       abc->naccepted++;
       _ncm_abc_update (abc, abct->mset, dist, new_weight);
       j++;
-      _NCM_MUTEX_UNLOCK (&update_lock);
+      G_UNLOCK (update_lock);
     }
   }
 
