@@ -38,20 +38,24 @@
 
 #include "math/ncm_cfg.h"
 #include "math/ncm_rng.h"
+#include "math/ncm_vector.h"
 #include "math/ncm_spline_gsl.h"
 #include "math/ncm_spline_cubic.h"
 #include "math/ncm_spline_cubic_notaknot.h"
 #include "math/ncm_spline2d_bicubic.h"
 #include "math/ncm_spline2d_gsl.h"
 #include "math/ncm_spline2d_spline.h"
+#include "math/ncm_powspec.h"
+#include "math/ncm_powspec_filter.h"
 #include "math/ncm_model.h"
 #include "math/ncm_model_ctrl.h"
 #include "math/ncm_model_builder.h"
 #include "math/ncm_reparam_linear.h"
+#include "math/ncm_data.h"
 #include "math/ncm_stats_vec.h"
+#include "math/ncm_fit_esmcmc_walker_stretch.h"
 #include "nc_hicosmo.h"
 #include "nc_cbe_precision.h"
-#include "perturbations/nc_hipert_iadiab.h"
 #include "model/nc_hicosmo_qconst.h"
 #include "model/nc_hicosmo_qlinear.h"
 #include "model/nc_hicosmo_qspline.h"
@@ -62,8 +66,11 @@
 #include "model/nc_hicosmo_de_qe.h"
 #include "model/nc_hicosmo_qgrw.h"
 #include "model/nc_hicosmo_de_reparam_ok.h"
+#include "model/nc_hicosmo_de_reparam_cmb.h"
 #include "model/nc_hiprim_power_law.h"
 #include "model/nc_hiprim_atan.h"
+#include "model/nc_hiprim_expc.h"
+#include "model/nc_hiprim_bpl.h"
 #include "lss/nc_window_tophat.h"
 #include "lss/nc_window_gaussian.h"
 #include "lss/nc_growth_func.h"
@@ -72,7 +79,6 @@
 #include "lss/nc_transfer_func_eh.h"
 #include "lss/nc_transfer_func_camb.h"
 #include "lss/nc_transfer_func_pert.h"
-#include "lss/nc_matter_var.h"
 #include "lss/nc_density_profile.h"
 #include "lss/nc_density_profile_nfw.h"
 #include "lss/nc_multiplicity_func.h"
@@ -84,7 +90,7 @@
 #include "lss/nc_multiplicity_func_tinker_mean.h"
 #include "lss/nc_multiplicity_func_tinker_crit.h"
 #include "lss/nc_multiplicity_func_tinker_mean_normalized.h"
-#include "lss/nc_mass_function.h"
+#include "lss/nc_halo_mass_function.h"
 #include "lss/nc_galaxy_acf.h"
 #include "lss/nc_cluster_mass.h"
 #include "lss/nc_cluster_mass_nodist.h"
@@ -110,9 +116,15 @@
 #include "nc_recomb_seager.h"
 #include "nc_hireion.h"
 #include "nc_hireion_camb.h"
+#include "nc_powspec_ml.h"
+#include "nc_powspec_ml_transfer.h"
+#include "nc_powspec_ml_cbe.h"
+#include "nc_powspec_mnl.h"
+#include "nc_powspec_mnl_halofit.h"
 #include "nc_snia_dist_cov.h"
 #include "nc_planck_fi.h"
 #include "nc_planck_fi_cor_tt.h"
+#include "nc_planck_fi_cor_ttteee.h"
 #include "data/nc_data_bao_a.h"
 #include "data/nc_data_bao_dv.h"
 #include "data/nc_data_bao_dvdv.h"
@@ -210,6 +222,11 @@ void clencurt_gen (int M);
 #include <omp.h>
 #endif /* HAVE_OPENMP */
 
+void _nc_hicosmo_register_functions (void);
+void _nc_hicosmo_de_register_functions (void);
+void _nc_distance_register_functions (void);
+void _nc_planck_fi_cor_tt_register_functions (void);
+
 /**
  * ncm_cfg_init:
  *
@@ -253,12 +270,9 @@ ncm_cfg_init (void)
 #endif
 
   gsl_err = gsl_set_error_handler_off ();
-#if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 32)
-  g_thread_init (NULL);
-#endif
 
 #if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 36)
-  g_type_init();
+  g_type_init ();
 #endif
 
   _log_stream = stdout;
@@ -282,6 +296,9 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NCM_TYPE_SPLINE2D_GSL);
   ncm_cfg_register_obj (NCM_TYPE_SPLINE2D_SPLINE);
 
+  ncm_cfg_register_obj (NCM_TYPE_POWSPEC);
+  ncm_cfg_register_obj (NCM_TYPE_POWSPEC_FILTER);
+  
   ncm_cfg_register_obj (NCM_TYPE_MODEL);
   ncm_cfg_register_obj (NCM_TYPE_MODEL_CTRL);
   ncm_cfg_register_obj (NCM_TYPE_MODEL_BUILDER);
@@ -292,9 +309,14 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NCM_TYPE_BOOTSTRAP);
   ncm_cfg_register_obj (NCM_TYPE_STATS_VEC);
 
+  ncm_cfg_register_obj (NCM_TYPE_FIT_ESMCMC_WALKER_STRETCH);
+
+  ncm_cfg_register_obj (NCM_TYPE_DATA);
+  
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_QCONST);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_QLINEAR);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_QSPLINE);
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_QSPLINE_CONT_PRIOR);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_LCDM);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_XCDM);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_LINDER);
@@ -303,9 +325,12 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_QGRW);
 
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_REPARAM_OK);
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_REPARAM_CMB);
 
   ncm_cfg_register_obj (NC_TYPE_HIPRIM_POWER_LAW);
   ncm_cfg_register_obj (NC_TYPE_HIPRIM_ATAN);
+  ncm_cfg_register_obj (NC_TYPE_HIPRIM_EXPC);
+  ncm_cfg_register_obj (NC_TYPE_HIPRIM_BPL);
 
   ncm_cfg_register_obj (NC_TYPE_CBE_PRECISION);
 
@@ -321,8 +346,6 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NC_TYPE_TRANSFER_FUNC_CAMB);
   ncm_cfg_register_obj (NC_TYPE_TRANSFER_FUNC_PERT);
 
-  ncm_cfg_register_obj (NC_TYPE_MATTER_VAR);
-
   ncm_cfg_register_obj (NC_TYPE_DENSITY_PROFILE);
   ncm_cfg_register_obj (NC_TYPE_DENSITY_PROFILE_NFW);
 
@@ -336,7 +359,7 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NC_TYPE_MULTIPLICITY_FUNC_TINKER_CRIT);
   ncm_cfg_register_obj (NC_TYPE_MULTIPLICITY_FUNC_TINKER_MEAN_NORMALIZED);
 
-  ncm_cfg_register_obj (NC_TYPE_MASS_FUNCTION);
+  ncm_cfg_register_obj (NC_TYPE_HALO_MASS_FUNCTION);
 
   ncm_cfg_register_obj (NC_TYPE_GALAXY_ACF);
 
@@ -375,10 +398,18 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NC_TYPE_HIREION);
   ncm_cfg_register_obj (NC_TYPE_HIREION_CAMB);
 
+  ncm_cfg_register_obj (NC_TYPE_POWSPEC_ML);
+  ncm_cfg_register_obj (NC_TYPE_POWSPEC_ML_TRANSFER);
+  ncm_cfg_register_obj (NC_TYPE_POWSPEC_ML_CBE);
+
+  ncm_cfg_register_obj (NC_TYPE_POWSPEC_MNL);
+  ncm_cfg_register_obj (NC_TYPE_POWSPEC_MNL_HALOFIT);
+
   ncm_cfg_register_obj (NC_TYPE_SNIA_DIST_COV);
 
   ncm_cfg_register_obj (NC_TYPE_PLANCK_FI);
   ncm_cfg_register_obj (NC_TYPE_PLANCK_FI_COR_TT);
+  ncm_cfg_register_obj (NC_TYPE_PLANCK_FI_COR_TTTEEE);
 
   ncm_cfg_register_obj (NC_TYPE_DATA_BAO_A);
   ncm_cfg_register_obj (NC_TYPE_DATA_BAO_DV);
@@ -402,6 +433,11 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NC_TYPE_XCOR_LIMBER_LENSING);
   ncm_cfg_register_obj (NC_TYPE_DATA_XCOR);
 
+  _nc_hicosmo_register_functions ();
+  _nc_hicosmo_de_register_functions ();
+  _nc_distance_register_functions ();
+  _nc_planck_fi_cor_tt_register_functions ();
+
   numcosmo_init = TRUE;
   return;
 }
@@ -429,7 +465,7 @@ static uint nreg_model = 0;
 void
 ncm_cfg_register_obj (GType obj)
 {
-#if !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 33))
+#if !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 34))
   g_type_ensure (obj);
 #endif /* GLIB >= 2.34*/
   gpointer obj_class = g_type_class_ref (obj);
@@ -617,7 +653,6 @@ ncm_cfg_get_fullpath (const gchar *filename, ...)
  *
  * FIXME
  *
- * Returns: FIXME
  */
 void
 ncm_cfg_keyfile_to_arg (GKeyFile *kfile, const gchar *group_name, GOptionEntry *entries, gchar **argv, gint *argc)
@@ -741,12 +776,8 @@ ncm_cfg_entries_to_keyfile (GKeyFile *kfile, const gchar *group_name, GOptionEnt
       }
       case G_OPTION_ARG_INT64:
       {
-#if !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 26))
         gint64 arg_l = ((gint64 *)entries[i].arg_data)[0];
         g_key_file_set_int64 (kfile, group_name, entries[i].long_name, arg_l);
-#else
-        g_error ("ncm_cfg_entries_to_keyfile: gint64 not supported, recompile "PACKAGE_NAME" with glib >= 2.26");
-#endif
         break;
       }
       case G_OPTION_ARG_DOUBLE:
@@ -882,6 +913,34 @@ ncm_cfg_enum_print_all (GType enum_type, const gchar *header)
 }
 
 #ifdef NUMCOSMO_HAVE_FFTW3
+
+G_LOCK_DEFINE_STATIC (fftw_saveload_lock);
+G_LOCK_DEFINE_STATIC (fftw_plan_lock);
+
+/**
+ * ncm_cfg_lock_plan_fftw:
+ *
+ * FIXME
+ *
+ */
+void 
+ncm_cfg_lock_plan_fftw (void)
+{
+  G_LOCK (fftw_plan_lock);  
+}
+
+/**
+ * ncm_cfg_unlock_plan_fftw:
+ *
+ * FIXME
+ *
+ */
+void 
+ncm_cfg_unlock_plan_fftw (void)
+{
+  G_UNLOCK (fftw_plan_lock);  
+}
+
 /**
  * ncm_cfg_load_fftw_wisdom:
  * @filename: FIXME
@@ -894,33 +953,50 @@ ncm_cfg_enum_print_all (GType enum_type, const gchar *header)
 gboolean
 ncm_cfg_load_fftw_wisdom (const gchar *filename, ...)
 {
-  FILE *wis;
-  gchar *file;
+  gchar *file, *file_ext;
   gchar *full_filename;
   va_list ap;
   gboolean ret = FALSE;
   g_assert (numcosmo_init);
 
+  G_LOCK (fftw_saveload_lock);
+  
   va_start (ap, filename);
   file = g_strdup_vprintf (filename, ap);
   va_end (ap);
 
-  full_filename = g_build_filename (numcosmo_path, file, NULL);
+  g_free (file);
+  file = g_strdup ("ncm_cfg_wisdom"); /* overwrite, unifying wisdom */
+  
+  file_ext      = g_strdup_printf ("%s.fftw3", file);
+  full_filename = g_build_filename (numcosmo_path, file_ext, NULL);
 
   if (g_file_test (full_filename, G_FILE_TEST_EXISTS))
   {
-    wis = g_fopen (full_filename, "r");
-    if (wis == NULL)
-    {
-      g_error ("ncm_cfg_load_fftw_wisdom: cannot open wisdom file %s [%s].", full_filename, g_strerror (errno));
-    }
-
-    fftw_import_wisdom_from_file (wis);
-    fclose (wis);
+    fftw_import_wisdom_from_filename (full_filename);
     ret = TRUE;
   }
-  g_free (file);
+
+#ifdef HAVE_FFTW3F
+  g_free (file_ext);
   g_free (full_filename);
+  
+  file_ext      = g_strdup_printf ("%s.fftw3f", file);
+  full_filename = g_build_filename (numcosmo_path, file_ext, NULL);
+
+  if (g_file_test (full_filename, G_FILE_TEST_EXISTS))
+  {
+    fftwf_import_wisdom_from_filename (full_filename);
+    ret = TRUE;
+  }
+#endif
+
+  g_free (file);
+  g_free (file_ext);
+  g_free (full_filename);
+
+  G_UNLOCK (fftw_saveload_lock);
+  
   return ret;
 }
 
@@ -936,28 +1012,42 @@ ncm_cfg_load_fftw_wisdom (const gchar *filename, ...)
 gboolean
 ncm_cfg_save_fftw_wisdom (const gchar *filename, ...)
 {
-  FILE *wis;
-  gchar *file;
+  gchar *file, *file_ext;
   gchar *full_filename;
   va_list ap;
 
   g_assert (numcosmo_init);
 
+G_LOCK (fftw_saveload_lock);
+
   va_start (ap, filename);
   file = g_strdup_vprintf (filename, ap);
   va_end (ap);
-  full_filename = g_build_filename (numcosmo_path, file, NULL);
 
-  wis = g_fopen (full_filename, "w");
-  if (wis == NULL)
-  {
-    g_error ("ncm_cfg_save_fftw_wisdom: cannot save wisdom file %s [%s].", full_filename, g_strerror (errno));
-  }
-
-  fftw_export_wisdom_to_file(wis);
-  fclose (wis);
   g_free (file);
+  file = g_strdup ("ncm_cfg_wisdom"); /* overwrite, unifying wisdom */
+
+  file_ext      = g_strdup_printf ("%s.fftw3", file);
+  full_filename = g_build_filename (numcosmo_path, file_ext, NULL);
+
+  fftw_export_wisdom_to_filename (full_filename);
+
+#ifdef HAVE_FFTW3F
+  g_free (file_ext);
   g_free (full_filename);
+  
+  file_ext      = g_strdup_printf ("%s.fftw3f", file);
+  full_filename = g_build_filename (numcosmo_path, file_ext, NULL);
+
+  fftwf_export_wisdom_to_filename (full_filename);
+#endif
+  
+  g_free (file);
+  g_free (file_ext);
+  g_free (full_filename);
+
+  G_UNLOCK (fftw_saveload_lock);
+  
   return TRUE;
 }
 #endif /* NUMCOSMO_HAVE_FFTW3 */
