@@ -52,6 +52,8 @@ const GEnumValue *ncm_cfg_get_enum_by_id_name_nick (GType enum_type, const gchar
 const GEnumValue *ncm_cfg_enum_get_value (GType enum_type, guint n);
 void ncm_cfg_enum_print_all (GType enum_type, const gchar *header);
 
+void ncm_cfg_lock_plan_fftw (void);
+void ncm_cfg_unlock_plan_fftw (void);
 gboolean ncm_cfg_load_fftw_wisdom (const gchar *filename, ...);
 gboolean ncm_cfg_save_fftw_wisdom (const gchar *filename, ...);
 gboolean ncm_cfg_exists (const gchar *filename, ...);
@@ -119,24 +121,6 @@ G_STMT_START { \
 #define NCM_CHECK_PREPARED(obj,name)
 #endif /* NUMCOSMO_CHECK_PREPARE */
 
-#if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 32)
-#define _NCM_MUTEX_LOCK(l) g_static_mutex_lock (l)
-#define _NCM_MUTEX_UNLOCK(l) g_static_mutex_unlock (l)
-#define _NCM_MUTEX_TRYLOCK(l) g_static_mutex_trylock (l)
-#define _NCM_MUTEX_TYPE GStaticMutex
-#define _NCM_STATIC_MUTEX_DECL(l) static GStaticMutex l = G_STATIC_MUTEX_INIT
-#define _NCM_MUTEX_INIT(l) g_static_mutex_init (l)
-#define _NCM_MUTEX_CLEAR(l) g_static_mutex_free (l)
-#else
-#define _NCM_MUTEX_LOCK(l) g_mutex_lock (l)
-#define _NCM_MUTEX_UNLOCK(l) g_mutex_unlock (l)
-#define _NCM_MUTEX_TRYLOCK(l) g_mutex_trylock (l)
-#define _NCM_MUTEX_TYPE GMutex
-#define _NCM_STATIC_MUTEX_DECL(l) static GMutex l
-#define _NCM_MUTEX_INIT(l) g_mutex_init (l)
-#define _NCM_MUTEX_CLEAR(l) g_mutex_clear (l)
-#endif
-
 #define NCM_ZERO_LIMIT 1e-13
 #define NCM_DEFAULT_PRECISION 1e-7
 
@@ -158,7 +142,7 @@ G_STMT_START { \
 G_STMT_START { \
   if (*(ptr) != NULL) \
   { \
-    freefunc (*(ptr)); \
+    (freefunc) (*(ptr)); \
     *(ptr) = NULL; \
   } \
 } G_STMT_END
