@@ -30,6 +30,7 @@
 #include <glib-object.h>
 #include <numcosmo/build_cfg.h>
 #include <numcosmo/math/ncm_vector.h>
+#include <numcosmo/math/ncm_spline.h>
 #include <gsl/gsl_math.h>
 #ifndef NUMCOSMO_GIR_SCAN
 #include <complex.h>
@@ -52,45 +53,44 @@ G_BEGIN_DECLS
 typedef struct _NcmFftlogClass NcmFftlogClass;
 typedef struct _NcmFftlog NcmFftlog;
 
-/**
- * NcmFftlog:
- * 
- * FIXME
- *
- */
-struct _NcmFftlog
-{
-  /*< private >*/
-  GObject parent_instance;
-  gint N;
-  gint N_2;
-  gdouble lnk0;
-  gdouble lnr0;
-  gdouble Lk;
-  gdouble Lk_N;
-  gboolean prepared;
-  gboolean evaluated;
-  NcmVector *lnr_vec;
-  NcmVector **Gr_vec;
-#ifdef NUMCOSMO_HAVE_FFTW3
-  fftw_complex *Fk;
-  fftw_complex *Cm;
-  fftw_complex **Gr;
-  fftw_complex **Ym;
-  fftw_complex **CmYm;
-  fftw_plan p_Fk2Cm;
-  fftw_plan *p_CmYm2Gr;
-#endif /* NUMCOSMO_HAVE_FFTW3 */
-};
-
 struct _NcmFftlogClass
 {
   /*< private >*/
   GObjectClass parent_class;
-  guint ncomp;
   const gchar *name;
-  void (*get_Ym) (NcmFftlog *fftlog);
-  void (*generate_Gr) (NcmFftlog *fftlog);
+  void (*get_Ym) (NcmFftlog *fftlog, gpointer Ym_0);
+};
+
+struct _NcmFftlog
+{
+  /*< private >*/
+  GObject parent_instance;
+  gint Nr;
+  gint N;
+  gint N_2;
+  gint Nf;
+  gint Nf_2;
+  guint nderivs;
+  guint pad;
+  gdouble lnk0;
+  gdouble lnr0;
+  gdouble Lk;
+  gdouble Lk_N;
+  gdouble pad_p;
+  gboolean prepared;
+  gboolean evaluated;
+  NcmVector *lnr_vec;
+  GPtrArray *Gr_vec;
+  GPtrArray *Gr_s;
+#ifdef NUMCOSMO_HAVE_FFTW3
+  fftw_complex *Fk;
+  fftw_complex *Cm;
+  fftw_complex *Gr;
+  fftw_complex *CmYm;
+  GPtrArray *Ym;
+  fftw_plan p_Fk2Cm;
+  fftw_plan p_CmYm2Gr;
+#endif /* NUMCOSMO_HAVE_FFTW3 */
 };
 
 GType ncm_fftlog_get_type (void) G_GNUC_CONST;
@@ -101,17 +101,100 @@ void ncm_fftlog_clear (NcmFftlog **fftlog);
 
 void ncm_fftlog_set_name (NcmFftlog *fftlog, const gchar *name);
 const gchar *ncm_fftlog_peek_name (NcmFftlog *fftlog);
+
+void ncm_fftlog_set_nderivs (NcmFftlog *fftlog, guint nderivs);
+guint ncm_fftlog_get_nderivs (NcmFftlog *fftlog);
+
+void ncm_fftlog_set_lnr0 (NcmFftlog *fftlog, const gdouble lnr0);
+gdouble ncm_fftlog_get_lnr0 (NcmFftlog *fftlog);
+
+void ncm_fftlog_set_lnk0 (NcmFftlog *fftlog, const gdouble lnk0);
+gdouble ncm_fftlog_get_lnk0 (NcmFftlog *fftlog);
+
 void ncm_fftlog_set_size (NcmFftlog *fftlog, guint n);
-guint ncm_fftlog_get_size (NcmFftlog *fftlog);
+
+void ncm_fftlog_set_padding (NcmFftlog *fftlog, gdouble pad_p);
+gdouble ncm_fftlog_get_padding (NcmFftlog *fftlog);
+
 void ncm_fftlog_set_length (NcmFftlog *fftlog, gdouble Lk);
-gdouble ncm_fftlog_get_length (NcmFftlog *fftlog);
 
 void ncm_fftlog_eval_by_vector (NcmFftlog *fftlog, NcmVector *Fk);
 void ncm_fftlog_eval_by_function (NcmFftlog *fftlog, gsl_function *Fk);
 
+void ncm_fftlog_prepare_splines (NcmFftlog *fftlog);
+
 NcmVector *ncm_fftlog_get_vector_lnr (NcmFftlog *fftlog);
-NcmVector *ncm_fftlog_get_vector_Gr (NcmFftlog *fftlog, guint comp);
+NcmVector *ncm_fftlog_get_vector_Gr (NcmFftlog *fftlog, guint nderiv);
+
+NcmSpline *ncm_fftlog_peek_spline_Gr (NcmFftlog *fftlog, guint nderiv);
+
+gdouble ncm_fftlog_eval_output (NcmFftlog *fftlog, guint nderiv, const gdouble lnr);
+
+void ncm_fftlog_calibrate_size (NcmFftlog *fftlog, gsl_function *Fk, gdouble reltol);
+
+G_INLINE_FUNC guint ncm_fftlog_get_size (NcmFftlog *fftlog);
+G_INLINE_FUNC gint ncm_fftlog_get_full_size (NcmFftlog *fftlog);
+G_INLINE_FUNC gdouble ncm_fftlog_get_norma (NcmFftlog *fftlog);
+G_INLINE_FUNC gdouble ncm_fftlog_get_length (NcmFftlog *fftlog);
+G_INLINE_FUNC gdouble ncm_fftlog_get_full_length (NcmFftlog *fftlog);
+
+G_INLINE_FUNC gint ncm_fftlog_get_mode_index (NcmFftlog *fftlog, gint i);
+
+G_INLINE_FUNC NcmVector *ncm_fftlog_peek_output_vector (NcmFftlog *fftlog, guint nderiv);
 
 G_END_DECLS
 
 #endif /* _NCM_FFTLOG_H_ */
+
+#ifndef _NC_FFTLOG_INLINE_H_
+#define _NC_FFTLOG_INLINE_H_
+#ifdef NUMCOSMO_HAVE_INLINE
+
+G_BEGIN_DECLS
+
+G_INLINE_FUNC guint
+ncm_fftlog_get_size (NcmFftlog *fftlog)
+{
+  return fftlog->N;
+}
+
+G_INLINE_FUNC gint
+ncm_fftlog_get_full_size (NcmFftlog *fftlog)
+{
+  return fftlog->Nf;
+}
+
+G_INLINE_FUNC gdouble
+ncm_fftlog_get_norma (NcmFftlog *fftlog)
+{
+  return ncm_fftlog_get_full_size (fftlog);
+}
+
+G_INLINE_FUNC gdouble
+ncm_fftlog_get_length (NcmFftlog *fftlog)
+{
+  return fftlog->Lk;
+}
+
+G_INLINE_FUNC gdouble 
+ncm_fftlog_get_full_length (NcmFftlog *fftlog)
+{
+  return fftlog->Lk + 2.0 * fftlog->Lk_N * fftlog->pad;
+}
+
+G_INLINE_FUNC gint 
+ncm_fftlog_get_mode_index (NcmFftlog *fftlog, gint i)
+{
+  return (i > fftlog->Nf_2) ? i - fftlog->Nf : i;
+}
+
+G_INLINE_FUNC NcmVector *
+ncm_fftlog_peek_output_vector (NcmFftlog *fftlog, guint nderiv)
+{
+  return g_ptr_array_index (fftlog->Gr_vec, nderiv);
+}
+
+G_END_DECLS
+
+#endif /* NUMCOSMO_HAVE_INLINE */
+#endif /* _NC_FFTLOG_INLINE_H_ */
