@@ -1284,7 +1284,7 @@ ncm_fit_log_state (NcmFit *fit)
       g_message ("#  m2lnL     = % 20.15g ( ", ncm_fit_state_get_m2lnL_curval (fit->fstate));
       for (i = 0; i < ncm_vector_len (fit->lh->m2lnL_v); i++)
       {
-        g_message ("% 11.5g ", ncm_vector_get (fit->lh->m2lnL_v, i));
+        g_message ("% 13.8g ", ncm_vector_get (fit->lh->m2lnL_v, i));
       }
       g_message (")\n");
     }
@@ -2276,7 +2276,6 @@ ncm_fit_prob (NcmFit *fit, NcmModelID mid, guint pid, gdouble a, gdouble b)
  *
  * FIXME
  *
- * Returns: FIXME
  */
 void
 ncm_fit_dprob (NcmFit *fit, NcmModelID mid, guint pid, gdouble a, gdouble b, gdouble step, gdouble norm)
@@ -2458,7 +2457,7 @@ ncm_fit_function_error (NcmFit *fit, NcmMSetFunc *func, gdouble *x, gboolean pre
   if (ncm_mset_fparam_len (fit->mset) < 1)
   {
     g_warning ("ncm_fit_function_error: called but no free parameters were set in #NcmMSet.");
-    *f = ncm_mset_func_eval (func, fit->mset, x);
+    *f = ncm_mset_func_eval_nvar (func, fit->mset, x);
     *sigma_f = 0.0;
   }
   else
@@ -2476,7 +2475,7 @@ ncm_fit_function_error (NcmFit *fit, NcmMSetFunc *func, gdouble *x, gboolean pre
     ret = gsl_blas_ddot (ncm_vector_gsl (v), ncm_vector_gsl (tmp1), &result);
     NCM_TEST_GSL_RESULT("ncm_fit_function_error[v.covar.v]", ret);
 
-    *f = ncm_mset_func_eval (func, fit->mset, x);
+    *f = ncm_mset_func_eval_nvar (func, fit->mset, x);
     *sigma_f = sqrt(result);
     if (pretty_print)
       g_message ("# % -12.4g +/- % -12.4g\n", *f, *sigma_f);
