@@ -76,20 +76,10 @@ ncm_fit_mc_init (NcmFitMC *mc)
   mc->cur_sample_id   = -1; /* Represents that no samples were calculated yet. */
   mc->write_index     = 0;
   mc->started         = FALSE;
-#if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 32)
-  mc->dup_fit         = g_mutex_new ();
-  mc->resample_lock   = g_mutex_new ();
-  mc->write_cond      = g_cond_new ();
-#else
-  g_mutex_init (&mc->dup_fit_m);
-  mc->dup_fit = &mc->dup_fit_m;
-
-  g_mutex_init (&mc->resample_lock_m);
-  mc->resample_lock = &mc->resample_lock_m;
-
-  g_cond_init (&mc->write_cond_m);
-  mc->write_cond = &mc->write_cond_m;
-#endif
+  g_mutex_init (&mc->dup_fit);
+  g_mutex_init (&mc->resample_lock);
+  g_mutex_init (&mc->update_lock);
+  g_cond_init (&mc->write_cond);
 }
 
 static void _ncm_fit_mc_set_fit_obj (NcmFitMC *mc, NcmFit *fit);
@@ -191,15 +181,10 @@ ncm_fit_mc_finalize (GObject *object)
 {
   NcmFitMC *mc = NCM_FIT_MC (object);
 
-#if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 32)
-  g_mutex_free (mc->dup_fit);
-  g_mutex_free (mc->resample_lock);
-  g_cond_free (mc->write_cond);
-#else
-  g_mutex_clear (mc->dup_fit);
-  g_mutex_clear (mc->resample_lock);
-  g_cond_clear (mc->write_cond);
-#endif
+  g_mutex_clear (&mc->dup_fit);
+  g_mutex_clear (&mc->resample_lock);
+  g_mutex_clear (&mc->update_lock);
+  g_cond_clear (&mc->write_cond);
   
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_fit_mc_parent_class)->finalize (object);
@@ -265,7 +250,9 @@ _ncm_fit_mc_set_fit_obj (NcmFitMC *mc, NcmFit *fit)
 {
   g_assert (mc->fit == NULL);
   mc->fit = ncm_fit_ref (fit);
-  mc->mcat = ncm_mset_catalog_new (fit->mset, 1, 1, FALSE, NCM_MSET_CATALOG_M2LNL_COLNAME);
+  mc->mcat = ncm_mset_catalog_new (fit->mset, 1, 1, FALSE, 
+                                   NCM_MSET_CATALOG_M2LNL_COLNAME, NCM_MSET_CATALOG_M2LNL_SYMBOL, 
+                                   NULL);
 }
 
 /**
@@ -491,7 +478,7 @@ _ncm_fit_mc_update (NcmFitMC *mc, NcmFit *fit)
   const guint part = 5;
   const guint step = (mc->n / part) == 0 ? 1 : (mc->n / part);
 
-  ncm_mset_catalog_add_from_mset (mc->mcat, fit->mset, ncm_fit_state_get_m2lnL_curval (fit->fstate));
+  ncm_mset_catalog_add_from_mset (mc->mcat, fit->mset, ncm_fit_state_get_m2lnL_curval (fit->fstate), NULL);
   ncm_timer_task_increment (mc->nt);
 
   switch (mc->mtype)
@@ -519,8 +506,7 @@ _ncm_fit_mc_update (NcmFitMC *mc, NcmFit *fit)
     default:
     case NCM_FIT_RUN_MSGS_FULL:
       fit->mtype = mc->mtype;
-      ncm_fit_log_start (fit);
-      ncm_fit_log_end (fit);
+      ncm_fit_log_state (fit);
       ncm_mset_catalog_log_current_stats (mc->mcat);
       /* ncm_timer_task_increment (mc->nt); */
       ncm_timer_task_log_elapsed (mc->nt);
@@ -529,13 +515,6 @@ _ncm_fit_mc_update (NcmFitMC *mc, NcmFit *fit)
       ncm_timer_task_log_end_datetime (mc->nt);
       break;      
   }
-
-  if ((mc->mcat->fmode != NCM_FIT_MC_MIN_FLUSH_INTERVAL) &&
-      (ncm_timer_task_mean_time (mc->nt) < NCM_FIT_MC_MIN_FLUSH_INTERVAL))
-  {
-    ncm_mset_catalog_set_flush_mode (mc->mcat, NCM_MSET_CATALOG_FLUSH_TIMED);
-    ncm_mset_catalog_set_flush_interval (mc->mcat, NCM_FIT_MC_MIN_FLUSH_INTERVAL);
-  }
 }
 
 void
@@ -596,6 +575,9 @@ ncm_fit_mc_start_run (NcmFitMC *mc)
 
   mc->started = TRUE;
 
+  ncm_mset_catalog_set_sync_mode (mc->mcat, NCM_MSET_CATALOG_SYNC_TIMED);
+  ncm_mset_catalog_set_sync_interval (mc->mcat, NCM_FIT_MC_MIN_SYNC_INTERVAL);
+  
   ncm_mset_catalog_sync (mc->mcat, TRUE);
   if (mc->mcat->cur_id > mc->cur_sample_id)
   {
@@ -783,11 +765,11 @@ static gpointer
 _ncm_fit_mc_dup_fit (gpointer userdata)
 {
   NcmFitMC *mc = NCM_FIT_MC (userdata);
-  g_mutex_lock (mc->dup_fit);
+  g_mutex_lock (&mc->dup_fit);
   {
     NcmFit *fit = ncm_fit_dup (mc->fit, mc->ser);
     ncm_serialize_clear_instances (mc->ser);
-    g_mutex_unlock (mc->dup_fit);
+    g_mutex_unlock (&mc->dup_fit);
     return fit;
   }
 }
@@ -795,7 +777,6 @@ _ncm_fit_mc_dup_fit (gpointer userdata)
 static void 
 _ncm_fit_mc_mt_eval_keep_order (glong i, glong f, gpointer data)
 {
-  _NCM_STATIC_MUTEX_DECL (update_lock);
   NcmFitMC *mc = NCM_FIT_MC (data);
   NcmFit **fit_ptr = ncm_memory_pool_get (mc->mp);
   NcmFit *fit = *fit_ptr;
@@ -807,21 +788,21 @@ _ncm_fit_mc_mt_eval_keep_order (glong i, glong f, gpointer data)
 
     ncm_mset_param_set_vector (fit->mset, mc->bf);
 
-    g_mutex_lock (mc->resample_lock);
+    g_mutex_lock (&mc->resample_lock);
     sample_index = _ncm_fit_mc_resample (mc, fit);
-    g_mutex_unlock (mc->resample_lock);
+    g_mutex_unlock (&mc->resample_lock);
 
     ncm_fit_run (fit, NCM_FIT_RUN_MSGS_NONE);
 
-    _NCM_MUTEX_LOCK (&update_lock);    
+    g_mutex_lock (&mc->update_lock);    
     while (mc->write_index != sample_index)
-      g_cond_wait (mc->write_cond, &update_lock);
+      g_cond_wait (&mc->write_cond, &mc->update_lock);
 
     _ncm_fit_mc_update (mc, fit);
     mc->write_index++;
-    g_cond_broadcast (mc->write_cond);
+    g_cond_broadcast (&mc->write_cond);
 
-    _NCM_MUTEX_UNLOCK (&update_lock);
+    g_mutex_unlock (&mc->update_lock);    
   }
 
   ncm_memory_pool_return (fit_ptr);
@@ -831,7 +812,7 @@ _ncm_fit_mc_mt_eval_keep_order (glong i, glong f, gpointer data)
 static void 
 _ncm_fit_mc_mt_eval (glong i, glong f, gpointer data)
 {
-  _NCM_STATIC_MUTEX_DECL (update_lock);
+  G_LOCK_DEFINE_STATIC (update_lock);
   NcmFitMC *mc = NCM_FIT_MC (data);
   NcmFit **fit_ptr = ncm_memory_pool_get (mc->mp);
   NcmFit *fit = *fit_ptr;
@@ -841,16 +822,16 @@ _ncm_fit_mc_mt_eval (glong i, glong f, gpointer data)
   {
     ncm_mset_param_set_vector (fit->mset, mc->bf);
 
-    g_mutex_lock (mc->resample_lock);
+    g_mutex_lock (&mc->resample_lock);
     _ncm_fit_mc_resample (mc, fit);
-    g_mutex_unlock (mc->resample_lock);
+    g_mutex_unlock (&mc->resample_lock);
 
     ncm_fit_run (fit, NCM_FIT_RUN_MSGS_NONE);
 
-    _NCM_MUTEX_LOCK (&update_lock);    
+    G_LOCK (update_lock);    
     _ncm_fit_mc_update (mc, fit);
     mc->write_index++;
-    _NCM_MUTEX_UNLOCK (&update_lock);
+    G_UNLOCK (update_lock);
   }
 
   ncm_memory_pool_return (fit_ptr);
