@@ -76,20 +76,11 @@ ncm_fit_mcmc_init (NcmFitMCMC *mcmc)
   mcmc->ntotal          = 0;
   mcmc->write_index     = 0;
   mcmc->started         = FALSE;
-#if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 32)
-  mcmc->dup_fit         = g_mutex_new ();
-  mcmc->resample_lock   = g_mutex_new ();
-  mcmc->write_cond      = g_cond_new ();
-#else
-  g_mutex_init (&mcmc->dup_fit_m);
-  mcmc->dup_fit = &mcmc->dup_fit_m;
 
-  g_mutex_init (&mcmc->resample_lock_m);
-  mcmc->resample_lock = &mcmc->resample_lock_m;
-
-  g_cond_init (&mcmc->write_cond_m);
-  mcmc->write_cond = &mcmc->write_cond_m;
-#endif
+  g_mutex_init (&mcmc->dup_fit);
+  g_mutex_init (&mcmc->resample_lock);
+  g_mutex_init (&mcmc->update_lock);
+  g_cond_init (&mcmc->write_cond);
 }
 
 static void _ncm_fit_mcmc_set_fit_obj (NcmFitMCMC *mcmc, NcmFit *fit);
@@ -180,15 +171,10 @@ ncm_fit_mcmc_finalize (GObject *object)
 {
   NcmFitMCMC *mcmc = NCM_FIT_MCMC (object);
 
-#if (GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 32)
-  g_mutex_free (mcmc->dup_fit);
-  g_mutex_free (mcmc->resample_lock);
-  g_cond_free (mcmc->write_cond);
-#else
-  g_mutex_clear (mcmc->dup_fit);
-  g_mutex_clear (mcmc->resample_lock);
-  g_cond_clear (mcmc->write_cond);
-#endif
+  g_mutex_clear (&mcmc->dup_fit);
+  g_mutex_clear (&mcmc->resample_lock);
+  g_mutex_clear (&mcmc->update_lock);
+  g_cond_clear (&mcmc->write_cond);
   
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_fit_mcmc_parent_class)->finalize (object);
@@ -240,7 +226,9 @@ _ncm_fit_mcmc_set_fit_obj (NcmFitMCMC *mcmc, NcmFit *fit)
 {
   g_assert (mcmc->fit == NULL);
   mcmc->fit = ncm_fit_ref (fit);
-  mcmc->mcat = ncm_mset_catalog_new (fit->mset, 1, 1, FALSE, NCM_MSET_CATALOG_M2LNL_COLNAME);
+  mcmc->mcat = ncm_mset_catalog_new (fit->mset, 1, 1, FALSE, 
+                                     NCM_MSET_CATALOG_M2LNL_COLNAME, NCM_MSET_CATALOG_M2LNL_SYMBOL, 
+                                     NULL);
 }
 
 /**
@@ -400,7 +388,7 @@ _ncm_fit_mcmc_update (NcmFitMCMC *mcmc, NcmFit *fit)
   const guint part = 5;
   const guint step = (mcmc->n / part) == 0 ? 1 : (mcmc->n / part);
 
-  ncm_mset_catalog_add_from_mset (mcmc->mcat, fit->mset, ncm_fit_state_get_m2lnL_curval (fit->fstate));
+  ncm_mset_catalog_add_from_mset (mcmc->mcat, fit->mset, ncm_fit_state_get_m2lnL_curval (fit->fstate), NULL);
   ncm_timer_task_increment (mcmc->nt);
 
   switch (mcmc->mtype)
@@ -430,8 +418,7 @@ _ncm_fit_mcmc_update (NcmFitMCMC *mcmc, NcmFit *fit)
     default:
     case NCM_FIT_RUN_MSGS_FULL:
       fit->mtype = mcmc->mtype;
-      ncm_fit_log_start (fit);
-      ncm_fit_log_end (fit);
+      ncm_fit_log_state (fit);
       ncm_mset_catalog_log_current_stats (mcmc->mcat);
       g_message ("# NcmFitMCMC:acceptance ratio %7.4f%%.\n", ncm_fit_mcmc_get_accept_ratio (mcmc) * 100.0);
       /* ncm_timer_task_increment (mcmc->nt); */
@@ -441,13 +428,6 @@ _ncm_fit_mcmc_update (NcmFitMCMC *mcmc, NcmFit *fit)
       ncm_timer_task_log_end_datetime (mcmc->nt);
       break;      
   }
-
-  if ((mcmc->mcat->fmode != NCM_MSET_CATALOG_FLUSH_TIMED) &&
-      (ncm_timer_task_mean_time (mcmc->nt) < NCM_FIT_MCMC_MIN_FLUSH_INTERVAL))
-  {
-    ncm_mset_catalog_set_flush_mode (mcmc->mcat, NCM_MSET_CATALOG_FLUSH_TIMED);
-    ncm_mset_catalog_set_flush_interval (mcmc->mcat, NCM_FIT_MCMC_MIN_FLUSH_INTERVAL);
-  }
 }
 
 static void ncm_fit_mcmc_intern_skip (NcmFitMCMC *mcmc, guint n);
@@ -494,7 +474,7 @@ ncm_fit_mcmc_start_run (NcmFitMCMC *mcmc)
   }
 
   mcmc->started = TRUE;
-
+  
   {
     guint fparam_len = ncm_mset_fparam_len (mcmc->fit->mset);
     if (mcmc->theta != NULL)
@@ -509,6 +489,9 @@ ncm_fit_mcmc_start_run (NcmFitMCMC *mcmc)
   mcmc->naccepted = 0;
   mcmc->ntotal = 0;
   
+  ncm_mset_catalog_set_sync_mode (mcmc->mcat, NCM_MSET_CATALOG_SYNC_TIMED);
+  ncm_mset_catalog_set_sync_interval (mcmc->mcat, NCM_FIT_MCMC_MIN_SYNC_INTERVAL);
+
   ncm_mset_catalog_sync (mcmc->mcat, TRUE);
   if (mcmc->mcat->cur_id > mcmc->cur_sample_id)
   {
@@ -737,11 +720,11 @@ static gpointer
 _ncm_fit_mcmc_dup_fit (gpointer userdata)
 {
   NcmFitMCMC *mcmc = NCM_FIT_MCMC (userdata);
-  g_mutex_lock (mcmc->dup_fit);
+  g_mutex_lock (&mcmc->dup_fit);
   {
     NcmFit *fit = ncm_fit_dup (mcmc->fit, mcmc->ser);
     ncm_serialize_clear_instances (mcmc->ser);
-    g_mutex_unlock (mcmc->dup_fit);
+    g_mutex_unlock (&mcmc->dup_fit);
     return fit;
   }
 }
@@ -749,7 +732,6 @@ _ncm_fit_mcmc_dup_fit (gpointer userdata)
 static void 
 _ncm_fit_mcmc_mt_eval (glong i, glong f, gpointer data)
 {
-  _NCM_STATIC_MUTEX_DECL (update_lock);
   NcmFitMCMC *mcmc = NCM_FIT_MCMC (data);
   NcmFit **fit_ptr = ncm_memory_pool_get (mcmc->mp);
   NcmFit *fit = *fit_ptr;
@@ -763,22 +745,22 @@ _ncm_fit_mcmc_mt_eval (glong i, glong f, gpointer data)
 
 //    ncm_mset_param_set_vector (fit->mset, mcmc->bf);
 
-    g_mutex_lock (mcmc->resample_lock);
+    g_mutex_lock (&mcmc->resample_lock);
 //    sample_index = _ncm_fit_mcmc_resample (mc, fit);
     sample_index = 0;
-    g_mutex_unlock (mcmc->resample_lock);
+    g_mutex_unlock (&mcmc->resample_lock);
 
     ncm_fit_run (fit, NCM_FIT_RUN_MSGS_NONE);
 
-    _NCM_MUTEX_LOCK (&update_lock);    
+    g_mutex_lock (&mcmc->update_lock);    
     while (mcmc->write_index != sample_index)
-      g_cond_wait (mcmc->write_cond, &update_lock);
+      g_cond_wait (&mcmc->write_cond, &mcmc->update_lock);
 
     _ncm_fit_mcmc_update (mcmc, fit);
     mcmc->write_index++;
-    g_cond_broadcast (mcmc->write_cond);
+    g_cond_broadcast (&mcmc->write_cond);
 
-    _NCM_MUTEX_UNLOCK (&update_lock);
+    g_mutex_unlock (&mcmc->update_lock);
   }
 
   ncm_memory_pool_return (fit_ptr);
