@@ -30,11 +30,13 @@
 
 G_BEGIN_DECLS
 
-typedef void (*NcmLoopFunc) (glong i, glong f, gpointer data);
+typedef void (*NcmFuncEvalLoop) (glong i, glong f, gpointer data);
 
 void ncm_func_eval_set_max_threads (gint mt);
-void ncm_func_eval_threaded_loop_nw (NcmLoopFunc lfunc, glong i, glong f, gpointer data, guint nworkers);
-void ncm_func_eval_threaded_loop (NcmLoopFunc lfunc, glong i, glong f, gpointer data);
+void ncm_func_eval_threaded_loop_nw (NcmFuncEvalLoop lfunc, glong i, glong f, gpointer data, guint nworkers);
+void ncm_func_eval_threaded_loop (NcmFuncEvalLoop lfunc, glong i, glong f, gpointer data);
+void ncm_func_eval_threaded_loop_full (NcmFuncEvalLoop lfunc, glong i, glong f, gpointer data);
+void ncm_func_eval_log_pool_stats (void);
 
 G_END_DECLS
 
