@@ -130,7 +130,8 @@ _ncm_mset_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
       for (i = 0; i < mset->model_array->len; i++)
       {
         NcmMSetItem *item = g_ptr_array_index (mset->model_array, i);
-        ncm_obj_array_add (oa, G_OBJECT (item->model));
+        if (!NCM_MODEL_GET_CLASS (item->model)->is_submodel)
+          ncm_obj_array_add (oa, G_OBJECT (item->model));
       }
       g_value_take_boxed (value, oa);
       break;
@@ -172,7 +173,11 @@ _ncm_mset_set_property (GObject *object, guint prop_id, const GValue *value, GPa
         guint i;
         for (i = 0; i < oa->len; i++)
         {
-          ncm_mset_push (mset, NCM_MODEL (ncm_obj_array_peek (oa, i)));
+          NcmModel *model = NCM_MODEL (ncm_obj_array_peek (oa, i));
+          if (NCM_MODEL_GET_CLASS (model)->is_submodel)
+            g_error ("_ncm_mset_set_property: NcmMSet model array cannot contain submodels `%s'.",
+                     G_OBJECT_TYPE_NAME (model));
+          ncm_mset_push (mset, model);
         }
       }
       break;
@@ -280,7 +285,7 @@ ncm_mset_pindex_new (NcmModelID mid, guint pid)
  *
  * FIXME
  *
- * Returns: FIXME
+ * Returns: (transfer full): FIXME
  */
 NcmMSetPIndex *
 ncm_mset_pindex_dup (NcmMSetPIndex *pi)
@@ -311,12 +316,13 @@ G_LOCK_DEFINE_STATIC (last_model_id);
  * @desc: short description
  * @long_desc: long description
  * @can_stack: whether the models can stack in a #NcmMSet
+ * @main_model_id: main model id, use -1 if this is a main model
  *
  * FIXME
  *
  */
 void
-ncm_mset_model_register_id (NcmModelClass *model_class, const gchar *ns, const gchar *desc, const gchar *long_desc, gboolean can_stack)
+ncm_mset_model_register_id (NcmModelClass *model_class, const gchar *ns, const gchar *desc, const gchar *long_desc, gboolean can_stack, NcmModelID main_model_id)
 {
   if (model_class->model_id < 0)
   {
@@ -327,7 +333,13 @@ ncm_mset_model_register_id (NcmModelClass *model_class, const gchar *ns, const g
 
     model_class->can_stack = can_stack;
 
-    model_class->model_id = last_model_id * NCM_MSET_MAX_SUBMODEL;
+    if (main_model_id > -1)
+    {
+      model_class->is_submodel   = TRUE;
+      model_class->main_model_id = main_model_id;
+    }
+
+    model_class->model_id = last_model_id * NCM_MSET_MAX_STACKSIZE;
     id = last_model_id;
     last_model_id++;
     g_array_set_size (mset_class->model_desc_array, last_model_id);
@@ -497,7 +509,8 @@ ncm_mset_shallow_copy (NcmMSet *mset)
   for (i = 0; i < nmodels; i++)
   {
     NcmModel *model = ncm_mset_peek_array_pos (mset, i);
-    ncm_mset_push (mset_sc, model);
+    if (!ncm_model_is_submodel (model))
+      ncm_mset_push (mset_sc, model);
   }
   if (mset->valid_map)
     ncm_mset_prepare_fparam_map (mset_sc);
@@ -554,16 +567,16 @@ ncm_mset_peek (NcmMSet *mset, NcmModelID mid)
  * ncm_mset_peek_pos:
  * @mset: a #NcmMSet
  * @base_mid: a #NcmModelID
- * @submodel_id: FIXME
+ * @stackpos_id: FIXME
  *
  * FIXME
  *
  * Returns: (transfer none): FIXME
  */
 NcmModel *
-ncm_mset_peek_pos (NcmMSet *mset, NcmModelID base_mid, guint submodel_id)
+ncm_mset_peek_pos (NcmMSet *mset, NcmModelID base_mid, guint stackpos_id)
 {
-  NcmModelID mid = base_mid + submodel_id;
+  NcmModelID mid = base_mid + stackpos_id;
   return ncm_mset_peek (mset, mid);
 }
 
@@ -609,7 +622,7 @@ ncm_mset_peek_array_pos (NcmMSet *mset, guint i)
  *
  * FIXME
  *
- * Returns: (transfer none): FIXME
+ * Returns: FIXME
  */
 NcmModelID
 ncm_mset_get_mid_array_pos (NcmMSet *mset, guint i)
@@ -672,44 +685,60 @@ ncm_mset_push (NcmMSet *mset, NcmModel *model)
   g_assert (model != NULL);
   {
     NcmModelID base_mid  = ncm_model_id (model);
-    guint submodel_id = 0;
+    guint stackpos_id = 0;
     while (TRUE)
     {
-      NcmModelID mid = base_mid + submodel_id;
+      NcmModelID mid = base_mid + stackpos_id;
       if (g_hash_table_lookup (mset->mid_item_hash, GINT_TO_POINTER (mid)) == NULL)
       {
-        ncm_mset_set_pos (mset, model, submodel_id);
+        ncm_mset_set_pos (mset, model, stackpos_id);
         break;
       }
-      submodel_id++;
+      stackpos_id++;
     }
   }
 }
 
+static void _ncm_mset_set_pos_intern (NcmMSet *mset, NcmModel *model, guint stackpos_id);
+
 /**
  * ncm_mset_set_pos:
  * @mset: a #NcmMSet
  * @model: a #NcmModel
- * @submodel_id: FIXME
+ * @stackpos_id: FIXME
  *
  * FIXME
  *
  */
 void
-ncm_mset_set_pos (NcmMSet *mset, NcmModel *model, guint submodel_id)
+ncm_mset_set_pos (NcmMSet *mset, NcmModel *model, guint stackpos_id)
+{
+  NcmModelClass *model_class = NCM_MODEL_GET_CLASS (model);
+  
+  if (model_class->is_submodel)
+    g_error ("ncm_mset_set_pos: cannot add model `%s' directly to a NcmMSet.\n"
+             "                  This model is a submodel of `%s' and it should"
+             " be added to it instead of directly to a NcmMSet", 
+             G_OBJECT_TYPE_NAME (model), ncm_mset_get_ns_by_id (model_class->main_model_id));
+  else
+    _ncm_mset_set_pos_intern (mset, model, stackpos_id);
+}
+  
+static void
+_ncm_mset_set_pos_intern (NcmMSet *mset, NcmModel *model, guint stackpos_id)
 {
   g_assert (model != NULL);
   {
-    NcmModelID mid  = ncm_model_id (model) + submodel_id;
+    NcmModelID mid  = ncm_model_id (model) + stackpos_id;
     guint model_len = ncm_model_len (model);
     NcmMSetItem *item = _ncm_mset_item_new (model, mid);
     GArray *fpi_array;
     NcmMSetItem *item0;
     guint i;
 
-    if (submodel_id > 0 && !(NCM_MODEL_GET_CLASS (model)->can_stack))
+    if (stackpos_id > 0 && !(NCM_MODEL_GET_CLASS (model)->can_stack))
       g_error ("ncm_mset_set_pos: cannot stack object in position %u NcmMSet, type `%s' not allowed.", 
-               submodel_id, G_OBJECT_TYPE_NAME (model));
+               stackpos_id, G_OBJECT_TYPE_NAME (model));
 
     ncm_mset_remove (mset, mid);
 
@@ -748,6 +777,12 @@ ncm_mset_set_pos (NcmMSet *mset, NcmModel *model, guint submodel_id)
 
     if (mset->valid_map)
       ncm_mset_prepare_fparam_map (mset);
+
+    for (i = 0; i < ncm_model_get_submodel_len (model); i++)
+    {
+      NcmModel *submodel = ncm_model_peek_submodel (model, i);
+      _ncm_mset_set_pos_intern (mset, submodel, 0);
+    }
   }
 }
 
@@ -775,16 +810,16 @@ ncm_mset_exists (NcmMSet *mset, NcmModel *model)
  * ncm_mset_exists_pos:
  * @mset: a #NcmMSet
  * @model: a #NcmModel
- * @submodel_id: FIXME
+ * @stackpos_id: FIXME
  *
  * FIXME
  *
  * Returns: FIXME
  */
 gboolean
-ncm_mset_exists_pos (NcmMSet *mset, NcmModel *model, guint submodel_id)
+ncm_mset_exists_pos (NcmMSet *mset, NcmModel *model, guint stackpos_id)
 {
-  NcmModelID mid = NCM_MSET_MID (ncm_model_id (model), submodel_id);
+  NcmModelID mid = NCM_MSET_MID (ncm_model_id (model), stackpos_id);
   if (ncm_mset_peek (mset, mid) != NULL)
     return TRUE;
   else
@@ -817,6 +852,33 @@ ncm_mset_is_subset (NcmMSet *mset, NcmMSet *sub_mset)
   return TRUE;
 }
 
+/**
+ * ncm_mset_get_id_by_type:
+ * @model_type: FIXME
+ *
+ * FIXME
+ *
+ * Returns: FIXME
+ */
+gint
+ncm_mset_get_id_by_type (GType model_type)
+{
+  g_assert (g_type_is_a (model_type, NCM_TYPE_MODEL));
+  {
+    NcmMSetClass *mset_class = g_type_class_ref (NCM_TYPE_MSET);
+    const gchar *ns = g_type_name (model_type);
+    gpointer model_id;
+    gboolean has = g_hash_table_lookup_extended (mset_class->ns_table, ns, NULL, &model_id);
+
+    g_type_class_unref (mset_class);
+
+    if (has)
+      return GPOINTER_TO_INT (model_id);
+    else
+      return -1;
+  }
+}
+
 /**
  * ncm_mset_get_id_by_ns:
  * @ns: FIXME
@@ -850,7 +912,7 @@ const gchar *
 ncm_mset_get_ns_by_id (gint id)
 {
   NcmMSetClass *mset_class = g_type_class_ref (NCM_TYPE_MSET);
-  guint base_mid = id / NCM_MSET_MAX_SUBMODEL;
+  guint base_mid = id / NCM_MSET_MAX_STACKSIZE;
 
   g_assert_cmpint (base_mid, <, mset_class->model_desc_array->len);
   {
@@ -860,6 +922,30 @@ ncm_mset_get_ns_by_id (gint id)
   }
 }
 
+/**
+ * ncm_mset_get_type_by_id:
+ * @id: namespace id
+ *
+ * Returns: GType of model @id
+ */
+GType
+ncm_mset_get_type_by_id (gint id)
+{
+  NcmMSetClass *mset_class = g_type_class_ref (NCM_TYPE_MSET);
+  guint base_mid = id / NCM_MSET_MAX_STACKSIZE;
+
+  g_assert_cmpint (base_mid, <, mset_class->model_desc_array->len);
+  {
+    const gchar *ns = g_array_index (mset_class->model_desc_array, NcmMSetModelDesc, base_mid).ns;
+    GType t = g_type_from_name (ns);
+    g_type_class_unref (mset_class);
+
+    g_assert_cmpint (t, !=, 0);
+    
+    return t;
+  }
+}
+
 /**
  * ncm_mset_prepare_fparam_map:
  * @mset: a #NcmMSet
@@ -956,6 +1042,7 @@ ncm_mset_set_fmap (NcmMSet *mset, const gchar *const *fmap, gboolean update_mode
         g_array_append_val (mset->pi_array, *pi);
         g_array_index (fpi_array, gint, pi->pid) = i;
       }
+      ncm_mset_pindex_free (pi);
     }
 
     g_ptr_array_set_size (mset->fullname_parray, mset->fparam_len);
@@ -1278,10 +1365,11 @@ ncm_mset_params_print_vals (NcmMSet *mset, FILE *out)
 void
 ncm_mset_fparams_log_covar (NcmMSet *mset, NcmMatrix *covar)
 {
+  const guint name_size       = ncm_mset_max_fparam_name (mset);
+  const guint free_params_len = ncm_mset_fparam_len (mset);
+  const gchar *box            = "---------------";
   guint i, j;
-  guint name_size = ncm_mset_max_fparam_name (mset);
-  guint free_params_len = ncm_mset_fparam_len (mset);
-  const gchar *box = "---------------";
+
   g_assert (covar != NULL);
 
   ncm_cfg_msg_sepa ();
@@ -1864,7 +1952,7 @@ ncm_mset_fparams_get_vector (NcmMSet *mset, NcmVector *x)
  * ncm_mset_fparams_get_vector_offset:
  * @mset: a #NcmMSet
  * @x: a #NcmVector
- * @offset: FIXME
+ * @offset: starting index
  *
  * FIXME
  *
@@ -2034,10 +2122,10 @@ ncm_mset_fparam_full_name (NcmMSet *mset, guint n)
     NcmMSetPIndex pi = g_array_index (mset->pi_array, NcmMSetPIndex, n);
     const gchar *model_ns = ncm_mset_get_ns_by_id (pi.mid); /* ncm_model_nick (ncm_mset_peek (mset, pi.mid));*/
     const gchar *pname = ncm_mset_param_name (mset, pi.mid, pi.pid);
-    const guint submodel_id = pi.mid % NCM_MSET_MAX_SUBMODEL;
+    const guint stackpos_id = pi.mid % NCM_MSET_MAX_STACKSIZE;
 
-    if (submodel_id > 0)
-      fullname = g_strdup_printf ("%s:%02u:%s", model_ns, submodel_id, pname);
+    if (stackpos_id > 0)
+      fullname = g_strdup_printf ("%s:%02u:%s", model_ns, stackpos_id, pname);
     else
       fullname = g_strdup_printf ("%s:%s", model_ns, pname);
 
@@ -2065,33 +2153,33 @@ ncm_mset_param_get_by_full_name (NcmMSet *mset, const gchar *fullname)
   {
     gint nm = g_match_info_get_match_count (match_info);
     gchar *ns         = NULL;
-    gchar *submodel_s = NULL;
+    gchar *stackpos_s = NULL;
     gchar *pid_s      = NULL;
     gchar *endptr     = NULL;
     guint pid      = 0;
-    guint submodel = 0;
+    guint stackpos = 0;
     NcmModelID mid;
     NcmModel *model;
 
     g_assert_cmpint (nm, ==, 4);
 
     ns         = g_match_info_fetch (match_info, 1);
-    submodel_s = g_match_info_fetch (match_info, 2);
+    stackpos_s = g_match_info_fetch (match_info, 2);
     pid_s      = g_match_info_fetch (match_info, 3);
 
     mid = ncm_mset_get_id_by_ns (ns);
     if (mid < 0)
       g_error ("ncm_mset_param_get_by_full_name: namespace `%s' not found.", ns);
 
-    if (*submodel_s != '\0')
+    if (*stackpos_s != '\0')
     {
-      submodel = g_ascii_strtoll (submodel_s, &endptr, 10);
+      stackpos = g_ascii_strtoll (stackpos_s, &endptr, 10);
       if (*endptr != '\0')
-        g_error ("ncm_mset_param_get_by_full_name: invalid submodel number `%s'.", submodel_s);
+        g_error ("ncm_mset_param_get_by_full_name: invalid stackpos number `%s'.", stackpos_s);
     }
-    g_free (submodel_s);
+    g_free (stackpos_s);
 
-    mid += submodel;
+    mid += stackpos;
     model = ncm_mset_peek (mset, mid);
     if (model != NULL)
     {
@@ -2114,6 +2202,8 @@ ncm_mset_param_get_by_full_name (NcmMSet *mset, const gchar *fullname)
   else
     g_error ("ncm_mset_param_get_by_full_name: invalid full name `%s'.",  fullname);
 
+  g_match_info_free (match_info);
+
   return pi;
 }
 
@@ -2224,7 +2314,8 @@ ncm_mset_fparam_set_scale (NcmMSet *mset, guint n, gdouble scale)
 gboolean
 ncm_mset_fparam_valid_bounds (NcmMSet *mset, NcmVector *theta)
 {
-  g_assert (mset->valid_map && ncm_vector_len (theta) == mset->fparam_len);
+  g_assert (mset->valid_map);
+  g_assert_cmpuint (ncm_vector_len (theta), ==, mset->fparam_len);
   {
     guint i;
     for (i = 0; i < mset->fparam_len; i++)
@@ -2239,6 +2330,35 @@ ncm_mset_fparam_valid_bounds (NcmMSet *mset, NcmVector *theta)
   }
 }
 
+/**
+ * ncm_mset_fparam_valid_bounds_offset:
+ * @mset: a #NcmMSet
+ * @theta: free parameters vector
+ * @offset: starting index
+ *
+ * FIXME
+ *
+ * Returns: whether @theta contain values respecting the parameter bounds.
+ */
+gboolean
+ncm_mset_fparam_valid_bounds_offset (NcmMSet *mset, NcmVector *theta, guint offset)
+{
+  g_assert (mset->valid_map);
+  g_assert_cmpuint (ncm_vector_len (theta), ==, mset->fparam_len + offset);
+  {
+    guint i;
+    for (i = 0; i < mset->fparam_len; i++)
+    {
+      const gdouble lb  = ncm_mset_fparam_get_lower_bound (mset, i);
+      const gdouble ub  = ncm_mset_fparam_get_upper_bound (mset, i);
+      const gdouble val = ncm_vector_get (theta, i + offset);
+      if (val < lb || val > ub)
+        return FALSE;
+    }
+    return TRUE;
+  }
+}
+
 /**
  * ncm_mset_fparam_get:
  * @mset: a #NcmMSet
@@ -2413,14 +2533,14 @@ ncm_mset_save (NcmMSet *mset, NcmSerialize *ser, const gchar *filename, gboolean
   for (i = 0; i < mset->model_array->len; i++)
   {
     NcmMSetItem *item       = g_ptr_array_index (mset->model_array, i);
-    NcmModelID mid_base     = item->mid / NCM_MSET_MAX_SUBMODEL;
-    const guint submodel_id = item->mid % NCM_MSET_MAX_SUBMODEL;
+    NcmModelID mid_base     = item->mid / NCM_MSET_MAX_STACKSIZE;
+    const guint stackpos_id = item->mid % NCM_MSET_MAX_STACKSIZE;
     const gchar *ns = g_array_index (mset_class->model_desc_array, NcmMSetModelDesc, mid_base).ns;
     GError *error = NULL;
     gchar *group;
 
-    if (submodel_id > 0)
-      group = g_strdup_printf ("%s:%02u", ns, submodel_id);
+    if (stackpos_id > 0)
+      group = g_strdup_printf ("%s:%02u", ns, stackpos_id);
     else
       group = g_strdup_printf ("%s", ns);
 
@@ -2429,7 +2549,7 @@ ncm_mset_save (NcmMSet *mset, NcmSerialize *ser, const gchar *filename, gboolean
       NcmMSetItem *item0 = g_hash_table_lookup (mset->model_item_hash, item->model);
       g_assert (item0 != NULL);
 
-      g_key_file_set_uint64 (msetfile, group, ns, item0->mid % NCM_MSET_MAX_SUBMODEL);
+      g_key_file_set_uint64 (msetfile, group, ns, item0->mid % NCM_MSET_MAX_STACKSIZE);
       if (save_comment)
       {
         gchar *model_desc = ncm_cfg_string_to_comment (g_array_index (mset_class->model_desc_array, NcmMSetModelDesc, mid_base).desc);
@@ -2443,9 +2563,10 @@ ncm_mset_save (NcmMSet *mset, NcmSerialize *ser, const gchar *filename, gboolean
       NcmModelClass *model_class = NCM_MODEL_GET_CLASS (item->model);
       GObjectClass *oclass       = G_OBJECT_CLASS (model_class);
       GVariant *model_var        = ncm_serialize_to_variant (ser, G_OBJECT (item->model));
+      guint nsubmodels           = ncm_model_get_submodel_len (item->model);
       GVariant *params = NULL;
       gchar *obj_name = NULL;
-      guint nparams;
+      guint nparams, j;
 
       g_variant_get (model_var, "{s@a{sv}}", &obj_name, &params);
       nparams = g_variant_n_children (params);
@@ -2459,6 +2580,13 @@ ncm_mset_save (NcmMSet *mset, NcmSerialize *ser, const gchar *filename, gboolean
         g_free (model_desc);
       }
 
+      for (j = 0; j < nsubmodels; j++)
+      {
+        NcmModel *submodel = ncm_model_peek_submodel (item->model, j);
+        ncm_serialize_unset (ser, submodel);
+        ncm_serialize_remove_ser (ser, submodel);
+      }
+
       if (nparams != 0)
       {
         GVariantIter iter;
@@ -2467,26 +2595,40 @@ ncm_mset_save (NcmMSet *mset, NcmSerialize *ser, const gchar *filename, gboolean
         g_variant_iter_init (&iter, params);
         while (g_variant_iter_next (&iter, "{sv}", &key, &value))
         {
-          GParamSpec *param_spec = g_object_class_find_property (oclass, key);
-          gchar *param_str = g_variant_print (value, TRUE);
-          if (param_spec == NULL)
-            g_error ("ncm_mset_save: property `%s' not found in object `%s'.", key, obj_name);
-
-          g_key_file_set_value (msetfile, group, key, param_str);
-          if (save_comment)
+          if (strcmp (key, "submodel-array") == 0)
           {
-            gchar *desc = ncm_cfg_string_to_comment (g_param_spec_get_blurb (param_spec));
-            if (!g_key_file_set_comment (msetfile, group, key, desc, &error))
-              g_error ("ncm_mset_save: %s", error->message);
-            g_free (desc);
+            g_variant_unref (value);
+            g_free (key);
+            continue;
           }
+          else
+          {
+            GParamSpec *param_spec = g_object_class_find_property (oclass, key);
+            gchar *param_str = g_variant_print (value, TRUE);
+            if (param_spec == NULL)
+              g_error ("ncm_mset_save: property `%s' not found in object `%s'.", key, obj_name);
 
-          g_variant_unref (value);
-          g_free (key);
-          g_free (param_str);
+            g_key_file_set_value (msetfile, group, key, param_str);
+            if (save_comment)
+            {
+              const gchar *blurb = g_param_spec_get_blurb (param_spec);
+              if (blurb != NULL && blurb[0] != 0)
+              {
+                gchar *desc = ncm_cfg_string_to_comment (blurb);
+                if (!g_key_file_set_comment (msetfile, group, key, desc, &error))
+                  g_error ("ncm_mset_save: %s", error->message);
+                g_free (desc);
+              }
+            }
+
+            g_variant_unref (value);
+            g_free (key);
+            g_free (param_str);
+          }
         }
       }
 
+      g_free (obj_name);
       g_variant_unref (params);
       g_variant_unref (model_var);
     }
@@ -2507,6 +2649,7 @@ ncm_mset_save (NcmMSet *mset, NcmSerialize *ser, const gchar *filename, gboolean
   }
 }
 
+
 /**
  * ncm_mset_load:
  * @filename: mset filename
@@ -2525,8 +2668,11 @@ ncm_mset_load (const gchar *filename, NcmSerialize *ser)
   gchar **groups = NULL;
   gsize ngroups = 0;
   gboolean valid_map = FALSE;
+  GPtrArray *submodel_array = g_ptr_array_new ();
   guint i;
 
+  g_ptr_array_set_free_func (submodel_array, (GDestroyNotify) ncm_model_free);
+
   if (!g_key_file_load_from_file (msetfile, filename, G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, &error))
   {
     g_error ("ncm_mset_load: Invalid mset configuration file: %s %s", filename, error->message);
@@ -2554,12 +2700,12 @@ ncm_mset_load (const gchar *filename, NcmSerialize *ser)
     GString *obj_ser = g_string_sized_new (200);
     gchar *ns = g_strdup (groups[i]);
     gchar *twopoints = g_strrstr (ns, ":");
-    guint submodel = 0;
+    guint stackpos = 0;
 
     if (twopoints != NULL)
     {
       *twopoints = '\0';
-      submodel = atoi(++twopoints);
+      stackpos = atoi (++twopoints);
     }
 
     if (!g_key_file_has_key (msetfile, groups[i], ns, &error))
@@ -2575,7 +2721,7 @@ ncm_mset_load (const gchar *filename, NcmSerialize *ser)
       if (strlen (obj_type) < 5)
       {
         gchar *endptr = NULL;
-        guint submodel_orig = g_ascii_strtoll (obj_type, &endptr, 10);
+        guint stackpos_orig = g_ascii_strtoll (obj_type, &endptr, 10);
 
         g_assert (endptr != NULL);
 
@@ -2584,9 +2730,9 @@ ncm_mset_load (const gchar *filename, NcmSerialize *ser)
           NcmModelID id = ncm_mset_get_id_by_ns (ns);
           g_assert_cmpint (id, >, -1);
           {
-            NcmModel *model = ncm_mset_peek_pos (mset, id, submodel_orig);
+            NcmModel *model = ncm_mset_peek_pos (mset, id, stackpos_orig);
             g_assert (model != NULL);
-            ncm_mset_set_pos (mset, model, submodel);
+            ncm_mset_set_pos (mset, model, stackpos);
           }
           continue;
         }
@@ -2607,13 +2753,20 @@ ncm_mset_load (const gchar *filename, NcmSerialize *ser)
         g_error ("ncm_mset_load: %s", error->message);
       for (j = 0; j < nkeys; j++)
       {
-        gchar *propval = g_key_file_get_value (msetfile, groups[i], keys[j], &error);
-        if (error != NULL)
-          g_error ("ncm_mset_load: %s", error->message);
-        g_string_append_printf (obj_ser, "\'%s\':<%s>", keys[j], propval);
-        g_free (propval);
-        if (j + 1 != nkeys)
-          g_string_append (obj_ser, ", ");
+        if (strcmp (keys[j], "submodel-array") == 0)
+        {
+          g_error ("ncm_mset_load: serialized version of mset models cannot contain the submodel-array property.");
+        }
+        else
+        {
+          gchar *propval = g_key_file_get_value (msetfile, groups[i], keys[j], &error);
+          if (error != NULL)
+            g_error ("ncm_mset_load: %s", error->message);
+          g_string_append_printf (obj_ser, "\'%s\':<%s>", keys[j], propval);
+          g_free (propval);
+          if (j + 1 != nkeys)
+            g_string_append (obj_ser, ", ");
+        }
       }
       g_string_append (obj_ser, "}}");
       g_strfreev (keys);
@@ -2622,18 +2775,49 @@ ncm_mset_load (const gchar *filename, NcmSerialize *ser)
     {
       GObject *obj = ncm_serialize_from_string (ser, obj_ser->str);
       g_assert (NCM_IS_MODEL (obj));
-      if (ncm_mset_exists_pos (mset, NCM_MODEL (obj), submodel))
+      if (ncm_mset_exists_pos (mset, NCM_MODEL (obj), stackpos))
         g_error ("ncm_mset_load: a model ``%s'' already exists in NcmMSet.", obj_ser->str);
-      ncm_mset_set_pos (mset, NCM_MODEL (obj), submodel);
+      if (ncm_model_is_submodel (NCM_MODEL (obj)))
+      {
+        NcmModel *submodel = ncm_model_ref (NCM_MODEL (obj));
+        g_ptr_array_add (submodel_array, submodel);
+      }
+      else
+      {
+        ncm_mset_set_pos (mset, NCM_MODEL (obj), stackpos);
+      }
       ncm_model_free (NCM_MODEL (obj));
     }
     g_string_free (obj_ser, TRUE);
+    g_free (ns);
   }
 
+  for (i = 0; i < submodel_array->len; i++)
+  {
+    NcmModel *submodel = g_ptr_array_index (submodel_array, i);
+    NcmModelID mid = ncm_model_main_model (submodel);
+    NcmModel *mainmodel = ncm_mset_peek (mset, mid);
+
+    g_assert_cmpint (mid, >=, 0);
+    if (mainmodel == NULL)
+    {
+      g_error ("ncm_mset_load: cannot add submodel `%s', main model `%s' not found.", 
+               G_OBJECT_TYPE_NAME (submodel),
+               ncm_mset_get_ns_by_id (mid));
+    }
+    else
+    {
+      ncm_model_add_submodel (mainmodel, submodel);
+      _ncm_mset_set_pos_intern (mset, submodel, 0);
+    }
+  }
+  
   g_key_file_unref (msetfile);
   g_strfreev (groups);
+    
   if (valid_map)
     ncm_mset_prepare_fparam_map (mset);
 
+  g_ptr_array_unref (submodel_array);
   return mset;
 }
