@@ -43,11 +43,17 @@ typedef struct _NcmSerializeClass NcmSerializeClass;
 typedef struct _NcmSerialize NcmSerialize;
 typedef struct _NcmSerializePrivate NcmSerializePrivate;
 
+struct _NcmSerializeClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+};
+
 /**
  * NcmSerializeOpt:
  * @NCM_SERIALIZE_OPT_NONE: Use default serialization.
- * @NCM_SERIALIZE_OPT_AUTOSAVE_SER: Whether to automatically save objects serializations.
- * @NCM_SERIALIZE_OPT_AUTONAME_SER: Whether to automatically include objects created from saved serialization in the named instances.
+ * @NCM_SERIALIZE_OPT_AUTOSAVE_SER: Whether to automatically include named deserialized objects in the named instances.
+ * @NCM_SERIALIZE_OPT_AUTONAME_SER: Whether to automatically name objects on serialization.
  * @NCM_SERIALIZE_OPT_CLEAN_DUP: Combination of NCM_SERIALIZE_OPT_AUTOSAVE_SER and NCM_SERIALIZE_OPT_AUTONAME_SER
  *
  * Options for serialization.
@@ -75,12 +81,6 @@ struct _NcmSerialize
   guint autosave_count;
 };
 
-struct _NcmSerializeClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-};
-
 GType ncm_serialize_get_type (void) G_GNUC_CONST;
 
 NcmSerialize *ncm_serialize_new (NcmSerializeOpt sopt);
@@ -96,9 +96,13 @@ void ncm_serialize_log_stats (NcmSerialize *ser);
 gboolean ncm_serialize_contain_instance (NcmSerialize *ser, gpointer obj);
 gboolean ncm_serialize_contain_name (NcmSerialize *ser, const gchar *name);
 guint ncm_serialize_count_instances (NcmSerialize *ser);
+gpointer ncm_serialize_peek_by_name (NcmSerialize *ser, const gchar *name);
 gpointer ncm_serialize_get_by_name (NcmSerialize *ser, const gchar *name);
+
 gchar *ncm_serialize_peek_name (NcmSerialize *ser, gpointer obj);
 void ncm_serialize_set (NcmSerialize *ser, gpointer obj, const gchar *name, gboolean overwrite);
+void ncm_serialize_unset (NcmSerialize *ser, gpointer obj);
+void ncm_serialize_remove_ser (NcmSerialize *ser, gpointer obj);
 gboolean ncm_serialize_is_named (NcmSerialize *ser, const gchar *serobj, gchar **name);
 
 void ncm_serialize_set_property (NcmSerialize *ser, GObject *obj, const gchar *prop_str);
@@ -128,6 +132,8 @@ guint ncm_serialize_global_count_instances (void);
 gpointer ncm_serialize_global_get_by_name (const gchar *name);
 gchar *ncm_serialize_global_global_peek_name (gpointer obj);
 void ncm_serialize_global_set (gpointer obj, const gchar *name, gboolean overwrite);
+void ncm_serialize_global_unset (gpointer obj);
+void ncm_serialize_global_remove_ser (gpointer obj);
 gboolean ncm_serialize_global_is_named (const gchar *serobj, gchar **name);
 
 void ncm_serialize_global_set_property (GObject *obj, const gchar *prop_str);
@@ -150,8 +156,8 @@ GObject *ncm_serialize_global_dup_obj (GObject *obj);
 #define NCM_SERIALIZE_VECTOR_TYPE "ad"
 #define NCM_SERIALIZE_MATRIX_TYPE "aad"
 #define NCM_SERIALIZE_STRV_TYPE "as"
-#define NCM_SERIALIZE_AUTOSAVE_NAME "auto:saved:"
-#define NCM_SERIALIZE_AUTOSAVE_NFORMAT "%04u"
+#define NCM_SERIALIZE_AUTOSAVE_NAME "S"
+#define NCM_SERIALIZE_AUTOSAVE_NFORMAT "%u"
 
 G_END_DECLS
 
