@@ -43,18 +43,6 @@ G_BEGIN_DECLS
 typedef struct _NcmSplineClass NcmSplineClass;
 typedef struct _NcmSpline NcmSpline;
 
-struct _NcmSpline
-{
-  /*< private >*/
-  GObject parent_instance;
-  gsize len;
-  NcmVector *xv;
-  NcmVector *yv;
-  gsl_interp_accel *acc;
-  gboolean init;
-  gboolean empty;
-};
-
 struct _NcmSplineClass
 {
   /*< private >*/
@@ -72,6 +60,18 @@ struct _NcmSplineClass
   NcmSpline *(*copy_empty) (const NcmSpline *s);
 };
 
+struct _NcmSpline
+{
+  /*< private >*/
+  GObject parent_instance;
+  gsize len;
+  NcmVector *xv;
+  NcmVector *yv;
+  gsl_interp_accel *acc;
+  gboolean init;
+  gboolean empty;
+};
+
 GType ncm_spline_get_type (void) G_GNUC_CONST;
 
 NcmSpline *ncm_spline_copy_empty (const NcmSpline *s);
@@ -83,13 +83,18 @@ NcmSpline *ncm_spline_new_data (const NcmSpline *s, gdouble *x, gdouble *y, cons
 NcmSpline *ncm_spline_set (NcmSpline *s, NcmVector *xv, NcmVector *yv, gboolean init);
 NcmSpline *ncm_spline_ref (NcmSpline *s);
 
+void ncm_spline_acc (NcmSpline *s, gboolean enable);
+
+void ncm_spline_set_len (NcmSpline *s, guint len);
 void ncm_spline_set_xv (NcmSpline *s, NcmVector *xv, gboolean init);
 void ncm_spline_set_yv (NcmSpline *s, NcmVector *yv, gboolean init);
 void ncm_spline_set_array (NcmSpline *s, GArray *x, GArray *y, gboolean init);
 void ncm_spline_set_data_static (NcmSpline *s, gdouble *x, gdouble *y, gsize len, gboolean init);
 
+guint ncm_spline_get_len (NcmSpline *s);
 NcmVector *ncm_spline_get_xv (NcmSpline *s);
 NcmVector *ncm_spline_get_yv (NcmSpline *s);
+void ncm_spline_get_bounds (NcmSpline *s, gdouble *lb, gdouble *ub);
 
 void ncm_spline_free (NcmSpline *s);
 void ncm_spline_clear (NcmSpline **s);
