@@ -85,6 +85,9 @@
 #include <math.h>
 #include <gsl/gsl_math.h>
 
+#include "math/gsl_rstat.h"
+#include "math/rquantile.c"
+
 enum
 {
   PROP_0,
@@ -118,6 +121,9 @@ ncm_stats_vec_init (NcmStatsVec *svec)
   svec->saved_x  = NULL;
   svec->save_x   = FALSE;
 
+  svec->q_array  = g_ptr_array_new ();
+  g_ptr_array_set_free_func (svec->q_array, (GDestroyNotify) gsl_rstat_quantile_free);
+  
 #ifdef NUMCOSMO_HAVE_FFTW3
   svec->fft_size       = 0;
   svec->fft_plan_size  = 0;
@@ -137,12 +143,16 @@ _ncm_stats_vec_dispose (GObject *object)
   ncm_vector_clear (&svec->var);
   ncm_matrix_clear (&svec->cov);
   ncm_matrix_clear (&svec->real_cov);
+
   if (svec->saved_x != NULL)
   {
     g_ptr_array_unref (svec->saved_x);
     svec->saved_x = NULL;
     svec->save_x = FALSE;
   }
+
+  g_clear_pointer (&svec->q_array, g_ptr_array_unref);
+
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_stats_vec_parent_class)->dispose (object);
 }
@@ -308,9 +318,9 @@ ncm_stats_vec_class_init (NcmStatsVecClass *klass)
 
 /**
  * ncm_stats_vec_new:
- * @len: number of random variables.
- * @t: type of statistics to be calculated.
- * @save_x: whenever to save each vector x.
+ * @len: number of random variables
+ * @t: type of statistics to be calculated
+ * @save_x: whenever to save each vector x
  *
  * Creates a new #NcmStatsVec.
  *
@@ -329,7 +339,7 @@ ncm_stats_vec_new (guint len, NcmStatsVecType t, gboolean save_x)
 
 /**
  * ncm_stats_vec_ref:
- * @svec: a #NcmStatsVec.
+ * @svec: a #NcmStatsVec
  *
  * Increase the reference of @svec by one.
  *
@@ -343,7 +353,7 @@ ncm_stats_vec_ref (NcmStatsVec *svec)
 
 /**
  * ncm_stats_vec_free:
- * @svec: a #NcmStatsVec.
+ * @svec: a #NcmStatsVec
  *
  * Decrease the reference count of @svec by one.
  *
@@ -356,7 +366,7 @@ ncm_stats_vec_free (NcmStatsVec *svec)
 
 /**
  * ncm_stats_vec_clear:
- * @svec: a #NcmStatsVec.
+ * @svec: a #NcmStatsVec
  *
  * Decrease the reference count of @svec by one, and sets the pointer *svec to
  * NULL.
@@ -415,6 +425,20 @@ ncm_stats_vec_reset (NcmStatsVec *svec, gboolean rm_saved)
       g_assert_not_reached ();
       break;
   }
+
+  if (svec->q_array->len == svec->len)
+  {
+    guint i;
+    const gdouble p = ((gsl_rstat_quantile_workspace *)g_ptr_array_index (svec->q_array, 0))->p;
+
+    g_ptr_array_set_size (svec->q_array, 0);
+    
+    for (i = 0; i < svec->len; i++)
+    {
+      gsl_rstat_quantile_workspace *qws_i = gsl_rstat_quantile_alloc (p);
+      g_ptr_array_add (svec->q_array, qws_i);
+    }
+  }
 }
 
 static void
@@ -521,12 +545,24 @@ _ncm_stats_vec_update_from_vec_weight (NcmStatsVec *svec, const gdouble w, NcmVe
   svec->weight = curweight;
   svec->weight2 += w * w;
   svec->bias_wt = 1.0 / (svec->weight - svec->weight2 / svec->weight);
+
+  if (svec->q_array->len == svec->len)
+  {
+    guint i;
+
+    for (i = 0; i < svec->len; i++)
+    {
+      const gdouble x_i = ncm_vector_fast_get (x, i);
+      gsl_rstat_quantile_workspace *qws_i = g_ptr_array_index (svec->q_array, i);
+      gsl_rstat_quantile_add (x_i, qws_i);
+    }
+  }
 }
 
 /**
  * ncm_stats_vec_update_weight:
- * @svec: a #NcmStatsVec.
- * @w: The statistical weight.
+ * @svec: a #NcmStatsVec
+ * @w: The statistical weight
  *
  * Updates the statistics using @svec->x set in @svec and @weight, then reset
  * @svec->x to zero.
@@ -545,12 +581,13 @@ ncm_stats_vec_update_weight (NcmStatsVec *svec, const gdouble w)
 }
 
 /**
- * ncm_stats_vec_append:
- * @svec: a #NcmStatsVec.
- * @x: a #NcmVector to be added.
- * @dup: a #gboolean.
+ * ncm_stats_vec_append_weight:
+ * @svec: a #NcmStatsVec
+ * @x: a #NcmVector to be added
+ * @w: the weight of @x
+ * @dup: a boolean
  *
- * Appends and updates the statistics using the vector @x #NcmVector of same
+ * Appends and updates the statistics using weight @w for the vector @x #NcmVector of same
  * size #NcmStatsVec:length and with continuous allocation. i.e., NcmVector:stride == 1.
  *
  * If @svec was created with save_x TRUE, the paramenter @dup determines if the vector
@@ -558,9 +595,9 @@ ncm_stats_vec_update_weight (NcmStatsVec *svec, const gdouble w)
  *
  */
 void
-ncm_stats_vec_append (NcmStatsVec *svec, NcmVector *x, gboolean dup)
+ncm_stats_vec_append_weight (NcmStatsVec *svec, NcmVector *x, gdouble w, gboolean dup)
 {
-  _ncm_stats_vec_update_from_vec_weight (svec, 1.0, x);
+  _ncm_stats_vec_update_from_vec_weight (svec, w, x);
   if (svec->save_x)
   {
     if (dup)
@@ -571,12 +608,13 @@ ncm_stats_vec_append (NcmStatsVec *svec, NcmVector *x, gboolean dup)
 }
 
 /**
- * ncm_stats_vec_prepend:
- * @svec: a #NcmStatsVec.
- * @x: a #NcmVector to be added.
- * @dup: a boolean.
+ * ncm_stats_vec_prepend_weight:
+ * @svec: a #NcmStatsVec
+ * @x: a #NcmVector to be added
+ * @w: the weight of @x
+ * @dup: a boolean
  *
- * Prepends and updates the statistics using the vector @x and weight == 1.0.
+ * Prepends and updates the statistics using the vector @x and weight @w.
  * It assumes that #NcmVector is of same size #NcmStatsVec:length and
  * with continuous allocation. i.e., NcmVector:stride == 1.
  *
@@ -585,9 +623,9 @@ ncm_stats_vec_append (NcmStatsVec *svec, NcmVector *x, gboolean dup)
  *
  */
 void
-ncm_stats_vec_prepend (NcmStatsVec *svec, NcmVector *x, gboolean dup)
+ncm_stats_vec_prepend_weight (NcmStatsVec *svec, NcmVector *x, gdouble w, gboolean dup)
 {
-  _ncm_stats_vec_update_from_vec_weight (svec, 1.0, x);
+  _ncm_stats_vec_update_from_vec_weight (svec, w, x);
 
   if (svec->save_x)
   {
@@ -605,11 +643,50 @@ ncm_stats_vec_prepend (NcmStatsVec *svec, NcmVector *x, gboolean dup)
   }
 }
 
+/**
+ * ncm_stats_vec_append:
+ * @svec: a #NcmStatsVec
+ * @x: a #NcmVector to be added
+ * @dup: a boolean
+ *
+ * Appends and updates the statistics using weight 1.0 for the vector @x #NcmVector of same
+ * size #NcmStatsVec:length and with continuous allocation. i.e., NcmVector:stride == 1.
+ *
+ * If @svec was created with save_x TRUE, the paramenter @dup determines if the vector
+ * @x will be duplicated or if just a reference for @x will be saved.
+ *
+ */
+void
+ncm_stats_vec_append (NcmStatsVec *svec, NcmVector *x, gboolean dup)
+{
+  ncm_stats_vec_append_weight (svec, x, 1.0, dup);
+}
+
+/**
+ * ncm_stats_vec_prepend:
+ * @svec: a #NcmStatsVec
+ * @x: a #NcmVector to be added
+ * @dup: a boolean
+ *
+ * Prepends and updates the statistics using the vector @x and weight 1.0.
+ * It assumes that #NcmVector is of same size #NcmStatsVec:length and
+ * with continuous allocation. i.e., NcmVector:stride == 1.
+ *
+ * If @svec was created with save_x TRUE, the paramenter @dup determines if the vector
+ * will be duplicated or if just a reference for @x will be saved.
+ *
+ */
+void
+ncm_stats_vec_prepend (NcmStatsVec *svec, NcmVector *x, gboolean dup)
+{
+  ncm_stats_vec_prepend_weight (svec, x, 1.0, dup);
+}
+
 /**
  * ncm_stats_vec_append_data:
- * @svec: a #NcmStatsVec.
- * @data: (element-type NcmVector): a #GPtrArray containing #NcmVector s to be added.
- * @dup: a #gboolean.
+ * @svec: a #NcmStatsVec
+ * @data: (element-type NcmVector): a #GPtrArray containing #NcmVector s to be added
+ * @dup: a boolean
  *
  * Appends and updates the statistics using the data contained in @data and weight == 1.0.
  * It assumes that each element of @data is a #NcmVector of same size #NcmStatsVec:length and
@@ -641,9 +718,9 @@ ncm_stats_vec_append_data (NcmStatsVec *svec, GPtrArray *data, gboolean dup)
 
 /**
  * ncm_stats_vec_prepend_data:
- * @svec: a #NcmStatsVec.
- * @data: (element-type NcmVector): a #GPtrArray containing #NcmVector s to be added.
- * @dup: a boolean.
+ * @svec: a #NcmStatsVec
+ * @data: (element-type NcmVector): a #GPtrArray containing #NcmVector s to be added
+ * @dup: a boolean
  *
  * Prepends and updates the statistics using the data contained in @data and weight == 1.0.
  * It assumes that each element of @data is a #NcmVector of same size #NcmStatsVec:length and
@@ -683,6 +760,113 @@ ncm_stats_vec_prepend_data (NcmStatsVec *svec, GPtrArray *data, gboolean dup)
   }
 }
 
+/**
+ * ncm_stats_vec_enable_quantile:
+ * @svec: a #NcmStatsVec
+ * @p: double $\in (0, 1)$
+ * 
+ * Enables quantile calculation, it will calculate the $p$
+ * quantile. Warning, it does not support weighted samples, the results
+ * will disconsider the weights.
+ * 
+ */
+void 
+ncm_stats_vec_enable_quantile (NcmStatsVec *svec, gdouble p)
+{
+  g_assert_cmpfloat (p, >, 0.0);
+  g_assert_cmpfloat (p, <, 1.0);
+
+  {
+    guint i;
+    g_ptr_array_set_size (svec->q_array, 0);
+    
+    for (i = 0; i < svec->len; i++)
+    {
+      gsl_rstat_quantile_workspace *qws_i = gsl_rstat_quantile_alloc (p);
+      g_ptr_array_add (svec->q_array, qws_i);
+    }
+  }
+
+  if (svec->nitens > 0)
+  {
+    if (!svec->save_x)
+    {
+      g_warning ("ncm_stats_vec_enable_quantile: Enabling quantile calculation in a non-empty NcmStatsVec,"
+                 " all previous data will be ignored in the quantile.");
+    }
+    else
+    {
+      guint i;
+      for (i = 0; i < svec->saved_x->len; i++)
+      {
+        NcmVector *x = g_ptr_array_index (svec->saved_x, i);
+        guint j;
+
+        for (j = 0; j < svec->len; j++)
+        {
+          const gdouble x_j = ncm_vector_fast_get (x, j);
+          gsl_rstat_quantile_workspace *qws_j = g_ptr_array_index (svec->q_array, j);
+          gsl_rstat_quantile_add (x_j, qws_j);
+        }
+      }
+    }
+  }
+}
+
+/**
+ * ncm_stats_vec_disable_quantile:
+ * @svec: a #NcmStatsVec
+ * 
+ * Disables quantile calculation.
+ * 
+ */
+void 
+ncm_stats_vec_disable_quantile (NcmStatsVec *svec)
+{
+  g_ptr_array_set_size (svec->q_array, 0);
+}
+
+/**
+ * ncm_stats_vec_get_quantile:
+ * @svec: a #NcmStatsVec
+ * @i: a variable index
+ * 
+ * Returns the current estimate of the quantile initialized
+ * through ncm_stats_vec_enable_quantile().
+ * 
+ * Returns: the current estimate of the quantile.
+ */
+gdouble
+ncm_stats_vec_get_quantile (NcmStatsVec *svec, guint i)
+{
+  g_assert_cmpuint (i, <, svec->q_array->len);
+  
+  return gsl_rstat_quantile_get (g_ptr_array_index (svec->q_array, i));
+}
+
+/**
+ * ncm_stats_vec_get_quantile_spread:
+ * @svec: a #NcmStatsVec
+ * @i: a variable index
+ * 
+ * Returns the current estimate of the quantile spread, from the
+ * probability $p$ initialized through ncm_stats_vec_enable_quantile(),
+ * i.e., it returns the difference between $(p + 1)/2$ quantile
+ * and the $p/2$. For example, if $p = 0.5$ then it returns the
+ * interquartile range.
+ * 
+ * Returns: the current estimate of the quantile spread.
+ */
+gdouble
+ncm_stats_vec_get_quantile_spread (NcmStatsVec *svec, guint i)
+{
+  g_assert_cmpuint (i, <, svec->q_array->len);
+  {
+    gsl_rstat_quantile_workspace *qws_i = g_ptr_array_index (svec->q_array, i);
+    return qws_i->q[3] - qws_i->q[1];
+  }
+}
+
 static void
 _ncm_stats_vec_get_autocorr_alloc (NcmStatsVec *svec, guint size)
 {
@@ -710,10 +894,14 @@ _ncm_stats_vec_get_autocorr_alloc (NcmStatsVec *svec, guint size)
     g_clear_pointer (&svec->param_r2c, fftw_destroy_plan);
 
     /*g_debug ("# _ncm_stats_vec_get_autocorr_alloc: calculating wisdown %u\n", effsize);*/
-    ncm_cfg_load_fftw_wisdom ("NcmStatsVec:autocorr");
+    ncm_cfg_load_fftw_wisdom ("ncm_stats_vec_autocorr_%u", effsize);
+
+    ncm_cfg_lock_plan_fftw ();
     svec->param_r2c = fftw_plan_dft_r2c_1d (effsize, svec->param_data, svec->param_fft, fftw_default_flags | FFTW_DESTROY_INPUT);
     svec->param_c2r = fftw_plan_dft_c2r_1d (effsize, svec->param_fft, svec->param_data, fftw_default_flags | FFTW_DESTROY_INPUT);
-    ncm_cfg_save_fftw_wisdom ("NcmStatsVec:autocorr");
+    ncm_cfg_unlock_plan_fftw ();
+
+    ncm_cfg_save_fftw_wisdom ("ncm_stats_vec_autocorr_%u", effsize);
     svec->fft_plan_size = effsize;
     /*g_debug ("# _ncm_stats_vec_get_autocorr_alloc: calculated  wisdown %u\n", effsize);*/
   }
@@ -775,8 +963,8 @@ _ncm_stats_vec_get_autocov (NcmStatsVec *svec, guint p, guint subsample, guint p
 
 /**
  * ncm_stats_vec_get_autocorr:
- * @svec: a #NcmStatsVec.
- * @p: parameter id.
+ * @svec: a #NcmStatsVec
+ * @p: parameter id
  *
  * Calculates the autocorrelation vector, the j-th element represent
  * the selfcorrelation with lag-j.
@@ -804,9 +992,9 @@ ncm_stats_vec_get_autocorr (NcmStatsVec *svec, guint p)
 
 /**
  * ncm_stats_vec_get_subsample_autocorr:
- * @svec: a #NcmStatsVec.
- * @p: parameter id.
- * @subsample: size of the subsample (>0).
+ * @svec: a #NcmStatsVec
+ * @p: parameter id
+ * @subsample: size of the subsample ($>0$)
  *
  * Calculates the autocorrelation vector, the j-th element represent
  * the selfcorrelation with lag-j using the @subsample parameter.
@@ -835,10 +1023,10 @@ ncm_stats_vec_get_subsample_autocorr (NcmStatsVec *svec, guint p, guint subsampl
 
 /**
  * ncm_stats_vec_get_autocorr_tau:
- * @svec: a #NcmStatsVec.
- * @p: parameter id.
- * @max_lag: max lag in the computation.
- * @min_rho: minimum autocorrelation to be considered in the sum.
+ * @svec: a #NcmStatsVec
+ * @p: parameter id
+ * @max_lag: max lag in the computation
+ * @min_rho: minimum autocorrelation to be considered in the sum
  *
  * Calculates the integrated autocorrelation time for the parameter @p
  * using all rows of data.
@@ -879,11 +1067,11 @@ ncm_stats_vec_get_autocorr_tau (NcmStatsVec *svec, guint p, guint max_lag, const
 
 /**
  * ncm_stats_vec_get_subsample_autocorr_tau:
- * @svec: a #NcmStatsVec.
- * @p: parameter id.
- * @subsample: size of the subsample (>0).
- * @max_lag: max lag in the computation.
- * @min_rho: minimum autocorrelation to be considered in the sum.
+ * @svec: a #NcmStatsVec
+ * @p: parameter id
+ * @subsample: size of the subsample ($>0$)
+ * @max_lag: max lag in the computation
+ * @min_rho: minimum autocorrelation to be considered in the sum
  *
  * Calculates the integrated autocorrelation time for the parameter @p
  * using the @subsample parameter.
@@ -922,7 +1110,7 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
 
 /**
  * ncm_stats_vec_peek_x:
- * @svec: a #NcmStatsVec.
+ * @svec: a #NcmStatsVec
  *
  * Returns the vector containing the current value of the random variables.
  *
@@ -930,17 +1118,17 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_set:
- * @svec: a #NcmStatsVec.
- * @i: a variable index.
- * @x_i: the value of the @i-th variable.
+ * @svec: a #NcmStatsVec
+ * @i: a variable index
+ * @x_i: the value of the @i-th variable
  *
  * Sets the value of the current @i-th random variable to @x_i.
  *
  */
 /**
  * ncm_stats_vec_get:
- * @svec: a #NcmStatsVec.
- * @i: a variable index.
+ * @svec: a #NcmStatsVec
+ * @i: a variable index
  *
  * Returns the value of the current @i-th random variable.
  *
@@ -955,8 +1143,8 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_get_mean:
- * @svec: a #NcmStatsVec.
- * @i: a variable index.
+ * @svec: a #NcmStatsVec
+ * @i: a variable index
  *
  * Return the current value of the variable mean, i.e., $\bar{x}_n$.
  *
@@ -964,8 +1152,8 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_get_var:
- * @svec: a #NcmStatsVec.
- * @i: a variable index.
+ * @svec: a #NcmStatsVec
+ * @i: a variable index
  *
  * Return the current value of the variable variance, i.e., $Var_n$.
  *
@@ -973,8 +1161,8 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_get_sd:
- * @svec: a #NcmStatsVec.
- * @i: a variable index.
+ * @svec: a #NcmStatsVec
+ * @i: a variable index
  *
  * Return the current value of the variable standard deviation,
  * i.e., $\sigma_n \equiv sqrt (Var_n)$.
@@ -983,9 +1171,9 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_get_cov:
- * @svec: a #NcmStatsVec.
- * @i: a variable index.
- * @j: a variable index.
+ * @svec: a #NcmStatsVec
+ * @i: a variable index
+ * @j: a variable index
  *
  * Return the current value of the variance between the @i-th and the @j-th
  * variables, i.e., $Cov_{ij}$.
@@ -994,9 +1182,9 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_get_cor:
- * @svec: a #NcmStatsVec.
- * @i: a variable index.
- * @j: a variable index.
+ * @svec: a #NcmStatsVec
+ * @i: a variable index
+ * @j: a variable index
  *
  * Return the current value of the correlation between the @i-th and the @j-th
  * variables, i.e., $$Cor_{ij} \equiv \frac{Cov_{ij}}{\sigma_i\sigma_j}.$$
@@ -1005,7 +1193,7 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_get_weight:
- * @svec: a #NcmStatsVec.
+ * @svec: a #NcmStatsVec
  *
  * Return the current value of the weight, for non-weighted means this is simply
  * the number of elements.
@@ -1014,18 +1202,26 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_get_mean_vector:
- * @svec: a #NcmStatsVec.
- * @mean: a #NcmVector.
- * @offset: first parameter index.
+ * @svec: a #NcmStatsVec
+ * @mean: a #NcmVector
+ * @offset: first parameter index
  *
  * Copy the current value of the means to the vector @mean starting from parameter @offset.
  *
  */
+/**
+ * ncm_stats_vec_peek_mean:
+ * @svec: a #NcmStatsVec
+ * 
+ * Gets the local mean vector.
+ * 
+ * Returns: (transfer none): the internal mean #NcmVector.
+ */
 /**
  * ncm_stats_vec_get_cov_matrix:
- * @svec: a #NcmStatsVec.
- * @m: a #NcmMatrix.
- * @offset: first parameter index.
+ * @svec: a #NcmStatsVec
+ * @m: a #NcmMatrix
+ * @offset: first parameter index
  *
  * Copy the current value of the correlation between the variables to the
  * matrix @m starting from paramenter @offset.
@@ -1033,8 +1229,8 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_peek_cov_matrix:
- * @svec: a #NcmStatsVec.
- * @offset: first parameter index.
+ * @svec: a #NcmStatsVec
+ * @offset: first parameter index
  *
  * Gets the internal covariance matrix starting from paramenter @offset.
  * This is the internal matrix of @svec and can change with further
@@ -1044,7 +1240,7 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_nrows:
- * @svec: a #NcmStatsVec.
+ * @svec: a #NcmStatsVec
  *
  * Gets the number of saved rows, this function fails if the object
  * was not created with save_x == TRUE;
@@ -1053,8 +1249,8 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_peek_row:
- * @svec: a #NcmStatsVec.
- * @i: the row's index.
+ * @svec: a #NcmStatsVec
+ * @i: the row's index
  *
  * The i-th data row used in the statistics, this function fails if the object
  * was not created with save_x == TRUE;
@@ -1063,9 +1259,9 @@ ncm_stats_vec_get_subsample_autocorr_tau (NcmStatsVec *svec, guint p, guint subs
  */
 /**
  * ncm_stats_vec_get_param_at:
- * @svec: a #NcmStatsVec.
- * @i: the row's index.
- * @p: the parameter's index.
+ * @svec: a #NcmStatsVec
+ * @i: the row's index
+ * @p: the parameter's index
  *
  * Gets the p-th parameter in the i-th data row used in the statistics, this
  * function fails if the object was not created with save_x == TRUE;
