@@ -54,6 +54,12 @@ G_BEGIN_DECLS
 typedef struct _NcmStatsVecClass NcmStatsVecClass;
 typedef struct _NcmStatsVec NcmStatsVec;
 
+struct _NcmStatsVecClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+};
+
 /**
  * NcmStatsVecType:
  * @NCM_STATS_VEC_MEAN: Calculates mean only.
@@ -93,6 +99,7 @@ struct _NcmStatsVec
   NcmMatrix *cov;
   NcmMatrix *real_cov;
   GPtrArray *saved_x;
+  GPtrArray *q_array;
 #ifdef NUMCOSMO_HAVE_FFTW3
   guint fft_size;
   guint fft_plan_size;
@@ -103,12 +110,6 @@ struct _NcmStatsVec
 #endif /* NUMCOSMO_HAVE_FFTW3 */
 };
 
-struct _NcmStatsVecClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-};
-
 GType ncm_stats_vec_get_type (void) G_GNUC_CONST;
 
 NcmStatsVec *ncm_stats_vec_new (guint len, NcmStatsVecType t, gboolean save_x);
@@ -119,11 +120,19 @@ void ncm_stats_vec_clear (NcmStatsVec **svec);
 void ncm_stats_vec_reset (NcmStatsVec *svec, gboolean rm_saved);
 void ncm_stats_vec_update_weight (NcmStatsVec *svec, gdouble w);
 
+void ncm_stats_vec_append_weight (NcmStatsVec *svec, NcmVector *x, gdouble w, gboolean dup);
+void ncm_stats_vec_prepend_weight (NcmStatsVec *svec, NcmVector *x, gdouble w, gboolean dup);
+
 void ncm_stats_vec_append (NcmStatsVec *svec, NcmVector *x, gboolean dup);
 void ncm_stats_vec_prepend (NcmStatsVec *svec, NcmVector *x, gboolean dup);
 void ncm_stats_vec_append_data (NcmStatsVec *svec, GPtrArray *data, gboolean dup);
 void ncm_stats_vec_prepend_data (NcmStatsVec *svec, GPtrArray *data, gboolean dup);
 
+void ncm_stats_vec_enable_quantile (NcmStatsVec *svec, gdouble p);
+void ncm_stats_vec_disable_quantile (NcmStatsVec *svec);
+gdouble ncm_stats_vec_get_quantile (NcmStatsVec *svec, guint i);
+gdouble ncm_stats_vec_get_quantile_spread (NcmStatsVec *svec, guint i);
+
 NcmVector *ncm_stats_vec_get_autocorr (NcmStatsVec *svec, guint p);
 NcmVector *ncm_stats_vec_get_subsample_autocorr (NcmStatsVec *svec, guint p, guint subsample);
 gdouble ncm_stats_vec_get_autocorr_tau (NcmStatsVec *svec, guint p, guint max_lag, const gdouble min_rho);
@@ -140,6 +149,7 @@ G_INLINE_FUNC gdouble ncm_stats_vec_get_cov (NcmStatsVec *svec, guint i, guint j
 G_INLINE_FUNC gdouble ncm_stats_vec_get_cor (NcmStatsVec *svec, guint i, guint j);
 G_INLINE_FUNC gdouble ncm_stats_vec_get_weight (NcmStatsVec *svec);
 G_INLINE_FUNC void ncm_stats_vec_get_mean_vector (NcmStatsVec *svec, NcmVector *mean, guint offset);
+G_INLINE_FUNC NcmVector *ncm_stats_vec_peek_mean (NcmStatsVec *svec);
 G_INLINE_FUNC void ncm_stats_vec_get_cov_matrix (NcmStatsVec *svec, NcmMatrix *m, guint offset);
 G_INLINE_FUNC NcmMatrix *ncm_stats_vec_peek_cov_matrix (NcmStatsVec *svec, guint offset);
 G_INLINE_FUNC guint ncm_stats_vec_nrows (NcmStatsVec *svec);
@@ -232,6 +242,12 @@ ncm_stats_vec_get_mean_vector (NcmStatsVec *svec, NcmVector *x, guint offset)
   ncm_vector_memcpy2 (x, svec->mean, 0, offset, svec->len - offset);
 }
 
+G_INLINE_FUNC NcmVector *
+ncm_stats_vec_peek_mean (NcmStatsVec *svec)
+{
+  return svec->mean;
+}
+
 G_INLINE_FUNC void 
 ncm_stats_vec_get_cov_matrix (NcmStatsVec *svec, NcmMatrix *m, guint offset)
 {
