@@ -46,6 +46,7 @@
 #include <gsl/gsl_complex_math.h>
 #include <gsl/gsl_statistics_double.h>
 #include <gsl/gsl_cdf.h>
+#include <gsl/gsl_sf_hyperg.h>
 #include <cvode/cvode.h>
 #include <cvode/cvode_dense.h>
 
@@ -172,7 +173,7 @@ gdouble
 ncm_topology_sigma_comoving_a0_lss (guint n, gdouble alpha, gdouble sigma_alpha)
 {
   gdouble sigma = sin (alpha) * tan (M_PI/n) / (cos (alpha) * cos (alpha) + tan (M_PI / n) * tan (M_PI / n));
-  sigma = sqrt(sigma * sigma * sigma_alpha * sigma_alpha);
+  sigma = sqrt (sigma * sigma * sigma_alpha * sigma_alpha);
   if (n == 2)
     sigma = sigma_alpha / alpha;
   return sigma;
@@ -207,13 +208,13 @@ _besselj_bs_free (gpointer p)
 NcmCoarseDbl **
 _ncm_coarse_dbl_get_bs (void)
 {
-  _NCM_STATIC_MUTEX_DECL (create_lock);
+  G_LOCK_DEFINE_STATIC (create_lock);
   static NcmMemoryPool *mp = NULL;
 
-  _NCM_MUTEX_LOCK (&create_lock);
+  G_LOCK (create_lock);
   if (mp == NULL)
     mp = ncm_memory_pool_new (_besselj_bs_alloc, NULL, _besselj_bs_free);
-  _NCM_MUTEX_UNLOCK (&create_lock);
+  G_UNLOCK (create_lock);
 
   return ncm_memory_pool_get (mp);
 }
@@ -865,6 +866,70 @@ ncm_sqrt1px_m1 (gdouble x)
   return res;
 }
 
+/**
+ * ncm_cmpdbl:
+ * @x: a double.
+ * @y: a double.
+ *
+ * Returns: FIXME
+ */
+gdouble 
+ncm_cmpdbl (const gdouble x, const gdouble y)
+{
+  if (x == y)
+    return 0.0;
+  else
+    return fabs (2.0 * (x - y) / (x + y));
+}
+
+/**
+ * ncm_exprel:
+ * @x: a double
+ *
+ * Returns: FIXME
+ */
+gdouble 
+ncm_exprel (const gdouble x)
+{
+  return gsl_sf_hyperg_1F1_int (1, 2, x);
+}
+
+/**
+ * ncm_d1exprel:
+ * @x: a double
+ *
+ * Returns: FIXME
+ */
+gdouble 
+ncm_d1exprel (const gdouble x)
+{
+  return 0.5 * gsl_sf_hyperg_1F1_int (2, 3, x);
+}
+
+/**
+ * ncm_d2exprel:
+ * @x: a double
+ *
+ * Returns: FIXME
+ */
+gdouble 
+ncm_d2exprel (const gdouble x)
+{
+  return gsl_sf_hyperg_1F1_int (3, 4, x) / 3.0;
+}
+
+/**
+ * ncm_d3exprel:
+ * @x: a double
+ *
+ * Returns: FIXME
+ */
+gdouble 
+ncm_d3exprel (const gdouble x)
+{
+  return 0.25 * gsl_sf_hyperg_1F1_int (4, 5, x);
+}
+
 /**
  * ncm_cmp:
  * @x: a double.
@@ -1090,6 +1155,99 @@ ncm_util_cvode_print_stats (gpointer cvode)
   return TRUE;
 }
 
+/**
+ * ncm_util_basename_fits:
+ * @fits_filename: a fits filename
+ * 
+ * Extracts the extension .fits or .fit from @fits_filename and returns
+ * the prefix. If the extension is not found a copy of @fits_filename is
+ * returned.
+ * 
+ * Returns: (transfer full): prefix of @fits_filename.
+ */
+gchar *
+ncm_util_basename_fits (const gchar *fits_filename)
+{
+  GError *error          = NULL;
+  GRegex *fits_ext       = g_regex_new ("(.*)\\.[fF][iI][tT][sS]?$", 0, 0, &error);
+  GMatchInfo *match_info = NULL;
+  gchar *base_name       = NULL;
+
+  if (g_regex_match (fits_ext, fits_filename, 0, &match_info))
+  {
+    base_name = g_match_info_fetch (match_info, 1);
+  }
+  else
+    base_name = g_strdup (fits_filename);
+
+  g_match_info_free (match_info);
+  g_regex_unref (fits_ext);
+
+  return base_name;
+}
+
+/**
+ * ncm_util_function_params:
+ * @func: string representing function and its parameters
+ * @x: (out) (array) (element-type double): the parameters or NULL if none found
+ * @len: (out caller-allocates): number of parameters
+ * 
+ * Extracts the function name and its numerical parameters.
+ * 
+ * Returns: (transfer full): function name or NULL if it fails.
+ */
+gchar *
+ncm_util_function_params (const gchar *func, gdouble **x, guint *len)
+{
+  GError *error          = NULL;
+  GRegex *func_params    = g_regex_new ("^([A-Za-z][A-Za-z0-9\\_\\:]*)\\s*(?:\\(\\s*([0-9\\.,eE\\-\\+\\s]*?)\\s*\\))?\\s*$", 0, 0, &error);
+  GMatchInfo *match_info = NULL;
+  gchar *func_name       = NULL;
+
+  if (error != NULL)
+    g_error ("ncm_util_function_params: `%s'.", error->message);
+
+  func_name = NULL;
+  *x        = NULL;
+  *len      = 0;
+
+  if (g_regex_match (func_params, func, 0, &match_info))
+  {
+    gchar *fpa    = g_match_info_fetch (match_info, 2);
+    func_name     = g_match_info_fetch (match_info, 1);
+
+    if (fpa != NULL)
+    {
+      gchar **xs = g_regex_split_simple ("\\s*,\\s*", fpa, 0, 0);
+      *len       = g_strv_length (xs);
+      
+      if (*len > 0)
+      {
+        guint i;
+        *x = g_new0 (gdouble, *len);
+
+        for (i = 0; i < *len; i++)
+        {
+          gchar *endptr = NULL;
+          (*x)[i] = strtod (xs[i], &endptr);
+
+          if ((endptr == xs[i]) || (strlen (endptr) > 0))
+            g_error ("ncm_util_function_params: cannot identify double in string `%s'.", xs[i]);
+        }
+      }
+
+      g_strfreev (xs);
+      g_free (fpa);
+    }
+  }
+
+  g_match_info_free (match_info);
+  g_regex_unref (func_params);
+
+  return func_name;
+}
+
+
 void
 _ncm_util_set_destroyed (gpointer b)
 {
