@@ -49,6 +49,11 @@ gdouble ncm_numdiff_1 (gsl_function *F, const gdouble x, const gdouble ho, gdoub
 gdouble ncm_numdiff_2 (gsl_function *F, gdouble *ofx, const gdouble x, const gdouble ho, gdouble *err);
 gdouble ncm_numdiff_2_err (gsl_function *F, gdouble *ofx, const gdouble x, const gdouble ho, gdouble err, gdouble *ferr);
 gdouble ncm_sqrt1px_m1 (gdouble x);
+gdouble ncm_cmpdbl (const gdouble x, const gdouble y);
+gdouble ncm_exprel (const gdouble x);
+gdouble ncm_d1exprel (const gdouble x);
+gdouble ncm_d2exprel (const gdouble x);
+gdouble ncm_d3exprel (const gdouble x);
 
 gsize ncm_mpfr_out_raw (FILE *stream, mpfr_t op);
 gsize ncm_mpfr_inp_raw (mpfr_t rop, FILE *stream);
@@ -60,13 +65,16 @@ gulong ncm_random_seed (void);
 gint ncm_cmp (gdouble x, gdouble y, gdouble reltol);
 
 void ncm_rational_coarce_double (gdouble x, mpq_t q);
-void ncm_mpz_inits (mpz_t z, ...);
-void ncm_mpz_clears (mpz_t z, ...);
+void ncm_mpz_inits (mpz_t z, ...) G_GNUC_NULL_TERMINATED;
+void ncm_mpz_clears (mpz_t z, ...) G_GNUC_NULL_TERMINATED;
 void _ncm_assertion_message_cmpdouble (const gchar *domain, const gchar *file, gint line, const gchar *func, const gchar *expr, gdouble arg1, const gchar *cmp, gdouble arg2);
 
 gboolean ncm_util_cvode_check_flag (gpointer flagvalue, const gchar *funcname, gint opt);
 gboolean ncm_util_cvode_print_stats (gpointer cvode);
 
+gchar *ncm_util_basename_fits (const gchar *fits_filename); 
+gchar *ncm_util_function_params (const gchar *func, gdouble **x, guint *len);
+
 typedef struct _NcmComplex NcmComplex;
 
 struct _NcmComplex
@@ -92,6 +100,14 @@ gdouble ncm_complex_Im (NcmComplex *c);
 
 /* Macros */
 
+#ifndef HAVE_EXP10
+#define exp10(x) (exp ((x) * M_LN10))
+#endif /* HAVE_EXP10 */
+
+#define ncm_acb_get_complex(z) (arf_get_d (arb_midref (acb_realref (z)), ARF_RND_NEAR) + I * arf_get_d (arb_midref (acb_imagref (z)), ARF_RND_NEAR))
+
+#define ncm_util_exp10(x) (exp ((x) * M_LN10))
+
 #define NCM_GARRAY_MEMCPY(dest,src) \
 G_STMT_START { \
 g_assert_cmpuint ((src)->len, ==, (dest)->len); \
