@@ -90,14 +90,14 @@ nc_hiprim_atan_class_init (NcHIPrimAtanClass *klass)
                               NC_HIPRIM_DEFAULT_PARAMS_ABSTOL, NC_HIPRIM_ATAN_DEFAULT_N_SA,
                               NCM_PARAM_TYPE_FIXED);
 
-  /* Set c1 param info */
-  ncm_model_class_set_sparam (model_class, NC_HIPRIM_ATAN_C1, "c_1", "c1",
-                              0.0, 1.0e3, 10.0,
-                              NC_HIPRIM_DEFAULT_PARAMS_ABSTOL, NC_HIPRIM_ATAN_DEFAULT_C1,
+  /* Set lnkc param info */
+  ncm_model_class_set_sparam (model_class, NC_HIPRIM_ATAN_LNKC, "\\ln(k_c)", "lnkc",
+                              -18.0, 1.0, 0.5,
+                              NC_HIPRIM_DEFAULT_PARAMS_ABSTOL, NC_HIPRIM_ATAN_DEFAULT_LNKC,
                               NCM_PARAM_TYPE_FIXED);
   /* Set c2 param info */
   ncm_model_class_set_sparam (model_class, NC_HIPRIM_ATAN_C2, "c_2", "c2",
-                              0.0, 50.0, 5.0,
+                              0.0, 5.0, 0.5,
                               NC_HIPRIM_DEFAULT_PARAMS_ABSTOL, NC_HIPRIM_ATAN_DEFAULT_C2,
                               NCM_PARAM_TYPE_FIXED);
   /* Set c3 param info */
@@ -130,7 +130,7 @@ nc_hiprim_atan_new (void)
 #define VECTOR     (NCM_MODEL (prim)->params)
 #define LN10E10ASA (ncm_vector_get (VECTOR, NC_HIPRIM_ATAN_LN10E10ASA))
 #define N_SA       (ncm_vector_get (VECTOR, NC_HIPRIM_ATAN_N_SA))
-#define C1         (ncm_vector_get (VECTOR, NC_HIPRIM_ATAN_C1))
+#define LNKC       (ncm_vector_get (VECTOR, NC_HIPRIM_ATAN_LNKC))
 #define C2         (ncm_vector_get (VECTOR, NC_HIPRIM_ATAN_C2))
 #define C3         (ncm_vector_get (VECTOR, NC_HIPRIM_ATAN_C3))
 
@@ -141,17 +141,17 @@ nc_hiprim_atan_new (void)
 static gdouble
 _nc_hiprim_atan_lnSA_powespec_lnk (NcHIPrim *prim, const gdouble lnk)
 {
-  const gdouble ln_ka = lnk - prim->lnk_pivot;
-  const gdouble ka = exp (ln_ka);
-  const gdouble c1 = C1;
-  const gdouble c2 = C2;
-  const gdouble c3 = C3;
-  const gdouble pi_2      = M_PI * 0.5;
+  const gdouble ln_ka     = lnk - prim->lnk_pivot;
+  const gdouble ln_k_kc   = lnk - LNKC;
+  const gdouble k_kc      = exp (ln_k_kc);
+  const gdouble c2        = C2;
+  const gdouble c3        = C3;
+  const gdouble pi_2      = ncm_c_pi () * 0.5;
   const gdouble pi_2_m_c3 = pi_2 - c3;
   const gdouble tc2       = c2 + tan (pi_2_m_c3);
 
-  const gdouble atan_fac    = atan2 (c1 * ka + tc2, 1.0) - pi_2_m_c3;
-  const gdouble ln_atan_fac = log (atan_fac);
+  const gdouble atan_fac    = atan2 (k_kc + tc2, 1.0) - pi_2_m_c3;
+  const gdouble ln_atan_fac = log (atan_fac / c3);
 
   return (N_SA - 1.0) * ln_ka + LN10E10ASA - 10.0 * M_LN10 + ln_atan_fac;
 }
