@@ -56,6 +56,7 @@
 #include "nc_cbe.h"
 #include "nc_enum_types.h"
 #include "math/ncm_spline_cubic_notaknot.h"
+#include "math/ncm_spline2d_bicubic.h"
 
 enum
 {
@@ -69,6 +70,8 @@ enum
   PROP_SCALAR_LMAX,
   PROP_VECTOR_LMAX,
   PROP_TENSOR_LMAX,
+  PROP_MATTER_PK_MAXZ,
+  PROP_MATTER_PK_MAXK,
 };
 
 struct _NcCBEPrivate
@@ -374,7 +377,7 @@ nc_cbe_init (NcCBE *cbe)
 }
 
 static void
-nc_cbe_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+_nc_cbe_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
   NcCBE *cbe = NC_CBE (object);
   g_return_if_fail (NC_IS_CBE (object));
@@ -408,6 +411,12 @@ nc_cbe_set_property (GObject *object, guint prop_id, const GValue *value, GParam
     case PROP_TENSOR_LMAX:
       nc_cbe_set_tensor_lmax (cbe, g_value_get_uint (value));
       break;
+    case PROP_MATTER_PK_MAXZ:
+      nc_cbe_set_max_matter_pk_z (cbe, g_value_get_double (value));
+      break;
+    case PROP_MATTER_PK_MAXK:
+      nc_cbe_set_max_matter_pk_k (cbe, g_value_get_double (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -415,7 +424,7 @@ nc_cbe_set_property (GObject *object, guint prop_id, const GValue *value, GParam
 }
 
 static void
-nc_cbe_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+_nc_cbe_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
   NcCBE *cbe = NC_CBE (object);
   g_return_if_fail (NC_IS_CBE (object));
@@ -449,6 +458,12 @@ nc_cbe_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *
     case PROP_TENSOR_LMAX:
       g_value_set_uint (value, nc_cbe_get_tensor_lmax (cbe));
       break;
+    case PROP_MATTER_PK_MAXZ:
+      g_value_set_double (value, nc_cbe_get_max_matter_pk_z (cbe));
+      break;
+    case PROP_MATTER_PK_MAXK:
+      g_value_set_double (value, nc_cbe_get_max_matter_pk_k (cbe));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -456,7 +471,7 @@ nc_cbe_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *
 }
 
 static void
-nc_cbe_dispose (GObject *object)
+_nc_cbe_dispose (GObject *object)
 {
   NcCBE *cbe = NC_CBE (object);
 
@@ -469,10 +484,26 @@ nc_cbe_dispose (GObject *object)
   G_OBJECT_CLASS (nc_cbe_parent_class)->dispose (object);
 }
 
-static void
-nc_cbe_finalize (GObject *object)
-{
+static void _nc_cbe_free_thermo (NcCBE *cbe);
 
+static void
+_nc_cbe_finalize (GObject *object)
+{
+  NcCBE *cbe = NC_CBE (object);
+
+  if (cbe->allocated)
+  {
+    g_assert (cbe->free != NULL);
+    cbe->free (cbe);
+    cbe->allocated = FALSE;
+  }
+
+  if (cbe->thermodyn_prepared)
+  {
+    _nc_cbe_free_thermo (cbe);
+    cbe->thermodyn_prepared = FALSE;
+  }
+  
   /* Chain up : end */
   G_OBJECT_CLASS (nc_cbe_parent_class)->finalize (object);
 }
@@ -484,10 +515,10 @@ nc_cbe_class_init (NcCBEClass *klass)
 
   g_type_class_add_private (klass, sizeof (NcCBEPrivate));
 
-  object_class->set_property = nc_cbe_set_property;
-  object_class->get_property = nc_cbe_get_property;
-  object_class->dispose      = nc_cbe_dispose;
-  object_class->finalize     = nc_cbe_finalize;
+  object_class->set_property = &_nc_cbe_set_property;
+  object_class->get_property = &_nc_cbe_get_property;
+  object_class->dispose      = &_nc_cbe_dispose;
+  object_class->finalize     = &_nc_cbe_finalize;
 
   g_object_class_install_property (object_class,
                                    PROP_PREC,
@@ -552,6 +583,20 @@ nc_cbe_class_init (NcCBEClass *klass)
                                                       "Tensor modes l_max",
                                                       0, G_MAXUINT, 500,
                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_MATTER_PK_MAXZ,
+                                   g_param_spec_double ("matter-pk-maxz",
+                                                        NULL,
+                                                        "Maximum redshift for matter Pk",
+                                                        0.0, G_MAXDOUBLE, 0.0,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_MATTER_PK_MAXK,
+                                   g_param_spec_double ("matter-pk-maxk",
+                                                        NULL,
+                                                        "Maximum mode k for matter Pk",
+                                                        0.0, G_MAXDOUBLE, 0.1,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 }
 
 /**
@@ -568,6 +613,7 @@ nc_cbe_new (void)
   NcCBE *cbe = g_object_new (NC_TYPE_CBE,
                              "precision", prec,
                              NULL);
+  nc_cbe_precision_free (prec);
   return cbe;
 }
 
@@ -590,7 +636,7 @@ nc_cbe_prec_new (NcCBEPrecision *cbe_prec)
 
 /**
  * nc_cbe_ref:
- * @cbe: a #NcCBE.
+ * @cbe: a #NcCBE
  *
  * Increases the reference count of @cbe.
  *
@@ -604,7 +650,7 @@ nc_cbe_ref (NcCBE *cbe)
 
 /**
  * nc_cbe_free:
- * @cbe: a #NcCBE.
+ * @cbe: a #NcCBE
  *
  * Decreases the reference count of @cbe.
  *
@@ -617,7 +663,7 @@ nc_cbe_free (NcCBE *cbe)
 
 /**
  * nc_cbe_clear:
- * @cbe: a #NcCBE.
+ * @cbe: a #NcCBE
  *
  * Decreases the reference count of *@cbe and sets *@cbe to NULL.
  *
@@ -631,7 +677,7 @@ nc_cbe_clear (NcCBE **cbe)
 /**
  * nc_cbe_set_precision:
  * @cbe: a #NcCBE
- * @cbe_prec: a #NcCBEPrecision.
+ * @cbe_prec: a #NcCBEPrecision
  *
  * Sets the @cbe_prec as the precision object.
  *
@@ -667,7 +713,7 @@ nc_cbe_set_target_Cls (NcCBE *cbe, NcDataCMBDataType target_Cls)
 /**
  * nc_cbe_set_calc_transfer:
  * @cbe: a #NcCBE
- * @calc_transfer: a boolean.
+ * @calc_transfer: a boolean
  *
  * Sets whether it should calculate the transfer function.
  *
@@ -675,8 +721,11 @@ nc_cbe_set_target_Cls (NcCBE *cbe, NcDataCMBDataType target_Cls)
 void 
 nc_cbe_set_calc_transfer (NcCBE *cbe, gboolean calc_transfer)
 {
-  cbe->calc_transfer = calc_transfer;
-  _nc_cbe_update_callbacks (cbe);
+  if ((calc_transfer && !cbe->calc_transfer) || (!calc_transfer && cbe->calc_transfer))
+  {
+    cbe->calc_transfer = calc_transfer;
+    _nc_cbe_update_callbacks (cbe);
+  }
 }
 
 /**
@@ -684,7 +733,7 @@ nc_cbe_set_calc_transfer (NcCBE *cbe, gboolean calc_transfer)
  * @cbe: a #NcCBE
  * @use_lensed_Cls: a boolean.
  *
- * Sets whether it should use lensed Cls.
+ * Sets whether it should use lensed Cl's.
  *
  */
 void 
@@ -697,7 +746,7 @@ nc_cbe_set_lensed_Cls (NcCBE *cbe, gboolean use_lensed_Cls)
 /**
  * nc_cbe_set_tensor:
  * @cbe: a #NcCBE
- * @use_tensor: a boolean.
+ * @use_tensor: a boolean
  *
  * Sets whether it should use tensor contribution.
  *
@@ -712,7 +761,7 @@ nc_cbe_set_tensor (NcCBE *cbe, gboolean use_tensor)
 /**
  * nc_cbe_set_thermodyn:
  * @cbe: a #NcCBE
- * @use_thermodyn: a boolean.
+ * @use_thermodyn: a boolean
  *
  * Sets whether it should use the thermodynamics module.
  *
@@ -729,8 +778,9 @@ nc_cbe_set_thermodyn (NcCBE *cbe, gboolean use_thermodyn)
  * @cbe: a #NcCBE
  * @scalar_lmax: a guint
  *
- * Sets $\ell_\mathrm{max}$ for scalar modes.
- *
+ * Sets the maximum multipole $\ell_\textrm{max}$ at which the  
+ * angular power spectrum $C_{\ell}$ of the scalar mode is computed.
+ * 
  */
 void 
 nc_cbe_set_scalar_lmax (NcCBE *cbe, guint scalar_lmax)
@@ -747,7 +797,8 @@ nc_cbe_set_scalar_lmax (NcCBE *cbe, guint scalar_lmax)
  * @cbe: a #NcCBE
  * @vector_lmax: a guint
  *
- * Sets $\ell_\mathrm{max}$ for vector modes.
+ * Sets the maximum multipole $\ell_\textrm{max}$ at which the  
+ * angular power spectrum $C_{\ell}$ of the vector mode is computed.
  *
  */
 void 
@@ -765,7 +816,8 @@ nc_cbe_set_vector_lmax (NcCBE *cbe, guint vector_lmax)
  * @cbe: a #NcCBE
  * @tensor_lmax: a guint
  *
- * Sets $\ell_\mathrm{max}$ for tensor modes.
+ * Sets the maximum multipole $\ell_\textrm{max}$ at which the  
+ * angular power spectrum $C_{\ell}$ of the tensor mode is computed.
  *
  */
 void 
@@ -778,6 +830,70 @@ nc_cbe_set_tensor_lmax (NcCBE *cbe, guint tensor_lmax)
   }
 }
 
+/**
+ * nc_cbe_set_max_matter_pk_z:
+ * @cbe: a #NcCBE
+ * @zmax: maximum redshift
+ *
+ * Sets $z_\mathrm{max}$ for (until?) which the matter power spectrum $P(k, z)$ is evaluated.
+ *
+ */
+void 
+nc_cbe_set_max_matter_pk_z (NcCBE *cbe, gdouble zmax)
+{
+  if (cbe->priv->psp.z_max_pk != zmax)
+  {
+    cbe->priv->psp.z_max_pk = zmax;
+    ncm_model_ctrl_force_update (cbe->ctrl_cosmo);
+  }
+}
+
+/**
+ * nc_cbe_get_max_matter_pk_z:
+ * @cbe: a #NcCBE
+ *
+ * Gets the maximum redshift $z_\mathrm{max}$ for which the matter power spectrum $P(k, z)$ is evaluated.
+ * 
+ * Returns: $z_\mathrm{max}$.
+ */
+gdouble
+nc_cbe_get_max_matter_pk_z (NcCBE *cbe)
+{
+  return cbe->priv->psp.z_max_pk;
+}
+
+/**
+ * nc_cbe_set_max_matter_pk_k:
+ * @cbe: a #NcCBE
+ * @kmax: maximum mode
+ *
+ * Sets $k_\mathrm{max}$ for which the matter power spectrum $P (k, z)$ is evaluated.
+ *
+ */
+void 
+nc_cbe_set_max_matter_pk_k (NcCBE *cbe, gdouble kmax)
+{
+  if (cbe->priv->ppt.k_max_for_pk != kmax)
+  {
+    cbe->priv->ppt.k_max_for_pk = kmax;
+    ncm_model_ctrl_force_update (cbe->ctrl_cosmo);
+  }
+}
+
+/**
+ * nc_cbe_get_max_matter_pk_k:
+ * @cbe: a #NcCBE
+ *
+ * Gets the maximum mode $k_\mathrm{max}$ for which the matter power spectrum $P (k, z)$ is evaluated.
+ * 
+ * Returns: $k_\mathrm{max}$.
+ */
+gdouble 
+nc_cbe_get_max_matter_pk_k (NcCBE *cbe)
+{
+  return cbe->priv->ppt.k_max_for_pk;
+}
+
 /**
  * nc_cbe_peek_precision:
  * @cbe: a #NcCBE
@@ -824,7 +940,7 @@ nc_cbe_calc_transfer (NcCBE *cbe)
  * nc_cbe_lensed_Cls:
  * @cbe: a #NcCBE
  *
- * Gets whether it uses lensed Cls.
+ * Gets whether it uses lensed $C_{\ell}$'s.
  * 
  * Returns: a boolean.
  */
@@ -866,9 +982,10 @@ nc_cbe_thermodyn (NcCBE *cbe)
  * nc_cbe_get_scalar_lmax:
  * @cbe: a #NcCBE
  *
- * Gets the scalar $\ell_\textrm{max}$.
+ * Gets the maximum multipole $\ell_\textrm{max}$ at which the  
+ * angular power spectrum $C_{\ell}$ of the scalar mode is computed.
  * 
- * Returns: the scalar $\ell_\textrm{max}$.
+ * Returns: the maximum (scalar) multipole $\ell_\textrm{max}$.
  */
 guint 
 nc_cbe_get_scalar_lmax (NcCBE *cbe)
@@ -880,9 +997,10 @@ nc_cbe_get_scalar_lmax (NcCBE *cbe)
  * nc_cbe_get_vector_lmax:
  * @cbe: a #NcCBE
  *
- * Gets the vector $\ell_\textrm{max}$.
+ * Gets the maximum multipole $\ell_\textrm{max}$ at which the  
+ * angular power spectrum $C_{\ell}$ of the vector mode is computed.
  * 
- * Returns: the vector $\ell_\textrm{max}$.
+ * Returns: the maximum (vector) multipole $\ell_\textrm{max}$.
  */
 guint 
 nc_cbe_get_vector_lmax (NcCBE *cbe)
@@ -894,9 +1012,10 @@ nc_cbe_get_vector_lmax (NcCBE *cbe)
  * nc_cbe_get_tensor_lmax:
  * @cbe: a #NcCBE
  *
- * Gets the tensor $\ell_\textrm{max}$.
+ * Gets the maximum multipole $\ell_\textrm{max}$ at which the  
+ * angular power spectrum $C_{\ell}$ of the tensor mode is computed.
  * 
- * Returns: the tensor $\ell_\textrm{max}$.
+ * Returns: the maximum (tensor) multipole $\ell_\textrm{max}$.
  */
 guint 
 nc_cbe_get_tensor_lmax (NcCBE *cbe)
@@ -964,13 +1083,16 @@ _nc_cbe_set_bg (NcCBE *cbe, NcHICosmo *cosmo)
 }
 
 static void
-_nc_cbe_set_thermo (NcCBE *cbe, NcHIReion *reion, NcHICosmo *cosmo)
+_nc_cbe_set_thermo (NcCBE *cbe, NcHICosmo *cosmo)
 {
+  NcHIReion *reion = nc_hicosmo_peek_reion (cosmo);
   struct precision *ppr = (struct precision *)cbe->prec->priv;
 
+  g_assert (reion != NULL);
+
   cbe->priv->pth.YHe                      = nc_hicosmo_Yp_4He (cosmo);
   cbe->priv->pth.recombination            = recfast;
-  cbe->priv->pth.reio_parametrization     = reion == NULL ? reio_none : reio_camb;
+  cbe->priv->pth.reio_parametrization     = reio_camb;
   if (NC_IS_HIREION_CAMB (reion))
   {
     cbe->priv->pth.reio_z_or_tau            = reio_z;
@@ -1075,7 +1197,7 @@ _nc_cbe_set_pert (NcCBE *cbe, NcHICosmo *cosmo)
   cbe->priv->ppt.l_vector_max = cbe->vector_lmax;
   cbe->priv->ppt.l_tensor_max = cbe->tensor_lmax;
   cbe->priv->ppt.l_lss_max    = 300;
-  cbe->priv->ppt.k_max_for_pk = 0.1;
+  /*cbe->priv->ppt.k_max_for_pk = 0.1;*/
 
   cbe->priv->ppt.gauge = synchronous;
 
@@ -1120,9 +1242,10 @@ _external_Pk_callback_pkt (const double lnk, gpointer data)
 }
 
 static void
-_nc_cbe_set_prim (NcCBE *cbe, NcHIPrim *prim, NcHICosmo *cosmo)
+_nc_cbe_set_prim (NcCBE *cbe, NcHICosmo *cosmo)
 {
-
+  NcHIPrim *prim = nc_hicosmo_peek_prim (cosmo);
+  
   cbe->priv->ppm.primordial_spec_type = external_Pk_callback;
   /*cbe->priv->ppm.primordial_spec_type = analytic_Pk;*/
   cbe->priv->ppm.external_Pk_callback_pks  = &_external_Pk_callback_pks;
@@ -1230,7 +1353,7 @@ _nc_cbe_set_transfer (NcCBE *cbe, NcHICosmo *cosmo)
 static void
 _nc_cbe_set_spectra (NcCBE *cbe, NcHICosmo *cosmo)
 {
-  cbe->priv->psp.z_max_pk = 0.0;
+  /*cbe->priv->psp.z_max_pk = 0.0;*/
   cbe->priv->psp.non_diag = 0;
 
   cbe->priv->psp.spectra_verbose = cbe->spectra_verbose;
@@ -1267,7 +1390,7 @@ _nc_cbe_call_bg (NcCBE *cbe, NcHICosmo *cosmo)
   _nc_cbe_set_bg (cbe, cosmo);
   if (background_init (ppr, &cbe->priv->pba) == _FAILURE_)
     g_error ("_nc_cbe_call_bg: Error running background_init `%s'\n", cbe->priv->pba.error_message);
-
+  
   if (FALSE)
   {
     const gdouble RH = nc_hicosmo_RH_Mpc (cosmo);
@@ -1359,19 +1482,19 @@ _nc_cbe_call_bg (NcCBE *cbe, NcHICosmo *cosmo)
 }
 
 static void
-_nc_cbe_call_thermo (NcCBE *cbe, NcHIReion *reion, NcHICosmo *cosmo)
+_nc_cbe_call_thermo (NcCBE *cbe, NcHICosmo *cosmo)
 {
   struct precision *ppr = (struct precision *)cbe->prec->priv;
 
   _nc_cbe_call_bg (cbe, cosmo);
   
-  _nc_cbe_set_thermo (cbe, reion, cosmo);
+  _nc_cbe_set_thermo (cbe, cosmo);
   if (thermodynamics_init (ppr, &cbe->priv->pba, &cbe->priv->pth) == _FAILURE_)
     g_error ("_nc_cbe_call_thermo: Error running thermodynamics_init `%s'\n", cbe->priv->pth.error_message);
 }
 
 static void
-_nc_cbe_call_pert (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *cosmo)
+_nc_cbe_call_pert (NcCBE *cbe, NcHICosmo *cosmo)
 {
   struct precision *ppr = (struct precision *)cbe->prec->priv;
 
@@ -1384,24 +1507,24 @@ _nc_cbe_call_pert (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *cosm
 }
 
 static void
-_nc_cbe_call_prim (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *cosmo)
+_nc_cbe_call_prim (NcCBE *cbe, NcHICosmo *cosmo)
 {
   struct precision *ppr = (struct precision *)cbe->prec->priv;
 
-  _nc_cbe_call_pert (cbe, prim, reion, cosmo);
+  _nc_cbe_call_pert (cbe, cosmo);
   cbe->free = &_nc_cbe_free_prim;
 
-  _nc_cbe_set_prim (cbe, prim, cosmo);
+  _nc_cbe_set_prim (cbe, cosmo);
   if (primordial_init (ppr, &cbe->priv->ppt, &cbe->priv->ppm) == _FAILURE_)
     g_error ("_nc_cbe_call_prim: Error running primordial_init `%s'\n", cbe->priv->ppm.error_message);  
 }
 
 static void
-_nc_cbe_call_nonlin (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *cosmo)
+_nc_cbe_call_nonlin (NcCBE *cbe, NcHICosmo *cosmo)
 {
   struct precision *ppr = (struct precision *)cbe->prec->priv;
 
-  _nc_cbe_call_prim (cbe, prim, reion, cosmo);
+  _nc_cbe_call_prim (cbe, cosmo);
   cbe->free = &_nc_cbe_free_nonlin;
 
   _nc_cbe_set_nonlin (cbe, cosmo);
@@ -1410,11 +1533,11 @@ _nc_cbe_call_nonlin (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *co
 }
 
 static void
-_nc_cbe_call_transfer (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *cosmo)
+_nc_cbe_call_transfer (NcCBE *cbe, NcHICosmo *cosmo)
 {
   struct precision *ppr = (struct precision *)cbe->prec->priv;
 
-  _nc_cbe_call_nonlin (cbe, prim, reion, cosmo);
+  _nc_cbe_call_nonlin (cbe, cosmo);
   cbe->free = &_nc_cbe_free_transfer;
 
   _nc_cbe_set_transfer (cbe, cosmo);
@@ -1423,11 +1546,11 @@ _nc_cbe_call_transfer (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *
 }
 
 static void
-_nc_cbe_call_spectra (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *cosmo)
+_nc_cbe_call_spectra (NcCBE *cbe, NcHICosmo *cosmo)
 {
   struct precision *ppr = (struct precision *)cbe->prec->priv;
 
-  _nc_cbe_call_transfer (cbe, prim, reion, cosmo);
+  _nc_cbe_call_transfer (cbe, cosmo);
   cbe->free = &_nc_cbe_free_spectra;
 
   _nc_cbe_set_spectra (cbe, cosmo);
@@ -1436,11 +1559,11 @@ _nc_cbe_call_spectra (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *c
 }
 
 static void
-_nc_cbe_call_lensing (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *cosmo)
+_nc_cbe_call_lensing (NcCBE *cbe, NcHICosmo *cosmo)
 {
   struct precision *ppr = (struct precision *)cbe->prec->priv;
 
-  _nc_cbe_call_spectra (cbe, prim, reion, cosmo);
+  _nc_cbe_call_spectra (cbe, cosmo);
   cbe->free = &_nc_cbe_free_lensing;
 
   _nc_cbe_set_lensing (cbe, cosmo);
@@ -1536,24 +1659,20 @@ _nc_cbe_update_callbacks (NcCBE *cbe)
   
   if (has_Cls && cbe->use_lensed_Cls)
     cbe->call = _nc_cbe_call_lensing;
-  else if (has_Cls)
+  else if (has_Cls || cbe->calc_transfer)
     cbe->call = _nc_cbe_call_spectra;
-  else if (cbe->calc_transfer)
-    cbe->call = _nc_cbe_call_transfer;
-
 }
 
 /**
  * nc_cbe_thermodyn_prepare:
  * @cbe: a #NcCBE
- * @reion: a #NcHIReion
  * @cosmo: a #NcHICosmo
  * 
  * Prepares the thermodynamic Class structure.
  * 
  */
 void
-nc_cbe_thermodyn_prepare (NcCBE *cbe, NcHIReion *reion, NcHICosmo *cosmo)
+nc_cbe_thermodyn_prepare (NcCBE *cbe, NcHICosmo *cosmo)
 {
   if (cbe->thermodyn_prepared)
   {
@@ -1561,25 +1680,24 @@ nc_cbe_thermodyn_prepare (NcCBE *cbe, NcHIReion *reion, NcHICosmo *cosmo)
     cbe->thermodyn_prepared = FALSE;
   }
 
-  _nc_cbe_call_thermo (cbe, reion, cosmo);
+  _nc_cbe_call_thermo (cbe, cosmo);
   cbe->thermodyn_prepared = TRUE;
 }
 
 /**
  * nc_cbe_thermodyn_prepare_if_needed:
  * @cbe: a #NcCBE
- * @reion: a #NcHIReion
  * @cosmo: a #NcHICosmo
  * 
  * Prepares the thermodynamic Class structure.
  * 
  */
 void
-nc_cbe_thermodyn_prepare_if_needed (NcCBE *cbe, NcHIReion *reion, NcHICosmo *cosmo)
+nc_cbe_thermodyn_prepare_if_needed (NcCBE *cbe, NcHICosmo *cosmo)
 {
   if (ncm_model_ctrl_update (cbe->ctrl_cosmo, NCM_MODEL (cosmo)))
   {
-    nc_cbe_thermodyn_prepare (cbe, reion, cosmo);
+    nc_cbe_thermodyn_prepare (cbe, cosmo);
     ncm_model_ctrl_force_update (cbe->ctrl_prim);
   }
 }
@@ -1587,16 +1705,15 @@ nc_cbe_thermodyn_prepare_if_needed (NcCBE *cbe, NcHIReion *reion, NcHICosmo *cos
 /**
  * nc_cbe_prepare:
  * @cbe: a #NcCBE
- * @prim: a #NcHIPrim
- * @reion: a #NcHIReion
  * @cosmo: a #NcHICosmo
  * 
  * Prepares all necessary Class structures.
  * 
  */
 void
-nc_cbe_prepare (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *cosmo)
+nc_cbe_prepare (NcCBE *cbe, NcHICosmo *cosmo)
 {
+  /*printf ("Preparing CLASS!\n");*/
   if (cbe->allocated)
   {
     g_assert (cbe->free != NULL);
@@ -1610,12 +1727,12 @@ nc_cbe_prepare (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *cosmo)
     cbe->thermodyn_prepared = FALSE;
   }
 
-  _nc_cbe_call_thermo (cbe, reion, cosmo);
+  _nc_cbe_call_thermo (cbe, cosmo);
   cbe->thermodyn_prepared = TRUE;
 
   if (cbe->call != NULL)
   {
-    cbe->call (cbe, prim, reion, cosmo);
+    cbe->call (cbe, cosmo);
     cbe->allocated = TRUE;
   }
 }
@@ -1623,35 +1740,44 @@ nc_cbe_prepare (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *cosmo)
 /**
  * nc_cbe_prepare_if_needed:
  * @cbe: a #NcCBE
- * @prim: a #NcHIPrim
- * @reion: a #NcHIReion
  * @cosmo: a #NcHICosmo
  * 
  * Prepares all necessary Class structures.
  * 
  */
 void
-nc_cbe_prepare_if_needed (NcCBE *cbe, NcHIPrim *prim, NcHIReion *reion, NcHICosmo *cosmo)
+nc_cbe_prepare_if_needed (NcCBE *cbe, NcHICosmo *cosmo)
 {
-  gboolean cosmo_up = ncm_model_ctrl_update (cbe->ctrl_cosmo, NCM_MODEL (cosmo));
-  gboolean prim_up = ncm_model_ctrl_update (cbe->ctrl_cosmo, NCM_MODEL (prim));
+  ncm_model_ctrl_update (cbe->ctrl_cosmo, NCM_MODEL (cosmo));
 
-  if (cosmo_up)
-  {    
-    nc_cbe_prepare (cbe, prim, reion, cosmo);
-  }
-  else if (prim_up)
+  if (!ncm_model_ctrl_model_has_submodel (cbe->ctrl_cosmo, nc_hiprim_id ()))
   {
-    if (cbe->allocated)
-    {
-      g_assert (cbe->free != NULL);
-      cbe->free (cbe);
-      cbe->allocated = FALSE;
+    g_error ("nc_cbe_prepare_if_needed: cosmo model must contain a NcHIPrim submodel.");
+  }
+  else
+  {
+    gboolean cosmo_up = ncm_model_ctrl_model_last_update (cbe->ctrl_cosmo);
+    gboolean prim_up  = ncm_model_ctrl_submodel_last_update (cbe->ctrl_cosmo, nc_hiprim_id ());
+
+    /*printf ("cosmo_up %d prim_up %d [%p]\n", cosmo_up, prim_up, cosmo);*/
+    
+    if (cosmo_up)
+    {    
+      nc_cbe_prepare (cbe, cosmo);
     }
-    if (cbe->call != NULL)
+    else if (prim_up)
     {
-      cbe->call (cbe, prim, reion, cosmo);
-      cbe->allocated = TRUE;
+      if (cbe->allocated)
+      {
+        g_assert (cbe->free != NULL);
+        cbe->free (cbe);
+        cbe->allocated = FALSE;
+      }
+      if (cbe->call != NULL)
+      {
+        cbe->call (cbe, cosmo);
+        cbe->allocated = TRUE;
+      }
     }
   }
 }
@@ -1690,6 +1816,55 @@ nc_cbe_thermodyn_get_Xe (NcCBE *cbe)
   return Xe_s;
 }
 
+/**
+ * nc_cbe_get_matter_ps:
+ * @cbe: a #NcCBE
+ * 
+ * Gets the logarithm base e of the matter power spectrum as a function of the redshift $z$ and mode $\ln (k)$.
+ * 
+ * Returns: (transfer full): a #NcmSpline2d for the logarithm base e of the matter power spectrum, $\ln P(\ln k, z)$.
+ */
+NcmSpline2d *
+nc_cbe_get_matter_ps (NcCBE *cbe)
+{
+  NcmVector *lnk_v = ncm_vector_new (cbe->priv->psp.ln_k_size);
+  NcmVector *z_v   = ncm_vector_new (cbe->priv->psp.ln_tau_size);
+  NcmMatrix *lnPk  = ncm_matrix_new (cbe->priv->psp.ln_tau_size, cbe->priv->psp.ln_k_size);
+  guint i;
+
+  for (i = 0; i < cbe->priv->psp.ln_k_size; i++)
+  {
+    ncm_vector_set (lnk_v, i, cbe->priv->psp.ln_k[i]);
+    /*printf ("lnk[%u] % 20.15g k % 20.15g\n", i, cbe->priv->psp.ln_k[i], exp (cbe->priv->psp.ln_k[i]));*/
+  }
+
+  for (i = 0; i < cbe->priv->psp.ln_tau_size; i++)
+  {
+    const gdouble z_i = cbe->priv->psp.z_max_pk / (cbe->priv->psp.ln_tau_size - 1.0) * i;
+    ncm_vector_set (z_v, i, z_i);
+    spectra_pk_at_z (&cbe->priv->pba, &cbe->priv->psp, logarithmic, z_i, ncm_matrix_ptr (lnPk, i, 0), NULL);
+
+/*    printf ("z % 20.15g\n", z_i);
+    guint j;
+    for (j = 0; j < cbe->priv->psp.ln_k_size; j++)
+    {
+      printf ("lnPk[%u] % 20.15g % 20.15g\n", j, ncm_matrix_get (lnPk, i, j), exp (ncm_matrix_get (lnPk, i, j)));
+    }
+*/
+  }
+
+  {
+    NcmSpline2d *lnPk_s = ncm_spline2d_bicubic_notaknot_new ();
+    ncm_spline2d_set (lnPk_s, lnk_v, z_v, lnPk, TRUE);
+
+    ncm_vector_free (z_v);
+    ncm_vector_free (lnk_v);
+    ncm_matrix_free (lnPk);
+
+    return lnPk_s;
+  }
+}
+
 /**
  * nc_cbe_get_all_Cls:
  * @cbe: a #NcCBE
@@ -1698,8 +1873,8 @@ nc_cbe_thermodyn_get_Xe (NcCBE *cbe)
  * @BB_Cls: a #NcmVector
  * @TE_Cls: a #NcmVector
  * 
- * Gets and store the Cls calculated in the vectors @TT_Cls, @EE_Cls, 
- * @BB_Cls and @TE_Cls. If any of these vectors are NULL then it is
+ * Gets and store the angular power spectra $C_l$'s calculated in the vectors @TT_Cls, @EE_Cls, 
+ * @BB_Cls and @TE_Cls. If any of these vectors are NULL, then it is (they are)
  * ignored.
  * 
  */
