@@ -47,6 +47,12 @@ typedef struct _NcDistance NcDistance;
 typedef gdouble (*NcDistanceFunc0) (NcDistance *dist, NcHICosmo *cosmo);
 typedef gdouble (*NcDistanceFunc1) (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
 
+struct _NcDistanceClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+};
+
 struct _NcDistance
 {
   /*< private >*/
@@ -58,7 +64,7 @@ struct _NcDistance
   NcmFunctionCache *conformal_time_cache;
   NcmFunctionCache *sound_horizon_cache;
   NcmModelCtrl *ctrl;
-  gdouble z_f;
+  gdouble zf;
   gboolean use_cache;
 };
 
@@ -78,34 +84,19 @@ typedef struct _NcDistanceFuncZ
   NcHICosmoImpl impl;
 } NcDistanceFuncZ;
 
-struct _NcDistanceClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-  GArray *func_table;
-  GArray *func_z_table;
-  GHashTable *func_hash;
-  GHashTable *func_z_hash;
-};
-
 GType nc_distance_get_type (void) G_GNUC_CONST;
 
-GArray *nc_distance_class_func_table (void);
-GArray *nc_distance_class_func_z_table (void);
-NcDistanceFunc *nc_distance_class_get_func (const gchar *name);
-NcDistanceFuncZ *nc_distance_class_get_func_z (const gchar *name);
-
-NcDistance *nc_distance_new (gdouble z_f);
+NcDistance *nc_distance_new (gdouble zf);
 NcDistance *nc_distance_ref (NcDistance *dist);
+
+void nc_distance_require_zf (NcDistance *dist, const gdouble zf);
+
 void nc_distance_prepare (NcDistance *dist, NcHICosmo *cosmo);
 G_INLINE_FUNC void nc_distance_prepare_if_needed (NcDistance *dist, NcHICosmo *cosmo);
+
 void nc_distance_free (NcDistance *dist);
 void nc_distance_clear (NcDistance **dist);
 
-NcmMSetFunc *nc_distance_func0_new (NcDistance *dist, NcDistanceFunc0 f0);
-NcmMSetFunc *nc_distance_func1_new (NcDistance *dist, NcDistanceFunc1 f1);
-NcmMSetFunc *nc_distance_create_mset_arrayfunc1 (NcDistance *dist, NcDistanceFunc1 f1, guint size);
-
 /***************************************************************************
  * Redshift independent 'distances'
  ****************************************************************************/
@@ -116,6 +107,7 @@ gdouble nc_distance_drag_redshift (NcDistance *dist, NcHICosmo *cosmo);
 gdouble nc_distance_shift_parameter_lss (NcDistance *dist, NcHICosmo *cosmo);
 gdouble nc_distance_comoving_lss (NcDistance *dist, NcHICosmo *cosmo);
 gdouble nc_distance_acoustic_scale (NcDistance *dist, NcHICosmo *cosmo);
+gdouble nc_distance_theta100CMB (NcDistance *dist, NcHICosmo *cosmo);
 gdouble nc_distance_Omega_k0 (NcDistance *dist, NcHICosmo *cosmo);
 gdouble nc_distance_angular_diameter_curvature_scale (NcDistance *dist, NcHICosmo *cosmo);
 gdouble nc_distance_r_zd (NcDistance *dist, NcHICosmo *cosmo);
@@ -154,9 +146,6 @@ gdouble nc_distance_lookback_time (NcDistance *dist, NcHICosmo *cosmo, gdouble z
 gdouble nc_distance_conformal_time (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
 gdouble nc_distance_conformal_lookback_time (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
 
-gdouble nc_distance_cosmic_time_mks_scale (NcDistance *dist, NcHICosmo *cosmo);
-gdouble nc_distance_conformal_time_mks_scale (NcDistance *dist, NcHICosmo *cosmo);
-
 G_END_DECLS
 
 #endif /* _NC_DISTANCE_INLINE_H_ */
