@@ -32,6 +32,7 @@
 #include <numcosmo/math/ncm_model.h>
 #include <numcosmo/math/ncm_mset_func.h>
 #include <numcosmo/math/ncm_likelihood.h>
+#include <numcosmo/math/ncm_powspec_filter.h>
 
 G_BEGIN_DECLS
 
@@ -51,7 +52,6 @@ G_BEGIN_DECLS
  * @NC_HICOSMO_IMPL_Omega_r0: Radiation density today $\Omega_{r0}$
  * @NC_HICOSMO_IMPL_Omega_c0: Cold dark matter density today $\Omega_{c0}$
  * @NC_HICOSMO_IMPL_Omega_t0: Total density today $\Omega_{t0}$
- * @NC_HICOSMO_IMPL_sigma_8: Standard deviation of the matter density contrast at scale $R = 8h^{-1} \text{Mpc}$, $\sigma_8$
  * @NC_HICOSMO_IMPL_T_gamma0: Radiation temperature today
  * @NC_HICOSMO_IMPL_Yp_4He: Primordial Helium mass fraction 
  * @NC_HICOSMO_IMPL_z_lss: Redshift of the last scatering surface
@@ -62,7 +62,6 @@ G_BEGIN_DECLS
  * @NC_HICOSMO_IMPL_d2E2_dz2: Second derivative of the dimensionless Hubble function squared.
  * @NC_HICOSMO_IMPL_bgp_cs2: Baryon-photon plasma speed of sound squared $c_s^2$.
  * @NC_HICOSMO_IMPL_Dc: Comoving distance
- * @NC_HICOSMO_IMPL_powspec: Perturbations power spectrum
  *
  * Flags defining the implementation options of the NcHICosmo abstract object. 
  * 
@@ -76,22 +75,21 @@ typedef enum _NcHICosmoImpl
   NC_HICOSMO_IMPL_Omega_nu0 = 1 << 4,
   NC_HICOSMO_IMPL_Omega_r0  = 1 << 5,
   NC_HICOSMO_IMPL_Omega_t0  = 1 << 6,
-  NC_HICOSMO_IMPL_sigma_8   = 1 << 7,
-  NC_HICOSMO_IMPL_T_gamma0  = 1 << 8,
-  NC_HICOSMO_IMPL_Yp_4He    = 1 << 9,
-  NC_HICOSMO_IMPL_z_lss     = 1 << 10,
-  NC_HICOSMO_IMPL_as_drag   = 1 << 11,
-  NC_HICOSMO_IMPL_xb        = 1 << 12,
-  NC_HICOSMO_IMPL_E2        = 1 << 13,
-  NC_HICOSMO_IMPL_dE2_dz    = 1 << 14,
-  NC_HICOSMO_IMPL_d2E2_dz2  = 1 << 15,
-  NC_HICOSMO_IMPL_bgp_cs2   = 1 << 16,
-  NC_HICOSMO_IMPL_Dc        = 1 << 17,
-  NC_HICOSMO_IMPL_powspec   = 1 << 18, /*< private >*/
-  NC_HICOSMO_IMPL_LAST      = 1 << 19, /*< skip >*/
+  NC_HICOSMO_IMPL_T_gamma0  = 1 << 7,
+  NC_HICOSMO_IMPL_Yp_4He    = 1 << 8,
+  NC_HICOSMO_IMPL_z_lss     = 1 << 9,
+  NC_HICOSMO_IMPL_as_drag   = 1 << 10,
+  NC_HICOSMO_IMPL_xb        = 1 << 11,
+  NC_HICOSMO_IMPL_E2        = 1 << 12,
+  NC_HICOSMO_IMPL_dE2_dz    = 1 << 13,
+  NC_HICOSMO_IMPL_d2E2_dz2  = 1 << 14,
+  NC_HICOSMO_IMPL_bgp_cs2   = 1 << 15,
+  NC_HICOSMO_IMPL_Dc        = 1 << 16, /*< private >*/
+  NC_HICOSMO_IMPL_LAST      = 1 << 18, /*< skip >*/
 } NcHICosmoImpl;
 
 #define NC_HICOSMO_IMPL_RH_Mpc (NC_HICOSMO_IMPL_H0)
+#define NC_HICOSMO_IMPL_RH_planck (NC_HICOSMO_IMPL_H0)
 #define NC_HICOSMO_IMPL_Omega_k0 (NC_HICOSMO_IMPL_Omega_t0)
 #define NC_HICOSMO_IMPL_Omega_m0 (NC_HICOSMO_IMPL_Omega_c0 | NC_HICOSMO_IMPL_Omega_b0)
 #define NC_HICOSMO_IMPL_h (NC_HICOSMO_IMPL_H0)
@@ -122,18 +120,10 @@ typedef gdouble (*NcHICosmoFunc0) (NcHICosmo *cosmo);
 typedef gdouble (*NcHICosmoFunc1Z) (NcHICosmo *cosmo, gdouble z);
 typedef gdouble (*NcHICosmoFunc1K) (NcHICosmo *cosmo, gdouble k);
 
-/**
- * NcHICosmo:
- *
- * FIXME
- *
- */
-struct _NcHICosmo
-{
-  /*< private >*/
-  NcmModel parent_instance;
-  gboolean is_eternal;
-};
+#ifndef __GTK_DOC_IGNORE__
+typedef struct _NcHIPrim NcHIPrim;
+typedef struct _NcHIReion NcHIReion;
+#endif
 
 typedef struct _NcHICosmoFuncZ
 {
@@ -162,7 +152,6 @@ struct _NcHICosmoClass
   NcHICosmoFunc0  Omega_r0;
   NcHICosmoFunc0  Omega_c0;
   NcHICosmoFunc0  Omega_t0;
-  NcHICosmoFunc0  sigma_8;
   NcHICosmoFunc0  T_gamma0;
   NcHICosmoFunc0  Yp_4He;
   NcHICosmoFunc0  z_lss;
@@ -173,11 +162,21 @@ struct _NcHICosmoClass
   NcHICosmoFunc1Z d2E2_dz2;
   NcHICosmoFunc1Z bgp_cs2;
   NcHICosmoFunc1Z Dc;
-  NcHICosmoFunc1K powspec;
-  GArray *func_table;
-  GArray *func_z_table;
-  GHashTable *func_hash;
-  GHashTable *func_z_hash;
+};
+
+/**
+ * NcHICosmo:
+ *
+ * FIXME
+ *
+ */
+struct _NcHICosmo
+{
+  /*< private >*/
+  NcmModel parent_instance;
+  gboolean is_eternal;
+  NcHIPrim *prim;
+  NcHIReion *reion;
 };
 
 GType nc_hicosmo_get_type (void) G_GNUC_CONST;
@@ -191,7 +190,6 @@ void nc_hicosmo_set_Omega_nu0_impl (NcHICosmoClass *model_class, NcHICosmoFunc0
 void nc_hicosmo_set_Omega_r0_impl (NcHICosmoClass *model_class, NcHICosmoFunc0 f);
 void nc_hicosmo_set_Omega_c0_impl (NcHICosmoClass *model_class, NcHICosmoFunc0 f);
 void nc_hicosmo_set_Omega_t0_impl (NcHICosmoClass *model_class, NcHICosmoFunc0 f);
-void nc_hicosmo_set_sigma_8_impl (NcHICosmoClass *model_class, NcHICosmoFunc0 f);
 void nc_hicosmo_set_T_gamma0_impl (NcHICosmoClass *model_class, NcHICosmoFunc0 f);
 void nc_hicosmo_set_Yp_4He_impl (NcHICosmoClass *model_class, NcHICosmoFunc0 f);
 void nc_hicosmo_set_z_lss_impl (NcHICosmoClass *model_class, NcHICosmoFunc0 f);
@@ -203,7 +201,6 @@ void nc_hicosmo_set_dE2_dz_impl (NcHICosmoClass *model_class, NcHICosmoFunc1Z f)
 void nc_hicosmo_set_d2E2_dz2_impl (NcHICosmoClass *model_class, NcHICosmoFunc1Z f);
 void nc_hicosmo_set_bgp_cs2_impl (NcHICosmoClass *model_class, NcHICosmoFunc1Z f);
 void nc_hicosmo_set_Dc_impl (NcHICosmoClass *model_class, NcHICosmoFunc1Z f);
-void nc_hicosmo_set_powspec_impl (NcHICosmoClass *model_class, NcHICosmoFunc1Z f);
 
 NcHICosmo *nc_hicosmo_new_from_name (GType parent_type, gchar *cosmo_name);
 NcHICosmo *nc_hicosmo_ref (NcHICosmo *cosmo);
@@ -217,6 +214,7 @@ void nc_hicosmo_log_all_models (GType parent);
  */
 G_INLINE_FUNC gdouble nc_hicosmo_H0 (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_RH_Mpc (NcHICosmo *cosmo);
+G_INLINE_FUNC gdouble nc_hicosmo_RH_planck (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_h (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_h2 (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_Omega_b0 (NcHICosmo *cosmo);
@@ -243,7 +241,6 @@ G_INLINE_FUNC gdouble nc_hicosmo_baryon_density (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_He_number_density (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_H_number_density (NcHICosmo *cosmo);
 
-G_INLINE_FUNC gdouble nc_hicosmo_sigma_8 (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_z_lss (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_as_drag (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_xb (NcHICosmo *cosmo);
@@ -259,7 +256,6 @@ G_INLINE_FUNC gdouble nc_hicosmo_d2E2_dz2 (NcHICosmo *cosmo, gdouble z);
 
 G_INLINE_FUNC gdouble nc_hicosmo_bgp_cs2 (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_Dc (NcHICosmo *cosmo, gdouble z);
-G_INLINE_FUNC gdouble nc_hicosmo_powspec (NcHICosmo *cosmo, gdouble k);
 
 G_INLINE_FUNC gdouble nc_hicosmo_q (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_dec (NcHICosmo *cosmo, gdouble z);
@@ -267,17 +263,13 @@ G_INLINE_FUNC gdouble nc_hicosmo_wec (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_qp (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_j (NcHICosmo *cosmo, gdouble z);
 
-G_INLINE_FUNC gdouble nc_hicosmo_abs_alpha (NcHICosmo *cosmo, gdouble z);
+G_INLINE_FUNC gdouble nc_hicosmo_abs_alpha (NcHICosmo *cosmo, gdouble x);
 G_INLINE_FUNC gdouble nc_hicosmo_x_alpha (NcHICosmo *cosmo, gdouble alpha);
 
-GArray *nc_hicosmo_class_func_table (void);
-GArray *nc_hicosmo_class_func_z_table (void);
-NcHICosmoFunc *nc_hicosmo_class_get_func (const gchar *name);
-NcHICosmoFuncZ *nc_hicosmo_class_get_func_z (const gchar *name);
+G_INLINE_FUNC NcHIPrim *nc_hicosmo_peek_prim (NcHICosmo *cosmo);
+G_INLINE_FUNC NcHIReion *nc_hicosmo_peek_reion (NcHICosmo *cosmo);
 
-NcmMSetFunc *nc_hicosmo_create_mset_func0 (NcHICosmoFunc0 f0);
-NcmMSetFunc *nc_hicosmo_create_mset_func1 (NcHICosmoFunc1Z f1);
-NcmMSetFunc *nc_hicosmo_create_mset_arrayfunc1 (NcHICosmoFunc1Z f1, guint size);
+gdouble nc_hicosmo_sigma8 (NcHICosmo *cosmo, NcmPowspecFilter *psf);
 
 #define NC_HICOSMO_DEFAULT_PARAMS_RELTOL (1e-7)
 #define NC_HICOSMO_DEFAULT_PARAMS_ABSTOL (0.0)
@@ -301,7 +293,6 @@ NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,Omega_c0)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,Omega_t0)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,T_gamma0)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,Yp_4He)
-NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,sigma_8)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,z_lss)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,as_drag)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,xb)
@@ -312,7 +303,6 @@ NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,d2E2_dz2,z)
 
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,bgp_cs2,z)
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,Dc,z)
-NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,powspec,k)
 
 G_INLINE_FUNC gdouble
 nc_hicosmo_RH_Mpc (NcHICosmo *cosmo)
@@ -320,6 +310,12 @@ nc_hicosmo_RH_Mpc (NcHICosmo *cosmo)
   return (ncm_c_c () / (1.0e3 * nc_hicosmo_H0 (cosmo)));
 }
 
+G_INLINE_FUNC gdouble
+nc_hicosmo_RH_planck (NcHICosmo *cosmo)
+{
+  return nc_hicosmo_RH_Mpc (cosmo) * ncm_c_Mpc () / ncm_c_planck_length ();
+}
+
 G_INLINE_FUNC gdouble
 nc_hicosmo_Omega_k0 (NcHICosmo *cosmo)
 {
@@ -465,20 +461,19 @@ nc_hicosmo_dH_dz (NcHICosmo *cosmo, gdouble z)
 G_INLINE_FUNC gdouble
 nc_hicosmo_q (NcHICosmo *cosmo, gdouble z)
 {
-  gdouble E2, dE2_dz;
-  E2 = nc_hicosmo_E2 (cosmo, z);
-  dE2_dz = nc_hicosmo_dE2_dz (cosmo, z);
+  const gdouble E2     = nc_hicosmo_E2 (cosmo, z);
+  const gdouble dE2_dz = nc_hicosmo_dE2_dz (cosmo, z);
+  
   return (dE2_dz * (1.0 + z) / (2.0 * E2) - 1.0);
 }
 
 G_INLINE_FUNC gdouble
 nc_hicosmo_dec (NcHICosmo *cosmo, gdouble z)
 {
-  gdouble q, E2, Ok;
-  gdouble x = 1.0 + z;
-  q = nc_hicosmo_q (cosmo, z);
-  E2 = nc_hicosmo_E2 (cosmo, z);
-  Ok = nc_hicosmo_Omega_k0 (cosmo);
+  const gdouble x  = 1.0 + z;
+  const gdouble q  = nc_hicosmo_q (cosmo, z);
+  const gdouble E2 = nc_hicosmo_E2 (cosmo, z);
+  const gdouble Ok = nc_hicosmo_Omega_k0 (cosmo);
 
   return (2.0 * E2 - q * E2 - 2.0 * Ok * x * x) / 3.0;
 }
@@ -486,10 +481,9 @@ nc_hicosmo_dec (NcHICosmo *cosmo, gdouble z)
 G_INLINE_FUNC gdouble
 nc_hicosmo_wec (NcHICosmo *cosmo, gdouble z)
 {
-  gdouble E2, Ok;
-  gdouble x = 1.0 + z;
-  E2 = nc_hicosmo_E2 (cosmo, z);
-  Ok = nc_hicosmo_Omega_k0 (cosmo);
+  const gdouble x  = 1.0 + z;
+  const gdouble E2 = nc_hicosmo_E2 (cosmo, z);
+  const gdouble Ok = nc_hicosmo_Omega_k0 (cosmo);
 
   return (E2 - Ok * x * x);
 }
@@ -497,10 +491,9 @@ nc_hicosmo_wec (NcHICosmo *cosmo, gdouble z)
 G_INLINE_FUNC gdouble
 nc_hicosmo_qp (NcHICosmo *cosmo, gdouble z)
 {
-  gdouble E2, dE2_dz, d2E2_dz2;
-  E2 = nc_hicosmo_E2 (cosmo, z);
-  dE2_dz = nc_hicosmo_dE2_dz (cosmo, z);
-  d2E2_dz2 = nc_hicosmo_d2E2_dz2 (cosmo, z);
+  const gdouble E2       = nc_hicosmo_E2 (cosmo, z);
+  const gdouble dE2_dz   = nc_hicosmo_dE2_dz (cosmo, z);
+  const gdouble d2E2_dz2 = nc_hicosmo_d2E2_dz2 (cosmo, z);
 
   return (1.0 + z) / (2.0 * E2) * (dE2_dz / (1.0 + z) + d2E2_dz2 - dE2_dz * dE2_dz / E2);
 }
@@ -508,10 +501,9 @@ nc_hicosmo_qp (NcHICosmo *cosmo, gdouble z)
 G_INLINE_FUNC gdouble
 nc_hicosmo_j (NcHICosmo *cosmo, gdouble z)
 {
-  gdouble E2, dE2_dz, d2E2_dz2;
-  E2 = nc_hicosmo_E2 (cosmo, z);
-  dE2_dz = nc_hicosmo_dE2_dz (cosmo, z);
-  d2E2_dz2 = nc_hicosmo_d2E2_dz2 (cosmo, z);
+  const gdouble E2       = nc_hicosmo_E2 (cosmo, z);
+  const gdouble dE2_dz   = nc_hicosmo_dE2_dz (cosmo, z);
+  const gdouble d2E2_dz2 = nc_hicosmo_d2E2_dz2 (cosmo, z);
 
   return gsl_pow_2 (1.0 + z) * (d2E2_dz2 - 2.0 * dE2_dz / (1.0 + z)) / (2.0 * E2) + 1.0;
 }
@@ -520,15 +512,26 @@ G_INLINE_FUNC gdouble
 nc_hicosmo_x_alpha (NcHICosmo *cosmo, gdouble alpha)
 {
   const gdouble xb = nc_hicosmo_xb (cosmo);
-  return xb * exp (- (alpha * alpha) * 0.5);
+  return xb * exp (- fabs (alpha));
 }
 
 G_INLINE_FUNC gdouble
-nc_hicosmo_abs_alpha (NcHICosmo *cosmo, gdouble z)
+nc_hicosmo_abs_alpha (NcHICosmo *cosmo, gdouble x)
 {
-  const gdouble x  = 1.0 + z;
   const gdouble xb = nc_hicosmo_xb (cosmo);
-  return sqrt (2.0 * log (xb / x));
+  return log (xb / x);
+}
+
+G_INLINE_FUNC NcHIPrim *
+nc_hicosmo_peek_prim (NcHICosmo *cosmo)
+{
+  return cosmo->prim;
+}
+
+G_INLINE_FUNC NcHIReion *
+nc_hicosmo_peek_reion (NcHICosmo *cosmo)
+{
+  return cosmo->reion;
 }
 
 G_END_DECLS
