@@ -38,6 +38,7 @@
 #include "build_cfg.h"
 
 #include "math/ncm_serialize.h"
+#include "nc_hicosmo.h"
 #include "nc_hiprim.h"
 
 enum
@@ -118,7 +119,8 @@ nc_hiprim_class_init (NcHIPrimClass *klass)
                               "NcHIPrim",
                               "Homogeneous and isotropic primordial cosmological models.",
                               NULL,
-                              FALSE);
+                              FALSE,
+                              nc_hicosmo_id ());
 
   ncm_model_class_check_params_info (model_class);
 
@@ -154,7 +156,7 @@ nc_hiprim_new_from_name (GType parent_type, gchar *prim_name)
 
 /**
  * nc_hiprim_ref:
- * @prim: a NcHIPrim
+ * @prim: a #NcHIPrim
  *
  * FIXME
  *
@@ -168,7 +170,7 @@ nc_hiprim_ref (NcHIPrim *prim)
 
 /**
  * nc_hiprim_free:
- * @prim: a NcHIPrim
+ * @prim: a #NcHIPrim
  *
  * FIXME
  *
@@ -181,7 +183,7 @@ nc_hiprim_free (NcHIPrim *prim)
 
 /**
  * nc_hiprim_clear:
- * @prim: a NcHIPrim
+ * @prim: a #NcHIPrim
  *
  * FIXME
  *
@@ -229,7 +231,7 @@ nc_hiprim_log_all_models (GType parent)
 
 /**
  * nc_hiprim_set_k_pivot:
- * @prim: a NcHIPrim
+ * @prim: a #NcHIPrim
  * @k_pivot: pivotal $k$ in units of $1/\mathrm{Mpc}$
  *
  * FIXME
@@ -245,7 +247,7 @@ nc_hiprim_set_k_pivot (NcHIPrim *prim, gdouble k_pivot)
 
 /**
  * nc_hiprim_get_k_pivot:
- * @prim: a NcHIPrim
+ * @prim: a #NcHIPrim
  *
  * Gets the value of the pivotal $k$.
  *
@@ -259,7 +261,7 @@ nc_hiprim_get_k_pivot (NcHIPrim *prim)
 
 /**
  * nc_hiprim_get_lnk_pivot:
- * @prim: a NcHIPrim
+ * @prim: a #NcHIPrim
  *
  * Gets the value of the pivotal $k$.
  *
