@@ -225,72 +225,6 @@ _nc_hipert_set_mode_k (NcHIPert *pert, gdouble k)
   }
 }
 
-/**
- * nc_hipert_set_sys_size:
- * @pert: a #NcHIPert.
- * @sys_size: the system size.
- *
- * Sets the system size.
- *
- */
-void
-nc_hipert_set_sys_size (NcHIPert *pert, guint sys_size)
-{
-  if (pert->sys_size != sys_size)
-  {
-    if (pert->y != NULL)
-    {
-      N_VDestroy (pert->y);
-      pert->y = NULL;
-    }
-    if (pert->vec_abstol != NULL)
-    {
-      N_VDestroy (pert->vec_abstol);
-      pert->vec_abstol = NULL;
-    }
-
-    pert->sys_size = sys_size;
-    if (pert->sys_size > 0)
-    {
-      pert->y          = N_VNew_Serial (sys_size);
-      pert->vec_abstol = N_VNew_Serial (sys_size);
-    }
-    pert->prepared = FALSE;
-  }
-}
-
-/**
- * nc_hipert_set_stiff_solver:
- * @pert: a #NcHIPert.
- * @stiff: whenever to enable or disable a stiff solver.
- *
- * Sets the ode algorithm to use.
- *
- */
-void
-nc_hipert_set_stiff_solver (NcHIPert *pert, gboolean stiff)
-{
-  guint a = stiff ? 1 : 0;
-  guint b = pert->cvode_stiff ? 1 : 0;
-  if (a != b)
-  {
-    if (pert->cvode != NULL)
-    {
-      CVodeFree (&pert->cvode);
-      pert->cvode = NULL;
-    }
-    pert->cvode_stiff = stiff;
-
-    if (stiff)
-      pert->cvode = CVodeCreate (CV_BDF, CV_NEWTON);
-    else
-      pert->cvode = CVodeCreate (CV_ADAMS, CV_FUNCTIONAL);
-
-    pert->cvode_init = FALSE;
-  }
-}
-
-
 /**
  * nc_hipert_set_reltol:
  * @pert: a #NcHIPert.
@@ -335,12 +269,6 @@ _nc_hipert_set_abstol (NcHIPert *pert, gdouble abstol)
  *
  * Returns: reltol
  */
-gdouble
-nc_hipert_get_reltol (NcHIPert *pert)
-{
-  return pert->reltol;
-}
-
 /**
  * nc_hipert_get_abstol:
  * @pert: a #NcHIPert.
@@ -348,8 +276,95 @@ nc_hipert_get_reltol (NcHIPert *pert)
  * Gets the value of the relative tolerance.
  *
  */
-gdouble
-nc_hipert_get_abstol (NcHIPert *pert)
+
+/**
+ * nc_hipert_set_sys_size:
+ * @pert: a #NcHIPert.
+ * @sys_size: the system size.
+ *
+ * Sets the system size.
+ *
+ */
+void
+nc_hipert_set_sys_size (NcHIPert *pert, guint sys_size)
 {
-  return pert->abstol;
+  if (pert->sys_size != sys_size)
+  {
+    if (pert->y != NULL)
+    {
+      N_VDestroy (pert->y);
+      pert->y = NULL;
+    }
+    if (pert->vec_abstol != NULL)
+    {
+      N_VDestroy (pert->vec_abstol);
+      pert->vec_abstol = NULL;
+    }
+
+    pert->sys_size = sys_size;
+    if (pert->sys_size > 0)
+    {
+      pert->y          = N_VNew_Serial (sys_size);
+      pert->vec_abstol = N_VNew_Serial (sys_size);
+    }
+    pert->prepared = FALSE;
+
+    nc_hipert_reset_solver (pert);
+  }
 }
+
+/**
+ * nc_hipert_set_stiff_solver:
+ * @pert: a #NcHIPert.
+ * @stiff: whenever to enable or disable a stiff solver.
+ *
+ * Sets the ode algorithm to use.
+ *
+ */
+void
+nc_hipert_set_stiff_solver (NcHIPert *pert, gboolean stiff)
+{
+  guint a = stiff ? 1 : 0;
+  guint b = pert->cvode_stiff ? 1 : 0;
+  if (a != b)
+  {
+    if (pert->cvode != NULL)
+    {
+      CVodeFree (&pert->cvode);
+      pert->cvode = NULL;
+    }
+    pert->cvode_stiff = stiff;
+
+    if (stiff)
+      pert->cvode = CVodeCreate (CV_BDF, CV_NEWTON);
+    else
+      pert->cvode = CVodeCreate (CV_ADAMS, CV_FUNCTIONAL);
+
+    pert->cvode_init = FALSE;
+  }
+}
+
+/**
+ * nc_hipert_reset_solver:
+ * @pert: a #NcHIPert
+ *
+ * Destroy and recreates the solver.
+ *
+ */
+void
+nc_hipert_reset_solver (NcHIPert *pert)
+{
+  if (pert->cvode != NULL)
+  {
+    CVodeFree (&pert->cvode);
+    pert->cvode = NULL;
+  }
+
+  if (pert->cvode_stiff)
+    pert->cvode = CVodeCreate (CV_BDF, CV_NEWTON);
+  else
+    pert->cvode = CVodeCreate (CV_ADAMS, CV_FUNCTIONAL);
+
+  pert->cvode_init = FALSE;
+}
+
