@@ -41,7 +41,6 @@
 #include "math/ncm_serialize.h"
 #include "math/ncm_cfg.h"
 #include "lss/nc_window_tophat.h"
-#include "lss/nc_matter_var.h"
 #include "xcor/nc_xcor.h"
 
 enum
@@ -111,21 +110,13 @@ void nc_xcor_clear (NcXcor** xcl)
 	g_clear_object (xcl);
 }
 
-void nc_xcor_prepare (NcXcor *xc, NcHIReion *reion, NcHICosmo *cosmo)
+void nc_xcor_prepare (NcXcor *xc, NcHICosmo *cosmo)
 {
-	nc_transfer_func_prepare (xc->tf, reion, cosmo);
+	nc_transfer_func_prepare (xc->tf, cosmo);
 	nc_growth_func_prepare (xc->gf, cosmo);
 	nc_distance_prepare_if_needed (xc->dist, cosmo);
-
-	NcWindow* wp = nc_window_tophat_new ();
-	NcMatterVar* var = nc_matter_var_new (NC_MATTER_VAR_FFT, wp, xc->tf);
-	gdouble ln8_hMpc = log (8.0);
-	gdouble sigma82_no_norm = nc_matter_var_var0 (var, cosmo, ln8_hMpc);
-	gdouble sigma82 = pow (nc_hicosmo_sigma_8 (cosmo), 2.0);
-	xc->normPS = sigma82 / sigma82_no_norm;
 }
 
-
 typedef struct _xcor_limber_cross_cl_int
 {
 	NcHICosmo* cosmo;
@@ -147,14 +138,16 @@ static gdouble _xcor_limber_cross_cl_int_z (gdouble z, gpointer ptr)
 	gdouble k2z = nc_xcor_limber_eval_kernel (xcli->xcl2, xcli->cosmo, z, xcli->l);
 
 	gdouble xi_z = nc_distance_comoving (xcli->dist, xcli->cosmo, z); // dimensionless, ie it's in unit of hubble radius
-	gdouble kh = xcli->l / (xi_z * ncm_c_hubble_radius ()); // in h Mpc-1, hubble
-	gdouble power_spec_init = nc_hicosmo_powspec (xcli->cosmo, kh); // k^ns
+	gdouble kh = xcli->l / (xi_z * ncm_c_hubble_radius_hm1_Mpc ()); // in h Mpc-1, hubble
+	gdouble power_spec_init = 1.0;/*nc_hicosmo_powspec (xcli->cosmo, kh);*/ // k^ns
 	gdouble t_k = nc_transfer_func_eval (xcli->tf, xcli->cosmo, kh);
 	gdouble D_z = nc_growth_func_eval (xcli->gf, xcli->cosmo, z);
 	gdouble power_spec = power_spec_init * t_k * t_k * D_z * D_z;
 
 	gdouble E_z = nc_hicosmo_E (xcli->cosmo, z);
 
+g_assert_not_reached ();
+  
 	return E_z * k1z * k2z * power_spec / (xi_z * xi_z);
 }
 
@@ -163,7 +156,6 @@ static gdouble _xcor_limber_cross_cl_int_z (gdouble z, gpointer ptr)
  * @xc: a #NcXcor
  * @xcl1: a #NcXcorLimber
  * @xcl2: a #NcXcorLimber
- * @reion: a #NcHIReion
  * @cosmo: a #NcHICosmo
  * @ell: a #NcmVector
  * @vp: a #NcmVector
@@ -171,11 +163,9 @@ static gdouble _xcor_limber_cross_cl_int_z (gdouble z, gpointer ptr)
  *
  * FIXME
  *
- * Returns: FIXME
- *
-*/
+ */
 void 
-nc_xcor_limber_cross_cl (NcXcor* xc, NcXcorLimber* xcl1, NcXcorLimber* xcl2, NcHIReion *reion, NcHICosmo* cosmo, NcmVector* ell, NcmVector* vp, guint lmin_idx)
+nc_xcor_limber_cross_cl (NcXcor* xc, NcXcorLimber* xcl1, NcXcorLimber* xcl2, NcHICosmo* cosmo, NcmVector* ell, NcmVector* vp, guint lmin_idx)
 {
 	guint nell = ncm_vector_len (ell);
 	guint nvp = ncm_vector_len (vp);
@@ -192,7 +182,7 @@ nc_xcor_limber_cross_cl (NcXcor* xc, NcXcorLimber* xcl1, NcXcorLimber* xcl2, NcH
 	{
 		nc_xcor_limber_prepare (xcl1, cosmo);
 		nc_xcor_limber_prepare (xcl2, cosmo);
-		nc_xcor_prepare (xc, reion, cosmo);
+		nc_xcor_prepare (xc, cosmo);
 	}
 
 	gdouble cl, cons_factor, err;
@@ -249,15 +239,17 @@ static gdouble _xcor_limber_auto_cl_int_z (gdouble z, gpointer ptr)
 	gdouble k1z = nc_xcor_limber_eval_kernel (xcli->xcl, xcli->cosmo, z, xcli->l);
 
 	gdouble xi_z = nc_distance_comoving (xcli->dist, xcli->cosmo, z); // dimensionless, ie it's in unit of hubble radius
-	gdouble kh = (xcli->l + 0.5) / (xi_z * ncm_c_hubble_radius ()); // in h Mpc-1, hubble
+	gdouble kh = (xcli->l + 0.5) / (xi_z * ncm_c_hubble_radius_hm1_Mpc ()); // in h Mpc-1, hubble
 	// radius = c[m/s] / (the +0.5 is not to be forgotten, cf. LoVerde, M., & Afshordi, N. (2008). Extended Limber approximation. Physical Review D, 78(1), 123506. http://doi.org/10.1103/PhysRevD.78.123506)
-	gdouble power_spec_init = nc_hicosmo_powspec (xcli->cosmo, kh); // k^ns
+	gdouble power_spec_init = 1.0;/*nc_hicosmo_powspec (xcli->cosmo, kh);*/ // k^ns
 	gdouble t_k = nc_transfer_func_eval (xcli->tf, xcli->cosmo, kh);
 	gdouble D_z = nc_growth_func_eval (xcli->gf, xcli->cosmo, z);
 	gdouble power_spec = power_spec_init * t_k * t_k * D_z * D_z;
 
 	gdouble E_z = nc_hicosmo_E (xcli->cosmo, z);
 
+g_assert_not_reached ();
+  
 	return E_z * k1z * k1z * power_spec / (xi_z * xi_z);
 }
 
@@ -265,7 +257,6 @@ static gdouble _xcor_limber_auto_cl_int_z (gdouble z, gpointer ptr)
  * nc_xcor_limber_auto_cl:
  * @xc: a #NcXcor
  * @xcl: a #NcXcorLimber
- * @reion: a #NcHIReion
  * @cosmo: a #NcHICosmo
  * @ell: a #NcmVector
  * @vp: a #NcmVector
@@ -274,11 +265,9 @@ static gdouble _xcor_limber_auto_cl_int_z (gdouble z, gpointer ptr)
  *
  * FIXME
  *
- * Returns: FIXME
- *
-*/
+ */
 void 
-nc_xcor_limber_auto_cl (NcXcor* xc, NcXcorLimber* xcl, NcHIReion *reion, NcHICosmo* cosmo, NcmVector* ell, NcmVector* vp, guint lmin_idx, gboolean withnoise)
+nc_xcor_limber_auto_cl (NcXcor* xc, NcXcorLimber* xcl, NcHICosmo* cosmo, NcmVector* ell, NcmVector* vp, guint lmin_idx, gboolean withnoise)
 {
 	guint nell = ncm_vector_len (ell);
 	guint nvp = ncm_vector_len (vp);
@@ -293,7 +282,7 @@ nc_xcor_limber_auto_cl (NcXcor* xc, NcXcorLimber* xcl, NcHIReion *reion, NcHICos
 	if (xcl_up || cosmo_up)
 	{
 		nc_xcor_limber_prepare (xcl, cosmo);
-		nc_xcor_prepare (xc, reion, cosmo);
+		nc_xcor_prepare (xc, cosmo);
 	}
 
 	gdouble cl, cons_factor, err;
