@@ -54,11 +54,18 @@ typedef struct _TestNcmStatsVec
 void test_ncm_stats_vec_mean_new (TestNcmStatsVec *test, gconstpointer pdata);
 void test_ncm_stats_vec_var_new (TestNcmStatsVec *test, gconstpointer pdata);
 void test_ncm_stats_vec_cov_new (TestNcmStatsVec *test, gconstpointer pdata);
+void test_ncm_stats_vec_autocorr_new (TestNcmStatsVec *test, gconstpointer pdata);
 void test_ncm_stats_vec_mean_test (TestNcmStatsVec *test, gconstpointer pdata);
 void test_ncm_stats_vec_var_test (TestNcmStatsVec *test, gconstpointer pdata);
 void test_ncm_stats_vec_cov_test (TestNcmStatsVec *test, gconstpointer pdata);
+void test_ncm_stats_vec_autocorr_test (TestNcmStatsVec *test, gconstpointer pdata);
+void test_ncm_stats_vec_subsample_autocorr_test (TestNcmStatsVec *test, gconstpointer pdata);
 void test_ncm_stats_vec_free (TestNcmStatsVec *test, gconstpointer pdata);
 
+void test_ncm_stats_vec_traps (TestNcmStatsVec *test, gconstpointer pdata);
+void test_ncm_stats_vec_invalid_get_cov (TestNcmStatsVec *test, gconstpointer pdata);
+void test_ncm_stats_vec_invalid_get_var (TestNcmStatsVec *test, gconstpointer pdata);
+
 gint
 main (gint argc, gchar *argv[])
 {
@@ -68,18 +75,46 @@ main (gint argc, gchar *argv[])
 
   /* Default vector allocation */
 
-  g_test_add ("/numcosmo/ncm_stats_vec/mean", TestNcmStatsVec, NULL, 
+  g_test_add ("/ncm/stats_vec/mean", TestNcmStatsVec, NULL, 
               &test_ncm_stats_vec_mean_new, 
               &test_ncm_stats_vec_mean_test, 
               &test_ncm_stats_vec_free);
-  g_test_add ("/numcosmo/ncm_stats_vec/var", TestNcmStatsVec, NULL, 
+  g_test_add ("/ncm/stats_vec/var", TestNcmStatsVec, NULL, 
               &test_ncm_stats_vec_var_new, 
               &test_ncm_stats_vec_var_test, 
               &test_ncm_stats_vec_free);
-  g_test_add ("/numcosmo/ncm_stats_vec/cov", TestNcmStatsVec, NULL, 
+  g_test_add ("/ncm/stats_vec/cov", TestNcmStatsVec, NULL, 
               &test_ncm_stats_vec_cov_new, 
               &test_ncm_stats_vec_cov_test, 
               &test_ncm_stats_vec_free);
+  g_test_add ("/ncm/stats_vec/autocorr", TestNcmStatsVec, NULL, 
+              &test_ncm_stats_vec_autocorr_new, 
+              &test_ncm_stats_vec_autocorr_test, 
+              &test_ncm_stats_vec_free);
+  g_test_add ("/ncm/stats_vec/subsample_autocorr", TestNcmStatsVec, NULL, 
+              &test_ncm_stats_vec_autocorr_new, 
+              &test_ncm_stats_vec_subsample_autocorr_test, 
+              &test_ncm_stats_vec_free);
+  
+#if !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 38))
+  g_test_add ("/ncm/stats_vec/mean/get_var/subprocess", TestNcmStatsVec, NULL, 
+              &test_ncm_stats_vec_mean_new, 
+              &test_ncm_stats_vec_invalid_get_var, 
+              &test_ncm_stats_vec_free);
+  g_test_add ("/ncm/stats_vec/mean/get_cov/subprocess", TestNcmStatsVec, NULL, 
+              &test_ncm_stats_vec_mean_new, 
+              &test_ncm_stats_vec_invalid_get_cov, 
+              &test_ncm_stats_vec_free);
+  g_test_add ("/ncm/stats_vec/var/get_cov/subprocess", TestNcmStatsVec, NULL, 
+              &test_ncm_stats_vec_var_new, 
+              &test_ncm_stats_vec_invalid_get_cov,
+              &test_ncm_stats_vec_free);
+#endif
+  
+  g_test_add ("/ncm/stats_vec/traps", TestNcmStatsVec, NULL, 
+              &test_ncm_stats_vec_var_new, 
+              &test_ncm_stats_vec_traps, 
+              &test_ncm_stats_vec_free);
 
   g_test_run ();
 }
@@ -123,6 +158,19 @@ test_ncm_stats_vec_cov_new (TestNcmStatsVec *test, gconstpointer pdata)
   g_assert (NCM_IS_STATS_VEC (test->svec));
 }
 
+void
+test_ncm_stats_vec_autocorr_new (TestNcmStatsVec *test, gconstpointer pdata)
+{
+  test->v_size = g_test_rand_int_range (_TEST_NCM_VECTOR_MIN_SIZE, _TEST_NCM_VECTOR_STATIC_SIZE);
+  test->ntests = g_test_rand_int_range (_TEST_NCM_STATS_VEC_NTEST_MIN, _TEST_NCM_STATS_VEC_NTEST_MAX) * 100;
+  test->svec   = ncm_stats_vec_new (test->v_size, NCM_STATS_VEC_COV, TRUE);
+  test->xs     = ncm_matrix_new (test->ntests, test->v_size);
+  test->mu     = ncm_vector_new (test->v_size);
+  test->w      = ncm_vector_new (test->ntests);
+
+  g_assert (NCM_IS_STATS_VEC (test->svec));
+}
+
 void
 test_ncm_stats_vec_free (TestNcmStatsVec *test, gconstpointer pdata)
 {
@@ -131,14 +179,8 @@ test_ncm_stats_vec_free (TestNcmStatsVec *test, gconstpointer pdata)
   ncm_matrix_clear (&test->xs);
   ncm_vector_clear (&test->w);
   ncm_vector_clear (&test->mu);
-  
-  ncm_stats_vec_free (svec);
-  if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDOUT | G_TEST_TRAP_SILENCE_STDERR))
-  {
-    ncm_stats_vec_free (svec);
-    exit (0);
-  }
-  g_test_trap_assert_failed ();
+
+  NCM_TEST_FREE (ncm_stats_vec_free, svec);
 }
 
 void
@@ -181,20 +223,7 @@ test_ncm_stats_vec_mean_test (TestNcmStatsVec *test, gconstpointer pdata)
     }
   }
 
-  if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDOUT | G_TEST_TRAP_SILENCE_STDERR))
-  {
-    ncm_stats_vec_get_var (test->svec, 0);
-    exit (0);
-  }
-  g_test_trap_assert_failed ();
-
-  if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDOUT | G_TEST_TRAP_SILENCE_STDERR))
-  {
-    ncm_stats_vec_get_cov (test->svec, 0, 1);
-    exit (0);
-  }
-  g_test_trap_assert_failed ();
-  
+  NCM_TEST_FAIL (ncm_stats_vec_get_var (test->svec, 0));  
 }
 
 void
@@ -241,13 +270,7 @@ test_ncm_stats_vec_var_test (TestNcmStatsVec *test, gconstpointer pdata)
     }
   }
 
-  if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDOUT | G_TEST_TRAP_SILENCE_STDERR))
-  {
-    ncm_stats_vec_get_cov (test->svec, 0, 1);
-    exit (0);
-  }
-  g_test_trap_assert_failed ();
-  
+  NCM_TEST_FAIL (ncm_stats_vec_get_cov (test->svec, 0, 1));  
 }
 
 void
@@ -309,3 +332,152 @@ test_ncm_stats_vec_cov_test (TestNcmStatsVec *test, gconstpointer pdata)
     }
   }  
 }
+
+void
+test_ncm_stats_vec_autocorr_test (TestNcmStatsVec *test, gconstpointer pdata)
+{
+  NcmRNG *rng = ncm_rng_pool_get ("test_ncm_stats_vec");
+  const gdouble a = 0.9 + fabs (g_test_rand_double ()) * 1.0e-2;
+  const gdouble sigma = fabs (g_test_rand_double ()) * 1.0e-1;
+  NcmVector *last = ncm_vector_new (test->v_size);
+  guint i;
+
+  for (i = 0; i < test->v_size; i++)
+  {
+    ncm_vector_set (test->mu, i, 1.0 + fabs (g_test_rand_double ()));
+    ncm_vector_set (last, i, 0.0);
+  }
+
+  for (i = 0; i < test->ntests; i++)
+  {  
+    guint j;
+    for (j = 0; j < test->v_size; j++)
+    {
+      const gdouble epsilon_j = ncm_vector_get (test->mu, j) + sigma * gsl_ran_ugaussian (rng->r);
+      const gdouble x_j       = (a * ncm_vector_get (last, j) + epsilon_j);
+
+      ncm_vector_set (last, j, x_j);
+
+      ncm_stats_vec_set (test->svec, j, x_j);
+      ncm_matrix_set (test->xs, i, j, x_j);
+    }
+    ncm_stats_vec_update (test->svec);
+  }
+
+  for (i = 0; i < test->v_size; i++)
+  {
+    const gdouble gsl_mean = gsl_stats_mean (ncm_matrix_ptr (test->xs, 0, i), test->v_size, 
+                                       test->ntests);
+    const gdouble gsl_var = gsl_stats_variance (ncm_matrix_ptr (test->xs, 0, i), test->v_size, 
+                                          test->ntests);
+    const gdouble svec_mean = ncm_stats_vec_get_mean (test->svec, i);
+    const gdouble svec_var = ncm_stats_vec_get_var (test->svec, i);
+
+    ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC);
+    ncm_assert_cmpdouble_e (gsl_var, ==, svec_var, _TEST_NCM_STATS_VEC_PREC);
+
+    {
+      NcmVector *ac = ncm_stats_vec_get_autocorr (test->svec, i);
+      guint j;
+      guint tsize = GSL_MIN (10, ncm_vector_len (ac));
+      for (j = 0; j < tsize; j++)
+      {
+        if (ncm_vector_get (ac, j) < 0.0)
+          break;
+        ncm_assert_cmpdouble_e (ncm_vector_get (ac, j), ==, pow (a, j), 1.0e-1);
+      }
+      ncm_vector_free (ac);
+    }
+  }
+  
+  ncm_vector_free (last);
+}
+
+void
+test_ncm_stats_vec_subsample_autocorr_test (TestNcmStatsVec *test, gconstpointer pdata)
+{
+  NcmRNG *rng = ncm_rng_pool_get ("test_ncm_stats_vec");
+  const gdouble a = 0.9 + fabs (g_test_rand_double ()) * 1.0e-2;
+  const gdouble sigma = fabs (g_test_rand_double ()) * 1.0e-1;
+  const guint nchains = g_test_rand_int_range (10, 20);
+  NcmMatrix *last = ncm_matrix_new (nchains, test->v_size);
+  guint i;
+
+  ncm_matrix_set_zero (last);
+  for (i = 0; i < test->v_size; i++)
+  {
+    ncm_vector_set (test->mu, i, 1.0 + fabs (g_test_rand_double ()));
+  }
+
+  for (i = 0; i < test->ntests; i++)
+  {  
+    guint j;
+    guint chain_id = i % (nchains);
+    for (j = 0; j < test->v_size; j++)
+    {
+      const gdouble epsilon_j = ncm_vector_get (test->mu, j) + sigma * gsl_ran_ugaussian (rng->r);
+      const gdouble x_j       = (a * ncm_matrix_get (last, chain_id, j) + epsilon_j);
+
+      ncm_matrix_set (last, chain_id, j, x_j);
+
+      ncm_stats_vec_set (test->svec, j, x_j);
+      ncm_matrix_set (test->xs, i, j, x_j);
+    }
+    ncm_stats_vec_update (test->svec);
+  }
+
+  for (i = 0; i < test->v_size; i++)
+  {
+    const gdouble gsl_mean = gsl_stats_mean (ncm_matrix_ptr (test->xs, 0, i), test->v_size, 
+                                       test->ntests);
+    const gdouble gsl_var = gsl_stats_variance (ncm_matrix_ptr (test->xs, 0, i), test->v_size, 
+                                          test->ntests);
+    const gdouble svec_mean = ncm_stats_vec_get_mean (test->svec, i);
+    const gdouble svec_var = ncm_stats_vec_get_var (test->svec, i);
+
+    ncm_assert_cmpdouble_e (gsl_mean, ==, svec_mean, _TEST_NCM_STATS_VEC_PREC);
+    ncm_assert_cmpdouble_e (gsl_var, ==, svec_var, _TEST_NCM_STATS_VEC_PREC);
+
+    {
+      NcmVector *ac = ncm_stats_vec_get_subsample_autocorr (test->svec, i, nchains);
+      guint j;
+      guint tsize = GSL_MIN (10, ncm_vector_len (ac));
+      for (j = 0; j < tsize; j++)
+      {
+        if (ncm_vector_get (ac, j) < 0.0)
+          break;
+        ncm_assert_cmpdouble_e (ncm_vector_get (ac, j), ==, pow (a, j), 1.0e-1);
+      }
+      ncm_vector_free (ac);
+    }
+  }
+  
+  ncm_matrix_free (last);
+}
+
+void
+test_ncm_stats_vec_invalid_get_var (TestNcmStatsVec *test, gconstpointer pdata)
+{
+  ncm_stats_vec_get_var (test->svec, 0);  
+}
+
+void
+test_ncm_stats_vec_invalid_get_cov (TestNcmStatsVec *test, gconstpointer pdata)
+{
+  ncm_stats_vec_get_cov (test->svec, 0, 1);  
+}
+
+void 
+test_ncm_stats_vec_traps (TestNcmStatsVec *test, gconstpointer pdata)
+{
+#if !((GLIB_MAJOR_VERSION == 2) && (GLIB_MINOR_VERSION < 38))
+  g_test_trap_subprocess ("/ncm/stats_vec/mean/get_var/subprocess", 0, 0);
+  g_test_trap_assert_failed ();
+  
+  g_test_trap_subprocess ("/ncm/stats_vec/mean/get_cov/subprocess", 0, 0);
+  g_test_trap_assert_failed ();
+  
+  g_test_trap_subprocess ("/ncm/stats_vec/var/get_cov/subprocess", 0, 0);
+  g_test_trap_assert_failed ();
+#endif
+}
