@@ -40,7 +40,7 @@
 gint
 main (gint argc, gchar *argv[])
 {
-  NcHICosmo *model;
+  NcHICosmo *cosmo;
   NcmDataset *dset;
   NcmLikelihood *lh;
   NcmFit *fit;
@@ -51,6 +51,7 @@ main (gint argc, gchar *argv[])
   NcDEFitEntries de_fit = NC_DE_FIT_ENTRIES;
   NcDistance *dist;
   NcmMSet *mset, *fiduc = NULL;
+  NcmObjArray *funcs_oa = NULL;
   GError *error = NULL;
   GOptionContext *context;
   GOptionEntry *de_model_entries = NULL;
@@ -81,14 +82,15 @@ main (gint argc, gchar *argv[])
     gint i;
     for (i = 0; i < argc; i++)
     {
-      if (strncmp (argv[i], "--runconf", 9) == 0 || strncmp (argv[i], "-c", 2) == 0)
+      if ((strncmp (argv[i], "--runconf", 9) == 0) || (strcmp (argv[i], "-c") == 0))
       {
-        if (strlen (argv[i]) == 9)
+        guint rc_size = strlen (argv[i]);
+        if ((rc_size == 9) || (rc_size == 2))
         {
           if (i + 1 == argc || argv[i + 1] == NULL)
           {
             gchar *msg = g_option_context_get_help (context, TRUE, NULL);
-            fprintf (stderr, "Invalid run options:\n");
+            fprintf (stderr, "Invalid run options: missing argument for `%s'\n", argv[i]);
             printf ("%s", msg);
             g_free (msg);
             g_option_context_free (context);
@@ -97,7 +99,7 @@ main (gint argc, gchar *argv[])
           else
             de_run.runconf = argv[i + 1];
         }
-        else if (argv[i][9] == '=')
+        else if ((rc_size > 10) && (argv[i][9] == '='))
         {
           de_run.runconf = &argv[i][10];
         }
@@ -205,11 +207,12 @@ main (gint argc, gchar *argv[])
   if (de_run.nthreads > 0)
     ncm_func_eval_set_max_threads (de_run.nthreads);
   
-  if (de_data_simple.snia_id == NULL &&
-      de_data_simple.cmb_id == NULL &&
-      de_data_simple.bao_id == NULL &&
-      de_data_simple.H_id == NULL &&
+  if (de_data_simple.snia_id    == NULL &&
+      de_data_simple.cmb_id     == NULL &&
+      de_data_simple.bao_id     == NULL &&
+      de_data_simple.H_id       == NULL &&
       de_data_simple.cluster_id == NULL &&
+      de_data_simple.Planck     == NULL &&
       de_data_simple.data_files == NULL)
   {
     printf ("No action or data were chosen.\n");
@@ -223,22 +226,61 @@ main (gint argc, gchar *argv[])
   ncm_message ("# NumCosmo Version -- "NUMCOSMO_VERSION"\n");
   ncm_message ("# Command Line: %s\n", full_cmd_line);
 
-  dset = ncm_dataset_new ();
-  model = nc_hicosmo_new_from_name (NC_TYPE_HICOSMO, de_model.model_name);
-  mset = ncm_mset_new (NCM_MODEL (model), NULL);
   dist = nc_distance_new (2.0);
-  
   ncm_serialize_global_set (dist, "MainDist", FALSE);
 
-  if (g_type_is_a (G_OBJECT_TYPE (model), NC_TYPE_HICOSMO_DE))
+  if (de_model.mset_file != NULL)
+    mset = ncm_mset_load (de_model.mset_file, ser);
+  else
+    mset = ncm_mset_empty_new ();
+
+  dset  = ncm_dataset_new ();
+  if ((cosmo = NC_HICOSMO (ncm_mset_get (mset, nc_hicosmo_id ()))) == NULL)
+  {
+    if (de_model.model_name != NULL)
+      cosmo = nc_hicosmo_new_from_name (NC_TYPE_HICOSMO, de_model.model_name);
+    else
+      cosmo = NC_HICOSMO (nc_hicosmo_de_xcdm_new ());
+  }
+
+  if (ncm_mset_peek (mset, nc_hireion_id ()) == NULL)
+  {
+    NcHIReion *reion;
+
+    if (de_model.model_reion != NULL)
+      reion = nc_hireion_new_from_name (NC_TYPE_HIREION, de_model.model_reion);
+    else
+      reion = NC_HIREION (nc_hireion_camb_new ());
+    
+    ncm_model_add_submodel (NCM_MODEL (cosmo), NCM_MODEL (reion));
+    nc_hireion_free (reion);
+  }
+
+  if (ncm_mset_peek (mset, nc_hiprim_id ()) == NULL)
+  {
+    NcHIPrim *prim;
+
+    if (de_model.model_prim != NULL)
+      prim = nc_hiprim_new_from_name (NC_TYPE_HIPRIM, de_model.model_prim);
+    else
+      prim = NC_HIPRIM (nc_hiprim_power_law_new ());
+    
+    ncm_model_add_submodel (NCM_MODEL (cosmo), NCM_MODEL (prim));
+    nc_hiprim_free (prim);
+  }
+  
+  if (ncm_mset_peek (mset, nc_hicosmo_id ()) == NULL)
+    ncm_mset_push (mset, NCM_MODEL (cosmo));
+
+  if (g_type_is_a (G_OBJECT_TYPE (cosmo), NC_TYPE_HICOSMO_DE))
     is_de = TRUE;
 
   if (de_model.help_names)
   {
     gint i;
-    ncm_message ("# Model name -- %s\n", ncm_model_name (NCM_MODEL (model)));
-    for (i = 0; i < ncm_model_len (NCM_MODEL (model)); i++)
-      ncm_message ("# Model parameter [%02d] = %s\n", i, ncm_model_param_name (NCM_MODEL (model), i));
+    ncm_message ("# Cosmological model name -- %s\n", ncm_model_name (NCM_MODEL (cosmo)));
+    for (i = 0; i < ncm_model_len (NCM_MODEL (cosmo)); i++)
+      ncm_message ("# Model parameter [%02d] = %s\n", i, ncm_model_param_name (NCM_MODEL (cosmo), i));
     return 0;
   }
 
@@ -248,15 +290,15 @@ main (gint argc, gchar *argv[])
   {
     if (!is_de)
       g_error ("flat option is valid only for darkenergy models");
-    nc_hicosmo_de_omega_x2omega_k (NC_HICOSMO_DE (model));
-    ncm_model_param_set (NCM_MODEL (model), NC_HICOSMO_DE_OMEGA_X, 0.0);
+    nc_hicosmo_de_omega_x2omega_k (NC_HICOSMO_DE (cosmo));
+    ncm_model_param_set (NCM_MODEL (cosmo), NC_HICOSMO_DE_OMEGA_X, 0.0);
     ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X, NCM_PARAM_TYPE_FIXED);
   }
   else if (de_model.Omega_k)
   {
     if (!is_de)
       g_error ("omegak option is valid only for darkenergy models");
-    nc_hicosmo_de_omega_x2omega_k (NC_HICOSMO_DE (model));
+    nc_hicosmo_de_omega_x2omega_k (NC_HICOSMO_DE (cosmo));
     ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X, NCM_PARAM_TYPE_FREE);
   }
 
@@ -264,7 +306,7 @@ main (gint argc, gchar *argv[])
   {
     if (!is_de)
       g_error ("omegak > 0 option is valid only for darkenergy models");
-    ncm_prior_add_positive (lh, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X);
+    ncm_likelihood_priors_add_flat_param (lh, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X, 0.0, HUGE_VAL, 1.0);
   }
   
   if (de_data_simple.snia_id != NULL)
@@ -284,7 +326,6 @@ main (gint argc, gchar *argv[])
     }
     else if (snia_id->value >= NC_DATA_SNIA_COV_START && snia_id->value <= NC_DATA_SNIA_COV_END)
     {
-      NcSNIADistCov *dcov;
       NcmData *data = nc_data_snia_cov_new (de_data_simple.snia_use_det);
       guint sigma_int_len;
       
@@ -292,23 +333,25 @@ main (gint argc, gchar *argv[])
 
       sigma_int_len = nc_data_snia_cov_sigma_int_len (NC_DATA_SNIA_COV (data));
 
-      if (de_data_simple.snia_objser == NULL)
-        dcov = nc_snia_dist_cov_new (dist, sigma_int_len);
-      else
-        dcov = NC_SNIA_DIST_COV (ncm_serialize_global_from_string (de_data_simple.snia_objser));
+      if (ncm_mset_peek (mset, nc_snia_dist_cov_id ()) == NULL)
+      {
+        NcSNIADistCov *dcov;
+        if (de_data_simple.snia_objser == NULL)
+          dcov = nc_snia_dist_cov_new (dist, sigma_int_len);
+        else
+          dcov = NC_SNIA_DIST_COV (ncm_serialize_global_from_string (de_data_simple.snia_objser));
 
-      g_assert (NC_IS_SNIA_DIST_COV (dcov));
-      
-      ncm_mset_set (mset, NCM_MODEL (dcov));
+        g_assert (NC_IS_SNIA_DIST_COV (dcov));
+        ncm_mset_set (mset, NCM_MODEL (dcov));
+        nc_snia_dist_cov_free (dcov);
+      }
       ncm_dataset_append_data (dset, data);
       ncm_data_free (data);
-      nc_snia_dist_cov_free (dcov);
     }
     else
       g_assert_not_reached ();
   }
 
- 
   if (de_data_simple.cmb_id != NULL)
   {
     const GEnumValue *cmb_id = ncm_cfg_get_enum_by_id_name_nick (NC_TYPE_DATA_CMB_ID,
@@ -395,6 +438,31 @@ main (gint argc, gchar *argv[])
 
   }
 
+  if (de_data_simple.Planck != NULL)
+  {
+    guint i;
+    guint nPlanck = g_strv_length (de_data_simple.Planck);
+    NcHIPertBoltzmannCBE *boltz = nc_hipert_boltzmann_cbe_new ();
+
+    if (ncm_mset_peek (mset, nc_planck_fi_id ()) == NULL)
+    {
+      NcPlanckFI *planck_fi = nc_planck_fi_new_from_name (de_data_simple.PlanckFI == NULL ? "NcPlanckFICorTT" : de_data_simple.PlanckFI);
+      ncm_mset_push (mset, NCM_MODEL (planck_fi));
+      nc_planck_fi_free (planck_fi);
+    }
+    
+    for (i = 0; i < nPlanck; i++)
+    {
+      gchar *Planck_i       = de_data_simple.Planck[i];
+      NcDataPlanckLKL *plik = nc_data_planck_lkl_full_new (Planck_i, NC_HIPERT_BOLTZMANN (boltz));
+      ncm_dataset_append_data (dset, NCM_DATA (plik));
+
+      ncm_data_free (NCM_DATA (plik));
+    }
+
+    nc_hipert_boltzmann_cbe_free (boltz);
+  }
+
   if (de_data_simple.data_files != NULL)
   {
     guint ndata_files = g_strv_length (de_data_simple.data_files);
@@ -422,33 +490,62 @@ main (gint argc, gchar *argv[])
 
   if (de_data_simple.BBN_Ob)
   {
-    NcmMSetFunc *Omega_b0h2 = nc_hicosmo_create_mset_func0 (&nc_hicosmo_Omega_b0h2);
-    ncm_prior_add_gaussian_const_func (lh, Omega_b0h2, 0.022, 0.002);
+    NcmMSetFunc *Omega_b0h2 = NCM_MSET_FUNC (ncm_mset_func_list_new ("NcHICosmo:Omega_b0h2", NULL));
+    ncm_likelihood_priors_add_gauss_func (lh, Omega_b0h2, 0.022, 0.002, 0.0);
     ncm_mset_func_free (Omega_b0h2);
   }
 
-  if (de_fit.qspline_cp && TRUE)
+  if (de_fit.qspline_cp)
   {
-    if (!NC_IS_HICOSMO_QSPLINE (model))
+    if (!NC_IS_HICOSMO_QSPLINE (cosmo))
       g_error ("Continuity priors are only valid for NcHICosmoQSPline model");
     else
     {
       NcHICosmoQSplineContPrior *qspline_cp = 
-        nc_hicosmo_qspline_add_continuity_priors (NC_HICOSMO_QSPLINE (model), lh, 1.0e-10, de_fit.qspline_cp_sigma);
+        nc_hicosmo_qspline_add_continuity_priors (NC_HICOSMO_QSPLINE (cosmo), lh, 1.0e-10, de_fit.qspline_cp_sigma);
+
+      ncm_mset_set (mset, NCM_MODEL (qspline_cp));
       nc_hicosmo_qspline_cont_prior_free (qspline_cp);
     }
   }
 
+  if (de_data_simple.PlanckPriors)
+  {
+    NcPlanckFI *planck_fi = NC_PLANCK_FI (ncm_mset_peek (mset, nc_planck_fi_id ()));
+    if (planck_fi == NULL)
+      g_warning ("Planck Priors enabled but not NcPlanckFI model set.");
+    else
+    {
+      if (NC_IS_PLANCK_FI_COR_TT (planck_fi))
+      {
+        nc_planck_fi_cor_tt_add_all_default_priors (lh);
+      }
+    }
+  }
+
   if (de_fit.fiducial != NULL)
     fiduc = ncm_mset_load (de_fit.fiducial, ser);
   else
     fiduc = ncm_mset_ref (mset);
 
+  if (de_fit.fit_type == NULL)
+  {
+#ifdef NUMCOSMO_HAVE_NLOPT
+    de_fit.fit_type = g_strdup ("nlopt");
+#else
+    de_fit.fit_type = g_strdup ("gsl-mms");
+#endif
+  }
+  if (de_fit.fit_diff == NULL)
+    de_fit.fit_diff = g_strdup ("numdiff-forward");
+  
   {
     const GEnumValue *fit_type_id = ncm_cfg_get_enum_by_id_name_nick (NCM_TYPE_FIT_TYPE, de_fit.fit_type);
     const GEnumValue *fit_diff_id = ncm_cfg_get_enum_by_id_name_nick (NCM_TYPE_FIT_GRAD_TYPE, de_fit.fit_diff);
+
     if (fit_type_id == NULL)
       g_error ("Fit type '%s' not found run --fit-list to list the available options", de_fit.fit_type);
+
     if (fit_diff_id == NULL)
       g_error ("Fit type '%s' not found run --fit-list to list the available options", de_fit.fit_diff);
 
@@ -457,18 +554,6 @@ main (gint argc, gchar *argv[])
     ncm_fit_set_params_reltol (fit, de_fit.fit_params_reltol);
   }
 
-  if (de_fit.qspline_cp && FALSE)
-  {
-    if (!NC_IS_HICOSMO_QSPLINE (model))
-      g_error ("Continuity priors are only valid for NcHICosmoQSPline model");
-    else
-    {
-      NcHICosmoQSplineContPrior *qspline_cp = 
-        nc_hicosmo_qspline_add_continuity_constraints (NC_HICOSMO_QSPLINE (model), fit, de_fit.qspline_cp_sigma);
-      nc_hicosmo_qspline_cont_prior_free (qspline_cp);
-    }
-  }
-
   if (de_data_simple.priors_gauss != NULL)
   {
     guint i;
@@ -511,7 +596,7 @@ main (gint argc, gchar *argv[])
 
         g_assert_cmpfloat (sigma, >, 0.0);
 
-        ncm_prior_add_gaussian_data (lh, p_i.mid, p_i.pid, mu, sigma);
+        ncm_likelihood_priors_add_gauss_param (lh, p_i.mid, p_i.pid, mu, sigma);
         g_variant_unref (prior_hash);
         g_free (model_ns);
         g_free (p_name);
@@ -528,8 +613,8 @@ main (gint argc, gchar *argv[])
     ncm_dataset_resample (dset, fiduc, rng);
   }
 
-  de_fit.fisher = (de_fit.fisher || (de_fit.nsigma_fisher != -1) || (de_fit.nsigma != -1) || (de_fit.onedim_cr != NULL));
-  de_fit.fit = (de_fit.fit || de_fit.fisher);
+  de_fit.fisher        = (de_fit.fisher || (de_fit.nsigma_fisher != -1) || (de_fit.nsigma != -1) || (de_fit.onedim_cr != NULL));
+  de_fit.fit           = (de_fit.fit || de_fit.fisher);
   de_fit.save_best_fit = (de_fit.save_best_fit || de_fit.save_fisher);
 
   if (de_fit.fit)
@@ -545,7 +630,7 @@ main (gint argc, gchar *argv[])
     FILE *f_bf;
     gchar *bfile = NULL;
 
-    f_bf = nc_de_open_dataout_file (model, "best_fit", &bfile);
+    f_bf = nc_de_open_dataout_file (cosmo, "best_fit", &bfile);
     ncm_mset_params_pretty_print (fit->mset, f_bf, full_cmd_line);
     fclose (f_bf);
 
@@ -564,7 +649,7 @@ main (gint argc, gchar *argv[])
       FILE *f_MF;
       gchar *mfile = NULL;
 
-      f_MF = nc_de_open_dataout_file (model, "MF", &mfile);
+      f_MF = nc_de_open_dataout_file (cosmo, "MF", &mfile);
       ncm_fit_fishermatrix_print (fit, f_MF, full_cmd_line);
       fclose (f_MF);
 
@@ -576,6 +661,51 @@ main (gint argc, gchar *argv[])
     }
   }
 
+  if (de_fit.funcs != NULL)
+  {
+    const guint len = g_strv_length (de_fit.funcs);
+    guint i;
+
+    funcs_oa = ncm_obj_array_new ();
+
+    for (i = 0; i < len; i++)
+    {
+      NcmMSetFunc *func = NULL;
+      gdouble *x = NULL;
+      gchar *func_name;
+      guint len;
+
+      func_name = ncm_util_function_params (de_fit.funcs[i], &x, &len);
+      if (func_name == NULL)
+        g_error ("darkenergy: invalid function name: `%s'.", de_fit.funcs[i]);
+
+      if (ncm_mset_func_list_has_ns_name ("NcHICosmo", func_name))
+      {
+        func = NCM_MSET_FUNC (ncm_mset_func_list_new_ns_name ("NcHICosmo", func_name, NULL));
+      }
+      else if (ncm_mset_func_list_has_ns_name ("NcDistance", func_name))
+      {
+        func = NCM_MSET_FUNC (ncm_mset_func_list_new_ns_name ("NcDistance", func_name, G_OBJECT (dist)));
+      }
+      else
+      {
+        g_warning ("darkenergy: function `%s' not found, skipping!", de_fit.funcs[i]);
+        func = NULL;
+        continue;
+      }
+
+      if (len > 0)
+        ncm_mset_func_set_eval_x (func, x, len);
+
+      g_clear_pointer (&x, g_free);
+
+      g_assert (ncm_mset_func_is_scalar (func));
+      g_assert (ncm_mset_func_is_const (func));
+
+      ncm_obj_array_add (funcs_oa, G_OBJECT (func));
+    }
+  }
+  
   if (de_fit.mc)
   {
     NcmFitMC *mc = ncm_fit_mc_new (fit, de_fit.mc_rtype, de_fit.msg_level);
@@ -656,7 +786,7 @@ main (gint argc, gchar *argv[])
     mcat = ncm_fit_mcbs_get_catalog (mcbs);
     ncm_fit_mcbs_clear (&mcbs);
   }
-
+  
   if (de_fit.mcmc)
   {
     NcmMSetTransKernGauss *mcsg = ncm_mset_trans_kern_gauss_new (0);
@@ -671,7 +801,7 @@ main (gint argc, gchar *argv[])
     }
     else
     {
-      ncm_mset_trans_kern_gauss_set_cov_from_scale (mcsg);
+      ncm_mset_trans_kern_gauss_set_cov_from_rescale (mcsg, 0.1);
     }
     
     if (de_fit.mc_seed > -1)
@@ -700,11 +830,39 @@ main (gint argc, gchar *argv[])
   if (de_fit.esmcmc)
   {
     NcmMSetTransKernGauss *init_sampler = ncm_mset_trans_kern_gauss_new (0);
-    NcmFitESMCMC *esmcmc = ncm_fit_esmcmc_new (fit, 
+    NcmFitESMCMC *esmcmc;
+
+    if (de_fit.esmcmc_walk)
+    {
+      NcmFitESMCMCWalkerWalk *walk = ncm_fit_esmcmc_walker_walk_new (de_fit.mc_nwalkers);
+      
+      esmcmc = ncm_fit_esmcmc_new_funcs_array (fit, 
                                                de_fit.mc_nwalkers, 
                                                NCM_MSET_TRANS_KERN (init_sampler), 
-                                               NCM_FIT_ESMCMC_MOVE_TYPE_STRETCH, 
-                                               de_fit.msg_level);
+                                               NCM_FIT_ESMCMC_WALKER (walk),
+                                               de_fit.msg_level,
+                                               funcs_oa);
+      
+      ncm_fit_esmcmc_walker_free (NCM_FIT_ESMCMC_WALKER (walk));
+    }
+    else
+    {
+      NcmFitESMCMCWalkerStretch *stretch = ncm_fit_esmcmc_walker_stretch_new (de_fit.mc_nwalkers, ncm_mset_fparams_len (mset));
+      esmcmc = ncm_fit_esmcmc_new_funcs_array (fit, 
+                                               de_fit.mc_nwalkers, 
+                                               NCM_MSET_TRANS_KERN (init_sampler), 
+                                               NCM_FIT_ESMCMC_WALKER (stretch),
+                                               de_fit.msg_level,
+                                               funcs_oa);
+      
+      if (de_fit.esmcmc_sbox)
+      {
+        ncm_fit_esmcmc_walker_stretch_set_box_mset (stretch, mset);
+      }
+      ncm_fit_esmcmc_walker_stretch_multi (stretch, de_fit.esmcmc_ms);
+
+      ncm_fit_esmcmc_walker_free (NCM_FIT_ESMCMC_WALKER (stretch));
+    }
 
     ncm_mset_trans_kern_set_mset (NCM_MSET_TRANS_KERN (init_sampler), mset);
     ncm_mset_trans_kern_set_prior_from_mset (NCM_MSET_TRANS_KERN (init_sampler));
@@ -721,7 +879,7 @@ main (gint argc, gchar *argv[])
     }
     else
     {
-      ncm_mset_trans_kern_gauss_set_cov_from_scale (init_sampler);
+      ncm_mset_trans_kern_gauss_set_cov_from_rescale (init_sampler, 0.01);
     }
 
     if (de_fit.mc_seed > -1)
@@ -745,7 +903,8 @@ main (gint argc, gchar *argv[])
     ncm_mset_catalog_clear (&mcat);
     mcat = ncm_fit_esmcmc_get_catalog (esmcmc);
 
-    ncm_fit_esmcmc_clear (&esmcmc);    
+    ncm_fit_esmcmc_clear (&esmcmc);
+    ncm_mset_trans_kern_free (NCM_MSET_TRANS_KERN (init_sampler));
   }
 
   if (de_fit.onedim_cr != NULL)
@@ -832,7 +991,7 @@ main (gint argc, gchar *argv[])
       FILE *f_PL;
       gchar *pfile = NULL;
 
-      f_PL = nc_de_open_dataout_file (model, "PL", &pfile);
+      f_PL = nc_de_open_dataout_file (cosmo, "PL", &pfile);
 
       fprintf (f_PL, "# %s\n", full_cmd_line);
       if (rg_1sigma != NULL)
@@ -901,7 +1060,7 @@ main (gint argc, gchar *argv[])
       FILE *f_MFcr;
       gchar *mcrfile = NULL;
 
-      f_MFcr = nc_de_open_dataout_file (model, "MF_cr", &mcrfile);
+      f_MFcr = nc_de_open_dataout_file (cosmo, "MF_cr", &mcrfile);
 
       fprintf (f_MFcr, "# %s\n", full_cmd_line);
       if (rg_1sigma != NULL)
@@ -938,11 +1097,11 @@ main (gint argc, gchar *argv[])
     FILE *f_mf;
     gchar *mfile = NULL;
 
-    f_mf = nc_de_open_dataout_file (model, "MassFunction", &mfile);
+    f_mf = nc_de_open_dataout_file (cosmo, "MassFunction", &mfile);
     for (i = 0; i < ca_array->len; i++)
     {
       NcDataClusterNCount *dca_unbinned = g_ptr_array_index (ca_array, i);
-      nc_data_cluster_ncount_print (dca_unbinned, model, f_mf, full_cmd_line);
+      nc_data_cluster_ncount_print (dca_unbinned, cosmo, f_mf, full_cmd_line);
       fprintf (f_mf, "\n\n");
     }
     fclose (f_mf);
@@ -973,7 +1132,7 @@ main (gint argc, gchar *argv[])
   ncm_serialize_clear (&ser);
   ncm_mset_catalog_clear (&mcat);
   ncm_serialize_global_reset ();
-  ncm_model_free (NCM_MODEL (model));
+  ncm_model_free (NCM_MODEL (cosmo));
   ncm_mset_free (mset);
   ncm_fit_free (fit);
   ncm_likelihood_free (lh);
@@ -982,7 +1141,7 @@ main (gint argc, gchar *argv[])
   nc_distance_free (dist);
   g_free (full_cmd_line);
 
-  if (FALSE) /* Several unimportant leaks */
+  if (TRUE)
   {
     g_clear_pointer (&de_model.model_name, g_free);
 
@@ -997,8 +1156,8 @@ main (gint argc, gchar *argv[])
     g_clear_pointer (&de_data_simple.priors_gauss, g_strfreev);
     g_clear_pointer (&de_data_simple.data_files,   g_strfreev);
 
-    g_clear_pointer (&de_data_cluster.window_name,       g_free);
-    g_clear_pointer (&de_data_cluster.transfer_name,     g_free);
+    g_clear_pointer (&de_data_cluster.filter_type,       g_free);
+    g_clear_pointer (&de_data_cluster.ps_type,           g_free);
     g_clear_pointer (&de_data_cluster.multiplicity_name, g_free);
     g_clear_pointer (&de_data_cluster.clusterm_ser,      g_free);
     g_clear_pointer (&de_data_cluster.clusterz_ser,      g_free);
@@ -1017,6 +1176,7 @@ main (gint argc, gchar *argv[])
     g_clear_pointer (&de_fit.save_mset,   g_free);
     
     g_clear_pointer (&de_fit.onedim_cr, g_strfreev);
+    g_clear_pointer (&de_fit.funcs, g_strfreev);
   }
   
   return 0;
