@@ -78,6 +78,7 @@ main (gint argc, gchar *argv[])
 {
   gchar *cat_filename = NULL;
   gboolean info           = FALSE;
+  gboolean chain_evol     = FALSE;
   gboolean list_all       = FALSE;
   gboolean list_hicosmo   = FALSE;
   gboolean list_hicosmo_z = FALSE;
@@ -91,6 +92,7 @@ main (gint argc, gchar *argv[])
   gchar **funcs = NULL;
   gchar **distribs = NULL;
   gchar **params = NULL;
+  gchar **params_evol = NULL;
   gchar **mode_errors = NULL;
   gchar **median_errors = NULL;
   gchar **bestfit_errors = NULL;
@@ -102,6 +104,7 @@ main (gint argc, gchar *argv[])
   {
     { "catalog",        'c', 0, G_OPTION_ARG_FILENAME,     &cat_filename,   "Catalog filename.", NULL },
     { "info",           'i', 0, G_OPTION_ARG_NONE,         &info,           "Print catalog information.", NULL },
+    { "chain-evol",     'I', 0, G_OPTION_ARG_NONE,         &chain_evol,     "Print chain evolution.", NULL },
     { "list",           'l', 0, G_OPTION_ARG_NONE,         &list_all,       "Print all available functions.", NULL },
     { "list-hicosmo",     0, 0, G_OPTION_ARG_NONE,         &list_hicosmo,   "Print available constant functions from NcHICosmo.", NULL },
     { "list-hicosmo-z",   0, 0, G_OPTION_ARG_NONE,         &list_hicosmo_z, "Print available redshift functions from NcHICosmo.", NULL },
@@ -115,6 +118,7 @@ main (gint argc, gchar *argv[])
     { "function",       'f', 0, G_OPTION_ARG_STRING_ARRAY, &funcs,          "Redshift functions to be analyzed.", NULL},
     { "distribution",   'd', 0, G_OPTION_ARG_STRING_ARRAY, &distribs,       "Function distributions to be analyzed.", NULL},
     { "parameter",      'p', 0, G_OPTION_ARG_STRING_ARRAY, &params,         "Model parameters' to be analyzed.", NULL},
+    { "parameter-evol", 'P', 0, G_OPTION_ARG_STRING_ARRAY, &params_evol,    "Calculate the time evolution of the parameter.", NULL},
     { "mode-error",     'o', 0, G_OPTION_ARG_STRING_ARRAY, &mode_errors,    "Print mode and 1-3 sigma asymmetric error bars of the model parameters' to be analyzed.", NULL},
     { "median-error",   'e', 0, G_OPTION_ARG_STRING_ARRAY, &median_errors,  "Print median and 1-3 sigma asymmetric error bars of the model parameters' to be analyzed.", NULL},
     { "bestfit-error",  's', 0, G_OPTION_ARG_STRING_ARRAY, &bestfit_errors,  "Print best fit and 1-3 sigma asymmetric error bars of the model parameters' to be analyzed.", NULL},
@@ -137,49 +141,49 @@ main (gint argc, gchar *argv[])
 
   if (list_hicosmo || list_all)
   {
-    GArray *func_table = nc_hicosmo_class_func_table ();
+    GArray *func_table = ncm_mset_func_list_select ("NcHICosmo", 0, 1); 
     ncm_cfg_msg_sepa ();
     g_message ("# Available constant functions from NcHICosmo models:\n");    
     for (i = 0; i < func_table->len; i++)
     {
-      NcHICosmoFunc *f = &g_array_index (func_table, NcHICosmoFunc, i);
-      g_message ("# - %32s: %s\n", f->name, f->desc);
+      NcmMSetFuncListStruct *fdata = &g_array_index (func_table, NcmMSetFuncListStruct, i);
+      g_message ("# - %32s: %s\n", fdata->name, fdata->desc);
     }
     g_array_unref (func_table);
   }
   if (list_hicosmo_z || list_all)
   {
-    GArray *func_z_table = nc_hicosmo_class_func_z_table ();
+    GArray *func_z_table = ncm_mset_func_list_select ("NcHICosmo", 1, 1); 
     ncm_cfg_msg_sepa ();
     g_message ("# Available redshift functions from NcHICosmo models:\n");    
     for (i = 0; i < func_z_table->len; i++)
     {
-      NcHICosmoFuncZ *fz = &g_array_index (func_z_table, NcHICosmoFuncZ, i);
-      g_message ("# - %32s: %s\n", fz->name, fz->desc);
+      NcmMSetFuncListStruct *fdata = &g_array_index (func_z_table, NcmMSetFuncListStruct, i);
+      g_message ("# - %32s: %s\n", fdata->name, fdata->desc);
     }
     g_array_unref (func_z_table);
   }
   if (list_dist || list_all)
   {
-    GArray *func_table = nc_distance_class_func_table ();
+    GArray *func_table = ncm_mset_func_list_select ("NcDistance", 0, 1); 
     ncm_cfg_msg_sepa ();
     g_message ("# Available constant functions from NcDistance:\n");    
     for (i = 0; i < func_table->len; i++)
     {
-      NcDistanceFunc *f = &g_array_index (func_table, NcDistanceFunc, i);
-      g_message ("# - %32s: %s\n", f->name, f->desc);
+      NcmMSetFuncListStruct *fdata = &g_array_index (func_table, NcmMSetFuncListStruct, i);
+      g_message ("# - %32s: %s\n", fdata->name, fdata->desc);
     }
     g_array_unref (func_table);
   }
   if (list_dist_z || list_all)
   {
-    GArray *func_z_table = nc_distance_class_func_z_table ();
+    GArray *func_z_table = ncm_mset_func_list_select ("NcDistance", 1, 1);
     ncm_cfg_msg_sepa ();
     g_message ("# Available redshift functions from NcDistance:\n");    
     for (i = 0; i < func_z_table->len; i++)
     {
-      NcDistanceFuncZ *fz = &g_array_index (func_z_table, NcDistanceFuncZ, i);
-      g_message ("# - %32s: %s\n", fz->name, fz->desc);
+      NcmMSetFuncListStruct *fdata = &g_array_index (func_z_table, NcmMSetFuncListStruct, i);
+      g_message ("# - %32s: %s\n", fdata->name, fdata->desc);
     }
     g_array_unref (func_z_table);
   }
@@ -198,12 +202,13 @@ main (gint argc, gchar *argv[])
   }
   else
   {
-    NcmMSetCatalog *mcat = ncm_mset_catalog_new_from_file (cat_filename);
+    NcmMSetCatalog *mcat = ncm_mset_catalog_new_from_file_ro (cat_filename, burnin);
     NcmMSet *mset = ncm_mset_catalog_get_mset (mcat);
+    ncm_mset_catalog_estimate_autocorrelation_tau (mcat);
     
     if (info)
     {
-      NcmMatrix *cov = NULL;
+      /*NcmMatrix *cov = NULL;*/
       const gchar *rtype_str = ncm_mset_catalog_get_run_type (mcat);
       ncm_cfg_msg_sepa ();
       g_message ("# Catalog run type: `%s'.\n", rtype_str);
@@ -215,12 +220,40 @@ main (gint argc, gchar *argv[])
       g_message ("#\n");
       
       ncm_mset_pretty_log (mset);
-      ncm_mset_catalog_get_covar (mcat, &cov);
-      ncm_mset_fparams_log_covar (mset, cov);
-      ncm_matrix_clear (&cov);
+      /*ncm_mset_catalog_get_covar (mcat, &cov);*/
+      /*ncm_mset_fparams_log_covar (mset, cov);*/
+      ncm_mset_catalog_log_full_covar (mcat);
+      /*ncm_matrix_clear (&cov);*/
       ncm_mset_catalog_log_current_stats (mcat);
-
     }
+
+    if (chain_evol)
+    {
+      guint last_t = ncm_mset_catalog_max_time (mcat);
+      const gdouble nchains = mcat->nchains;
+      guint i;
+      
+      ncm_message ("# Chain evolution from 0 to %u\n", last_t);
+      for (i = 0; i < last_t; i++)
+      {
+        NcmVector *e_mean = ncm_mset_catalog_peek_e_mean_t (mcat, i);
+        NcmVector *e_var  = ncm_mset_catalog_peek_e_var_t (mcat, i);
+        const guint len = ncm_vector_len (e_mean);
+        guint j;
+        
+        ncm_message ("% 10u", i);
+        for (j = 0; j < len; j++)
+        {
+          const gdouble mean_j = ncm_stats_vec_get_mean (mcat->pstats, j);
+          const gdouble var_j  = ncm_stats_vec_get_var (mcat->pstats, j);
+          const gdouble sd_j   = sqrt (var_j / nchains);
+          
+          ncm_message (" % 20.15g % 20.15g", (ncm_vector_get (e_mean, j) - mean_j) / sd_j, sqrt (ncm_vector_get (e_var, j) / nchains) / sd_j);
+        }
+        ncm_message ("\n");
+      }
+    }
+    
     /*********************************************************************************************************
      * 
      * Functions
@@ -252,34 +285,32 @@ main (gint argc, gchar *argv[])
 
         if (mset_func == NULL)
         {
-          NcHICosmoFuncZ *fz = nc_hicosmo_class_get_func_z (funcs[i]);
-          if (fz != NULL)
+          if (ncm_mset_func_list_has_ns_name ("NcHICosmo", funcs[i]))
           {
-            if (!ncm_model_check_impl (NCM_MODEL (ncm_mset_peek (mset, nc_hicosmo_id ())), fz->impl))
+            mset_func = NCM_MSET_FUNC (ncm_mset_func_list_new_ns_name ("NcHICosmo", funcs[i], NULL));
+            if (ncm_mset_func_get_dim (mset_func) != 1 || ncm_mset_func_get_nvar (mset_func) != 1)
             {
-              g_warning ("# function `%s' not supported by NcHICosmo `%s', skipping...", funcs[i],
-                         ncm_model_name (NCM_MODEL (ncm_mset_peek (mset, nc_hicosmo_id ()))));
+              g_warning ("# Function `%s' is not R => R, skipping.", ncm_mset_func_peek_name (mset_func));
+              ncm_mset_func_clear (&mset_func);
               continue;
             }
-            mset_func = nc_hicosmo_create_mset_arrayfunc1 (fz->f, nsteps);
             ncm_cfg_msg_sepa ();
-            g_message ("# Printing NcHICosmo z function: `%s' in [% 20.15g % 20.15g].\n", fz->desc, zi, zf);
+            g_message ("# Printing NcHICosmo z function: `%s' in [% 20.15g % 20.15g].\n", ncm_mset_func_peek_desc (mset_func), zi, zf);
           }
         }
         if (mset_func == NULL)
         {
-          NcDistanceFuncZ *fz = nc_distance_class_get_func_z (funcs[i]);
-          if (fz != NULL)
+          if (ncm_mset_func_list_has_ns_name ("NcDistance", funcs[i]))
           {
-            if (!ncm_model_check_impl (NCM_MODEL (ncm_mset_peek (mset, nc_hicosmo_id ())), fz->impl))
+            mset_func = NCM_MSET_FUNC (ncm_mset_func_list_new_ns_name ("NcDistance", funcs[i], G_OBJECT (dist)));
+            if (ncm_mset_func_get_dim (mset_func) != 1 || ncm_mset_func_get_nvar (mset_func) != 1)
             {
-              g_warning ("# function `%s' not supported by NcHICosmo `%s', skipping...", funcs[i],
-                         ncm_model_name (NCM_MODEL (ncm_mset_peek (mset, nc_hicosmo_id ()))));
+              g_warning ("# Function `%s' is not R => R, skipping.", ncm_mset_func_peek_name (mset_func));
+              ncm_mset_func_clear (&mset_func);
               continue;
             }
-            mset_func = nc_distance_create_mset_arrayfunc1 (dist, fz->f, nsteps);
             ncm_cfg_msg_sepa ();
-            g_message ("# Printing NcDistance z function: `%s' in [% 20.15g % 20.15g].\n", fz->desc, zi, zf);
+            g_message ("# Printing NcDistance z function: `%s' in [% 20.15g % 20.15g].\n", ncm_mset_func_peek_desc (mset_func), zi, zf);
           }
         }
         if (mset_func == NULL)
@@ -289,9 +320,9 @@ main (gint argc, gchar *argv[])
         }
 
         if (use_direct)
-          res = ncm_mset_catalog_calc_ci_direct (mcat, burnin, mset_func, ncm_vector_ptr (z_vec, 0), p_val);
+          res = ncm_mset_catalog_calc_ci_direct (mcat, mset_func, z_vec, p_val);
         else
-          res = ncm_mset_catalog_calc_ci_interp (mcat, burnin, mset_func, ncm_vector_ptr (z_vec, 0), p_val, 100, NCM_FIT_RUN_MSGS_SIMPLE);
+          res = ncm_mset_catalog_calc_ci_interp (mcat, mset_func, z_vec, p_val, 100, NCM_FIT_RUN_MSGS_SIMPLE);
 
         for (k = 0; k < nsteps; k++)
         {
@@ -328,34 +359,32 @@ main (gint argc, gchar *argv[])
 
         if (mset_func == NULL)
         {
-          NcHICosmoFunc *f = nc_hicosmo_class_get_func (distribs[i]);
-          if (f != NULL)
+          if (ncm_mset_func_list_has_ns_name ("NcHICosmo", distribs[i]))
           {
-            if (!ncm_model_check_impl (NCM_MODEL (ncm_mset_peek (mset, nc_hicosmo_id ())), f->impl))
+            mset_func = NCM_MSET_FUNC (ncm_mset_func_list_new_ns_name ("NcHICosmo", distribs[i], NULL));
+            if (ncm_mset_func_get_dim (mset_func) != 1 || ncm_mset_func_get_nvar (mset_func) != 0)
             {
-              g_warning ("# function `%s' not supported by NcHICosmo `%s', skipping...", distribs[i],
-                         ncm_model_name (NCM_MODEL (ncm_mset_peek (mset, nc_hicosmo_id ()))));
+              g_warning ("# Function `%s' is not constant, skipping.", ncm_mset_func_peek_name (mset_func));
+              ncm_mset_func_clear (&mset_func);
               continue;
             }
-            mset_func = nc_hicosmo_create_mset_func0 (f->f);
             ncm_cfg_msg_sepa ();
-            g_message ("# Printing NcHICosmo function: `%s'.\n", f->desc);
+            g_message ("# Printing NcHICosmo function: `%s'.\n", ncm_mset_func_peek_desc (mset_func));
           }
         }
         if (mset_func == NULL)
         {
-          NcDistanceFunc *f = nc_distance_class_get_func (distribs[i]);
-          if (f != NULL)
+          if (ncm_mset_func_list_has_ns_name ("NcDistance", distribs[i]))
           {
-            if (!ncm_model_check_impl (NCM_MODEL (ncm_mset_peek (mset, nc_hicosmo_id ())), f->impl))
+            mset_func = NCM_MSET_FUNC (ncm_mset_func_list_new_ns_name ("NcDistance", distribs[i], G_OBJECT (dist)));
+            if (ncm_mset_func_get_dim (mset_func) != 1 || ncm_mset_func_get_nvar (mset_func) != 0)
             {
-              g_warning ("# function `%s' not supported by NcHICosmo `%s', skipping...", distribs[i],
-                         ncm_model_name (NCM_MODEL (ncm_mset_peek (mset, nc_hicosmo_id ()))));
+              g_warning ("# Function `%s' is not constant, skipping.", ncm_mset_func_peek_name (mset_func));
+              ncm_mset_func_clear (&mset_func);
               continue;
             }
-            mset_func = nc_distance_func0_new (dist, f->f);
             ncm_cfg_msg_sepa ();
-            g_message ("# Printing NcDistance function: `%s'.\n", f->desc);
+            g_message ("# Printing NcDistance function: `%s'.\n", ncm_mset_func_peek_desc (mset_func));
           }
         }
         if (mset_func == NULL)
@@ -365,7 +394,7 @@ main (gint argc, gchar *argv[])
         }
 
         {
-          NcmStatsDist1d *sd1 = ncm_mset_catalog_calc_distrib (mcat, burnin, mset_func, NCM_FIT_RUN_MSGS_SIMPLE);
+          NcmStatsDist1d *sd1 = ncm_mset_catalog_calc_distrib (mcat, mset_func, NCM_FIT_RUN_MSGS_SIMPLE);
           for (k = 0; k < nsteps; k++)
           {
             gdouble x = sd1->xi + (sd1->xf - sd1->xi) / (nsteps - 1.0) * k;
@@ -386,6 +415,7 @@ main (gint argc, gchar *argv[])
       }
       nc_distance_clear (&dist);
     }
+
     /*********************************************************************************************************
      * 
      * Parameters
@@ -411,9 +441,9 @@ main (gint argc, gchar *argv[])
         {
           NcmStatsDist1d *sd1;
           if (pi != NULL)
-            sd1 = ncm_mset_catalog_calc_param_distrib (mcat, burnin, pi, NCM_FIT_RUN_MSGS_SIMPLE);
+            sd1 = ncm_mset_catalog_calc_param_distrib (mcat, pi, NCM_FIT_RUN_MSGS_SIMPLE);
           else
-            sd1 = ncm_mset_catalog_calc_add_param_distrib (mcat, burnin, add_param, NCM_FIT_RUN_MSGS_SIMPLE);
+            sd1 = ncm_mset_catalog_calc_add_param_distrib (mcat, add_param, NCM_FIT_RUN_MSGS_SIMPLE);
           
           for (k = 0; k < nsteps; k++)
           {
@@ -433,6 +463,55 @@ main (gint argc, gchar *argv[])
       }
     }
     
+    /*********************************************************************************************************
+     * 
+     * Parameters evolution
+     * 
+     *********************************************************************************************************/
+    if (params_evol != NULL)
+    {
+      guint nparams = g_strv_length (params_evol);
+
+      for (i = 0; i < nparams; i++)
+      {
+        const NcmMSetPIndex *pi = ncm_mset_fparam_get_pi_by_name (mset, params_evol[i]);
+        gchar *end_ptr = NULL;
+        glong add_param = strtol (params_evol[i], &end_ptr, 10);
+        guint t, k;
+
+        if (pi == NULL && (params_evol[i] == end_ptr))
+        {
+          g_warning ("# Parameter `%s' not found, skipping...\n", params_evol[i]);
+          continue;
+        }
+
+        {
+          NcmVector *pv;
+          NcmMatrix *evol_t;
+          gchar *out_file = g_strdup_printf ("param_evol_%s.dat", params_evol[i]);
+          FILE *out = fopen (out_file, "w");
+          
+          if (pi != NULL)
+            ncm_mset_catalog_calc_param_ensemble_evol (mcat, pi, nsteps, NCM_FIT_RUN_MSGS_SIMPLE, &pv, &evol_t);
+          else
+            ncm_mset_catalog_calc_add_param_ensemble_evol (mcat, add_param, nsteps, NCM_FIT_RUN_MSGS_SIMPLE, &pv, &evol_t); 
+          
+          for (t = 0; t < ncm_matrix_nrows (evol_t); t++)
+          {
+            for (k = 0; k < ncm_matrix_ncols (evol_t); k++)
+            {
+              fprintf (out, "%u % 20.15g % 20.15g\n", t, ncm_vector_get (pv, k), ncm_matrix_get (evol_t, t, k));
+            }
+            fprintf (out, "\n\n");
+          }
+
+          fclose (out);
+          ncm_vector_free (pv);
+          ncm_matrix_free (evol_t);
+        }
+      }
+    }
+    
    /*********************************************************************************************************
      * 
      * Parameters - Mode and error bars
@@ -453,7 +532,7 @@ main (gint argc, gchar *argv[])
         glong add_param = strtol (mode_errors[i], &end_ptr, 10);
         ncm_message ("# Parameter `%s'| mode | 1l 1u | 2l 2u | 3l 3u\n", mode_errors[i]);
 
-        if (pi == NULL && (params[i] == end_ptr))
+        if (pi == NULL && (mode_errors[i] == end_ptr))
         {
           g_warning ("# Parameter `%s' not found, skipping...\n", mode_errors[i]);
           continue;
@@ -462,9 +541,9 @@ main (gint argc, gchar *argv[])
         {
           NcmStatsDist1d *sd1;
           if (pi != NULL)
-            sd1 = ncm_mset_catalog_calc_param_distrib (mcat, burnin, pi, NCM_FIT_RUN_MSGS_SIMPLE);
+            sd1 = ncm_mset_catalog_calc_param_distrib (mcat, pi, NCM_FIT_RUN_MSGS_SIMPLE);
           else
-            sd1 = ncm_mset_catalog_calc_add_param_distrib (mcat, burnin, add_param, NCM_FIT_RUN_MSGS_SIMPLE);
+            sd1 = ncm_mset_catalog_calc_add_param_distrib (mcat, add_param, NCM_FIT_RUN_MSGS_SIMPLE);
 
           {
             gdouble mode = ncm_stats_dist1d_eval_mode (sd1);
@@ -507,7 +586,7 @@ main (gint argc, gchar *argv[])
         glong add_param = strtol (median_errors[i], &end_ptr, 10);
         ncm_message ("# Parameter `%s' | 1l 1u | 2l 2u | 3l 3u\n", median_errors[i]);
 
-        if (pi == NULL && (params[i] == end_ptr))
+        if (pi == NULL && (median_errors[i] == end_ptr))
         {
           g_warning ("# Parameter `%s' not found, skipping...\n", median_errors[i]);
           continue;
@@ -516,9 +595,9 @@ main (gint argc, gchar *argv[])
         {
           NcmStatsDist1d *sd1;
           if (pi != NULL)
-            sd1 = ncm_mset_catalog_calc_param_distrib (mcat, burnin, pi, NCM_FIT_RUN_MSGS_SIMPLE);
+            sd1 = ncm_mset_catalog_calc_param_distrib (mcat, pi, NCM_FIT_RUN_MSGS_SIMPLE);
           else
-            sd1 = ncm_mset_catalog_calc_add_param_distrib (mcat, burnin, add_param, NCM_FIT_RUN_MSGS_SIMPLE);
+            sd1 = ncm_mset_catalog_calc_add_param_distrib (mcat, add_param, NCM_FIT_RUN_MSGS_SIMPLE);
 
           {
               const gdouble median   = ncm_stats_dist1d_eval_inv_pdf (sd1, 0.5);
@@ -585,7 +664,7 @@ main (gint argc, gchar *argv[])
             glong add_param = strtol (name, &end_ptr, 10);
             ncm_message ("# Parameter `%s'| best fit | 1l 1u | 2l 2u | 3l 3u\n", name);
 
-            if (pi == NULL && (params[i] == end_ptr))
+            if (pi == NULL && (bestfit_errors[i] == end_ptr))
             {
               g_warning ("# Parameter `%s' not found, skipping...\n", name);
               continue;
@@ -594,9 +673,9 @@ main (gint argc, gchar *argv[])
             {
               NcmStatsDist1d *sd1;
               if (pi != NULL)
-                sd1 = ncm_mset_catalog_calc_param_distrib (mcat, burnin, pi, NCM_FIT_RUN_MSGS_SIMPLE);
+                sd1 = ncm_mset_catalog_calc_param_distrib (mcat, pi, NCM_FIT_RUN_MSGS_SIMPLE);
               else
-                sd1 = ncm_mset_catalog_calc_add_param_distrib (mcat, burnin, add_param, NCM_FIT_RUN_MSGS_SIMPLE);
+                sd1 = ncm_mset_catalog_calc_add_param_distrib (mcat, add_param, NCM_FIT_RUN_MSGS_SIMPLE);
 
               {
                   gdouble x_l1 = 0.0, x_u1 = 0.0;
